searchNodes=[{"ref":"Money.html","title":"Money","type":"module","doc":"Money implements a set of functions to store, retrieve, convert and perform arithmetic on a Money.t type that is composed of a currency code and a decimal currency amount. Money is very opinionated in the interests of serving as a dependable library that can underpin accounting and financial applications. This opinion expressed by ensuring that: Money must always have both a amount and a currency code. The currency code must always be valid. Money arithmetic can only be performed when both operands are of the same currency. Money amounts are represented as a Decimal. Money is serialised to the database as a custom Postgres composite type that includes both the amount and the currency. Therefore for Ecto serialization Postgres is assumed as the data store. Serialization is entirely optional and Ecto is not a package dependency. All arithmetic functions work in fixed point decimal. No rounding occurs automatically (unless expressly called out for a function). Explicit rounding obeys the rounding rules for a given currency. The rounding rules are defined by the Unicode consortium in its CLDR repository as implemented by the hex package ex_cldr. These rules define the number of fractional digits for a currency and the rounding increment where appropriate."},{"ref":"Money.html#abs/1","title":"Money.abs/1","type":"function","doc":"The absolute value of a Money amount. Returns a Money type with a positive sign for the amount. Arguments money is any valid Money.t type returned by Money.new/2 Returns a Money.t Example iex&gt; m = Money.new(&quot;USD&quot;, -100) iex&gt; Money.abs(m) #Money&lt;:USD, 100&gt;"},{"ref":"Money.html#add/2","title":"Money.add/2","type":"function","doc":"Add two Money values. Arguments money_1 and money_2 are any valid Money.t types returned by Money.new/2 Returns {:ok, money} or {:error, reason} Example iex&gt; Money.add Money.new(:USD, 200), Money.new(:USD, 100) {:ok, Money.new(:USD, 300)} iex&gt; Money.add Money.new(:USD, 200), Money.new(:AUD, 100) {:error, {ArgumentError, &quot;Cannot add monies with different currencies. &quot; &lt;&gt; &quot;Received :USD and :AUD.&quot;}}"},{"ref":"Money.html#add!/2","title":"Money.add!/2","type":"function","doc":"Add two Money values and raise on error. Arguments money_1 and money_2 are any valid Money.t types returned by Money.new/2 Returns {:ok, money} or raises an exception Examples iex&gt; Money.add! Money.new(:USD, 200), Money.new(:USD, 100) #Money&lt;:USD, 300&gt; Money.add! Money.new(:USD, 200), Money.new(:CAD, 500) ** (ArgumentError) Cannot add two %Money{} with different currencies. Received :USD and :CAD."},{"ref":"Money.html#cmp/2","title":"Money.cmp/2","type":"function","doc":"Compares two Money values numerically. If the first number is greater than the second :gt is returned, if less than :lt is returned, if both numbers are equal :eq is returned. Arguments money_1 and money_2 are any valid Money.t types returned by Money.new/2 Returns :gt | :eq | :lt or {:error, {module(), String.t}} Examples iex&gt; Money.cmp Money.new(:USD, 200), Money.new(:USD, 100) :gt iex&gt; Money.cmp Money.new(:USD, 200), Money.new(:USD, 200) :eq iex&gt; Money.cmp Money.new(:USD, 200), Money.new(:USD, 500) :lt iex&gt; Money.cmp Money.new(:USD, 200), Money.new(:CAD, 500) {:error, {ArgumentError, &quot;Cannot compare monies with different currencies. Received :USD and :CAD.&quot;}}"},{"ref":"Money.html#cmp!/2","title":"Money.cmp!/2","type":"function","doc":"Compares two Money values numerically and raises on error. Arguments money_1 and money_2 are any valid Money.t types returned by Money.new/2 Returns :gt | :eq | :lt or raises an exception Examples Money.cmp! Money.new(:USD, 200), Money.new(:CAD, 500) ** (ArgumentError) Cannot compare monies with different currencies. Received :USD and :CAD."},{"ref":"Money.html#compare/2","title":"Money.compare/2","type":"function","doc":"Compares two Money values numerically. If the first number is greater than the second #Integer is returned, if less than Integer&lt;-1&gt; is returned. Otherwise, if both numbers are equal Integer is returned. Arguments money_1 and money_2 are any valid Money.t types returned by Money.new/2 Returns -1 | 0 | 1 or {:error, {module(), String.t}} Examples iex&gt; Money.compare Money.new(:USD, 200), Money.new(:USD, 100) 1 iex&gt; Money.compare Money.new(:USD, 200), Money.new(:USD, 200) 0 iex&gt; Money.compare Money.new(:USD, 200), Money.new(:USD, 500) -1 iex&gt; Money.compare Money.new(:USD, 200), Money.new(:CAD, 500) {:error, {ArgumentError, &quot;Cannot compare monies with different currencies. Received :USD and :CAD.&quot;}}"},{"ref":"Money.html#compare!/2","title":"Money.compare!/2","type":"function","doc":"Compares two Money values numerically and raises on error. Arguments money_1 and money_2 are any valid Money.t types returned by Money.new/2 Returns -1 | 0 | 1 or raises an exception Examples Money.compare! Money.new(:USD, 200), Money.new(:CAD, 500) ** (ArgumentError) Cannot compare monies with different currencies. Received :USD and :CAD."},{"ref":"Money.html#cross_rate/3","title":"Money.cross_rate/3","type":"function","doc":"Returns the effective cross-rate to convert from one currency to another. Arguments from is any Money.t struct returned by Cldr.Currency.new/2 or a valid currency code to_currency is a valid currency code into which the money is converted rates is a Map of currency rates where the map key is an upcased atom or string and the value is a Decimal conversion factor. The default is the latest available exchange rates returned from Money.ExchangeRates.latest_rates() Examples Money.cross_rate(Money.new(:USD, 100), :AUD, %{USD: Decimal.new(1), AUD: Decimal.new(&quot;0.7345&quot;)}) {:ok, #Decimal&lt;0.7345&gt;} Money.cross_rate Money.new(:USD, 100), :ZZZ, %{USD: Decimal.new(1), AUD: Decimal.new(0.7345)} ** (Cldr.UnknownCurrencyError) Currency :ZZZ is not known"},{"ref":"Money.html#cross_rate!/3","title":"Money.cross_rate!/3","type":"function","doc":"Returns the effective cross-rate to convert from one currency to another. Arguments from is any Money.t struct returned by Cldr.Currency.new/2 or a valid currency code to_currency is a valid currency code into which the money is converted rates is a Map of currency rates where the map key is an upcased atom or string and the value is a Decimal conversion factor. The default is the latest available exchange rates returned from Money.ExchangeRates.latest_rates() Examples iex&gt; Money.cross_rate!(Money.new(:USD, 100), :AUD, %{USD: Decimal.new(1), AUD: Decimal.new(&quot;0.7345&quot;)}) #Decimal&lt;0.7345&gt; iex&gt; Money.cross_rate!(:USD, :AUD, %{USD: Decimal.new(1), AUD: Decimal.new(&quot;0.7345&quot;)}) #Decimal&lt;0.7345&gt; Money.cross_rate Money.new(:USD, 100), :ZZZ, %{USD: Decimal.new(1), AUD: Decimal.new(&quot;0.7345&quot;)} ** (Cldr.UnknownCurrencyError) Currency :ZZZ is not known"},{"ref":"Money.html#div/2","title":"Money.div/2","type":"function","doc":"Divide a Money value by a number. Arguments money is any valid Money.t types returned by Money.new/2 number is an integer, float or Decimal.t Note that dividing one %Money{} by another is not supported. Returns {:ok, money} or {:error, reason} Example iex&gt; Money.div Money.new(:USD, 200), 2 {:ok, Money.new(:USD, 100)} iex&gt; Money.div(Money.new(:USD, 200), &quot;xx&quot;) {:error, {ArgumentError, &quot;Cannot divide money by \\&quot;xx\\&quot;&quot;}}"},{"ref":"Money.html#div!/2","title":"Money.div!/2","type":"function","doc":"Divide a Money value by a number and raise on error. Arguments money is any valid Money.t types returned by Money.new/2 number is an integer, float or Decimal.t Returns a Money.t struct or raises an exception Examples iex&gt; Money.div Money.new(:USD, 200), 2 {:ok, Money.new(:USD, 100)} Money.div(Money.new(:USD, 200), &quot;xx&quot;) ** (ArgumentError) &quot;Cannot divide money by \\&quot;xx\\&quot;&quot;]}}"},{"ref":"Money.html#equal?/2","title":"Money.equal?/2","type":"function","doc":"Returns a boolean indicating if two Money values are equal Arguments money_1 and money_2 are any valid Money.t types returned by Money.new/2 Returns true or false Example iex&gt; Money.equal? Money.new(:USD, 200), Money.new(:USD, 200) true iex&gt; Money.equal? Money.new(:USD, 200), Money.new(:USD, 100) false"},{"ref":"Money.html#from_float/2","title":"Money.from_float/2","type":"function","doc":"Returns a %Money{} struct from a currency code and a float amount, or an error tuple of the form {:error, {exception, message}}. Floats are fraught with danger in computer arithmetic due to the unexpected loss of precision during rounding. The IEEE754 standard indicates that a number with a precision of 16 digits should round-trip convert without loss of fidelity. This function supports numbers with a precision up to 15 digits and will error if the provided amount is outside that range. Note that Money cannot detect lack of precision or rounding errors introduced upstream. This function therefore should be used with great care and its use should be considered potentially harmful. Arguments currency_code is an ISO4217 three-character upcased binary or atom amount is a float Examples iex&gt; Money.from_float 1.23456, :USD #Money&lt;:USD, 1.23456&gt; iex&gt; Money.from_float 1.234567890987656, :USD {:error, {Money.InvalidAmountError, &quot;The precision of the float 1.234567890987656 is &quot; &lt;&gt; &quot;greater than 15 which could lead to unexpected results. &quot; &lt;&gt; &quot;Reduce the precision or call Money.new/2 with a Decimal or String amount&quot;}}"},{"ref":"Money.html#from_float!/2","title":"Money.from_float!/2","type":"function","doc":"Returns a %Money{} struct from a currency code and a float amount, or raises an exception if the currency code is invalid. See Money.from_float/2 for further information. Note that Money cannot detect lack of precision or rounding errors introduced upstream. This function therefore should be used with great care and its use should be considered potentially harmful. Arguments currency_code is an ISO4217 three-character upcased binary or atom amount is a float Examples iex&gt; Money.from_float!(:USD, 1.234) #Money&lt;:USD, 1.234&gt; Money.from_float!(:USD, 1.234567890987654) #=&gt; ** (Money.InvalidAmountError) The precision of the float 1.234567890987654 is greater than 15 which could lead to unexpected results. Reduce the precision or call Money.new/2 with a Decimal or String amount (ex_money) lib/money.ex:293: Money.from_float!/2"},{"ref":"Money.html#from_integer/2","title":"Money.from_integer/2","type":"function","doc":"Convert an integer representation of money into a Money struct. This is the inverse operation of Money.to_integer_exp/1. Note that the ISO definition of currency digits (subunit) is always used. This is, in some cases like the Colombian Peso (COP) different to the CLDR definition. Options integer is an integer representation of a mooney item including any decimal digits. ie. 20000 would interpreted to mean $200.00 currency is the currency code for the integer. The assumed decimal places is derived from the currency code. Returns A Money struct or {:error, {Cldr.UnknownCurrencyError, message}} Examples iex&gt; Money.from_integer(20000, :USD) #Money&lt;:USD, 200.00&gt; iex&gt; Money.from_integer(200, :JPY) #Money&lt;:JPY, 200&gt; iex&gt; Money.from_integer(20012, :USD) #Money&lt;:USD, 200.12&gt; iex&gt; Money.from_integer(20012, :COP) #Money&lt;:COP, 200.12&gt;"},{"ref":"Money.html#get_env/3","title":"Money.get_env/3","type":"function","doc":""},{"ref":"Money.html#known_currencies/0","title":"Money.known_currencies/0","type":"function","doc":"See Cldr.known_currencies/0."},{"ref":"Money.html#known_current_currencies/0","title":"Money.known_current_currencies/0","type":"function","doc":"See Money.Currency.known_current_currencies/0."},{"ref":"Money.html#known_historic_currencies/0","title":"Money.known_historic_currencies/0","type":"function","doc":"See Money.Currency.known_historic_currencies/0."},{"ref":"Money.html#known_tender_currencies/0","title":"Money.known_tender_currencies/0","type":"function","doc":"See Money.Currency.known_tender_currencies/0."},{"ref":"Money.html#mult/2","title":"Money.mult/2","type":"function","doc":"Multiply a Money value by a number. Arguments money is any valid Money.t type returned by Money.new/2 number is an integer, float or Decimal.t Note that multipling one %Money{} by another is not supported. Returns {:ok, money} or {:error, reason} Example iex&gt; Money.mult(Money.new(:USD, 200), 2) {:ok, Money.new(:USD, 400)} iex&gt; Money.mult(Money.new(:USD, 200), &quot;xx&quot;) {:error, {ArgumentError, &quot;Cannot multiply money by \\&quot;xx\\&quot;&quot;}}"},{"ref":"Money.html#mult!/2","title":"Money.mult!/2","type":"function","doc":"Multiply a Money value by a number and raise on error. Arguments money is any valid Money.t types returned by Money.new/2 number is an integer, float or Decimal.t Returns a Money.t or raises an exception Examples iex&gt; Money.mult!(Money.new(:USD, 200), 2) #Money&lt;:USD, 400&gt; Money.mult!(Money.new(:USD, 200), :invalid) ** (ArgumentError) Cannot multiply money by :invalid"},{"ref":"Money.html#new/3","title":"Money.new/3","type":"function","doc":"Returns a %Money{} struct from a currency code and a currency amount or an error tuple of the form {:error, {exception, message}}. Arguments currency_code is an ISO4217 three-character upcased binary or atom amount is an integer, string or Decimal Options :locale is any known locale. The locale is used to normalize any binary (String) amounts to a form that can be consumed by Decimal.new/1. This consists of removing any localised grouping characters and replacing the localised decimal separator with a &quot;.&quot;. Note that the currency_code and amount arguments can be supplied in either order, Examples iex&gt; Money.new(:USD, 100) #Money&lt;:USD, 100&gt; iex&gt; Money.new(100, :USD) #Money&lt;:USD, 100&gt; iex&gt; Money.new(&quot;USD&quot;, 100) #Money&lt;:USD, 100&gt; iex&gt; Money.new(&quot;thb&quot;, 500) #Money&lt;:THB, 500&gt; iex&gt; Money.new(&quot;EUR&quot;, Decimal.new(100)) #Money&lt;:EUR, 100&gt; iex&gt; Money.new(:EUR, &quot;100.30&quot;) #Money&lt;:EUR, 100.30&gt; iex&gt; Money.new(:XYZZ, 100) {:error, {Money.UnknownCurrencyError, &quot;The currency :XYZZ is invalid&quot;}} iex&gt; Money.new(&quot;1.000,99&quot;, :EUR, locale: &quot;de&quot;) #Money&lt;:EUR, 1000.99&gt; iex&gt; Money.new 123.445, :USD {:error, {Money.InvalidAmountError, &quot;Float amounts are not supported in new/2 due to potenial &quot; &lt;&gt; &quot;rounding and precision issues. If absolutely required, &quot; &lt;&gt; &quot;use Money.from_float/2&quot;}}"},{"ref":"Money.html#new!/2","title":"Money.new!/2","type":"function","doc":"Returns a %Money{} struct from a currency code and a currency amount. Raises an exception if the current code is invalid. Arguments currency_code is an ISO4217 three-character upcased binary or atom amount is an integer, float or Decimal Examples Money.new!(:XYZZ, 100) ** (Money.UnknownCurrencyError) Currency :XYZZ is not known (ex_money) lib/money.ex:177: Money.new!/2"},{"ref":"Money.html#parse/2","title":"Money.parse/2","type":"function","doc":"Parse a string and return a Money.t or an error. The string to be parsed is required to have a currency code and an amount. The currency code may be placed before the amount or after, but not both. Parsing is strict. Additional text surrounding the currency code and amount will cause the parse to fail. Arguments string is a string to be parsed options is a keyword list of options that is passed to Money.new/3 with the exception of the options listed below Options backend is any module() that includes use Cldr and therefore is a Cldr backend module(). THe default is Money.default_backend() locale_name is any valid locale name returned by Cldr.known_locale_names/1 or a Cldr.LanguageTag struct returned by Cldr.Locale.new!/2 The default is &lt;backend&gt;.get_locale() currency_filter is an atom or list of atoms representing the currency types to be considered for a match. If a list of atoms is given, the currency must meet all criteria for it to be considered. :all, the default, considers all currencies :current considers those currencies that have a :to date of nil and which also is a known ISO4217 currency :historic is the opposite of :current :tender considers currencies that are legal tender :unannotated considers currencies that don&#39;t have &quot;(some string)&quot; in their names. These are usually financial instruments. fuzzy is a float greater than 0.0 and less than or equal to 1.0 which is used as input to the String.jaro_distance/2 to determine is the provided currency string is close enough to a known currency string for it to identify definitively a currency code. It is recommended to use numbers greater than 0.8 in order to reduce false positives. Returns a Money.t if parsing is successful or {:error, {exception, reason}} if an error is detected. Examples iex&gt; Money.parse(&quot;USD 100&quot;) #Money&lt;:USD, 100&gt; iex&gt; Money.parse &quot;USD 100,00&quot;, locale: &quot;de&quot; #Money&lt;:USD, 100.00&gt; iex&gt; Money.parse(&quot;100 USD&quot;) #Money&lt;:USD, 100&gt; iex&gt; Money.parse(&quot;100 eurosports&quot;, fuzzy: 0.8) #Money&lt;:EUR, 100&gt; iex&gt; Money.parse(&quot;100 eurosports&quot;, fuzzy: 0.9) {:error, {Money.Invalid, &quot;Unable to create money from \\&quot;eurosports\\&quot; and \\&quot;100\\&quot;&quot;}} iex&gt; Money.parse(&quot;100 afghan afghanis&quot;) #Money&lt;:AFN, 100&gt; iex&gt; Money.parse(&quot;100&quot;) {:error, {Money.Invalid, &quot;A currency code must be specified but was not found in \\&quot;100\\&quot;&quot;}} iex&gt; Money.parse(&quot;USD 100 with trailing text&quot;) {:error, {Money.Invalid, &quot;A currency code can only be specified once. &quot; &lt;&gt; &quot;Found both \\&quot;usd\\&quot; and \\&quot;with trailing text\\&quot;.&quot;}}"},{"ref":"Money.html#put_fraction/2","title":"Money.put_fraction/2","type":"function","doc":"Set the fractional part of a Money. Arguments money is a %Money{} struct fraction is an integer amount that will be set as the fraction of the money Notes The fraction can only be set if it matches the number of decimal digits for the currency associated with the money. Therefore, for a currency with 2 decimal digits, the maximum for fraction is 99. Examples iex&gt; Money.put_fraction Money.new(:USD, &quot;2.49&quot;), 99 #Money&lt;:USD, 2.99&gt; iex&gt; Money.put_fraction Money.new(:USD, &quot;2.49&quot;), 0 #Money&lt;:USD, 2.0&gt; iex&gt; Money.put_fraction Money.new(:USD, &quot;2.49&quot;), 999 {:error, {Money.InvalidAmountError, &quot;Rounding up to 999 is invalid for currency :USD&quot;}}"},{"ref":"Money.html#reduce/1","title":"Money.reduce/1","type":"function","doc":"Calls Decimal.reduce/1 on the given Money.t() This will reduce the coefficient and exponent of the decimal amount in a standard way that may aid in native comparison of %Money.t() items. Example iex&gt; x = %Money{currency: :USD, amount: %Decimal{sign: 1, coef: 42, exp: 0}} #Money&lt;:USD, 42&gt; iex&gt; y = %Money{currency: :USD, amount: %Decimal{sign: 1, coef: 4200000000, exp: -8}} #Money&lt;:USD, 42.00000000&gt; iex&gt; x == y false iex&gt; y = Money.reduce(x) #Money&lt;:USD, 42&gt; iex&gt; x == y true"},{"ref":"Money.html#round/2","title":"Money.round/2","type":"function","doc":"Round a Money value into the acceptable range for the requested currency. Arguments money is a %Money{} struct opts is a keyword list of options Options :rounding_mode that defines how the number will be rounded. See Decimal.Context. The default is :half_even which is also known as &quot;banker&#39;s rounding&quot; :currency_digits which determines the rounding increment. The valid options are :cash, :accounting and :iso or an integer value representing the rounding factor. The default is :iso. Notes There are two kinds of rounding applied: Round to the appropriate number of fractional digits Apply an appropriate rounding increment. Most currencies round to the same precision as the number of decimal digits, but some such as :CHF round to a minimum such as 0.05 when its a cash amount. The rounding increment is applied when the option :currency_digits is set to :cash Examples iex&gt; Money.round Money.new(&quot;123.73&quot;, :CHF), currency_digits: :cash #Money&lt;:CHF, 123.75&gt; iex&gt; Money.round Money.new(&quot;123.73&quot;, :CHF), currency_digits: 0 #Money&lt;:CHF, 124&gt; iex&gt; Money.round Money.new(&quot;123.7456&quot;, :CHF) #Money&lt;:CHF, 123.75&gt; iex&gt; Money.round Money.new(&quot;123.7456&quot;, :JPY) #Money&lt;:JPY, 124&gt;"},{"ref":"Money.html#split/2","title":"Money.split/2","type":"function","doc":"Split a Money value into a number of parts maintaining the currency&#39;s precision and rounding and ensuring that the parts sum to the original amount. Arguments money is a %Money{} struct parts is an integer number of parts into which the money is split Returns a tuple {dividend, remainder} as the function result derived as follows: Round the money amount to the required currency precision using Money.round/1 Divide the result of step 1 by the integer divisor Round the result of the division to the precision of the currency using Money.round/1 Return two numbers: the result of the division and any remainder that could not be applied given the precision of the currency. Examples Money.split Money.new(123.5, :JPY), 3 {¥41, ¥1} Money.split Money.new(123.4, :JPY), 3 {¥41, ¥0} Money.split Money.new(123.7, :USD), 9 {$13.74, $0.04}"},{"ref":"Money.html#sub/2","title":"Money.sub/2","type":"function","doc":"Subtract one Money value struct from another. Options money_1 and money_2 are any valid Money.t types returned by Money.new/2 Returns {:ok, money} or {:error, reason} Example iex&gt; Money.sub Money.new(:USD, 200), Money.new(:USD, 100) {:ok, Money.new(:USD, 100)}"},{"ref":"Money.html#sub!/2","title":"Money.sub!/2","type":"function","doc":"Subtract one Money value struct from another and raise on error. Returns either {:ok, money} or {:error, reason}. Arguments money_1 and money_2 are any valid Money.t types returned by Money.new/2 Returns a Money.t struct or raises an exception Examples iex&gt; Money.sub! Money.new(:USD, 200), Money.new(:USD, 100) #Money&lt;:USD, 100&gt; Money.sub! Money.new(:USD, 200), Money.new(:CAD, 500) ** (ArgumentError) Cannot subtract monies with different currencies. Received :USD and :CAD."},{"ref":"Money.html#to_currency/3","title":"Money.to_currency/3","type":"function","doc":"Convert money from one currency to another. Arguments money is any Money.t struct returned by Cldr.Currency.new/2 to_currency is a valid currency code into which the money is converted rates is a Map of currency rates where the map key is an upcased atom or string and the value is a Decimal conversion factor. The default is the latest available exchange rates returned from Money.ExchangeRates.latest_rates() Examples Money.to_currency(Money.new(:USD, 100), :AUD, %{USD: Decimal.new(1), AUD: Decimal.from_float(0.7345)}) {:ok, #Money&lt;:AUD, 73.4500&gt;} Money.to_currency(Money.new(&quot;USD&quot;, 100), &quot;AUD&quot;, %{&quot;USD&quot; =&gt; Decimal.new(1), &quot;AUD&quot; =&gt; Decimal.from_float(0.7345)}) {:ok, #Money&lt;:AUD, 73.4500&gt;} iex&gt; Money.to_currency Money.new(:USD, 100), :AUDD, %{USD: Decimal.new(1), AUD: Decimal.from_float(0.7345)} {:error, {Cldr.UnknownCurrencyError, &quot;The currency :AUDD is invalid&quot;}} iex&gt; Money.to_currency Money.new(:USD, 100), :CHF, %{USD: Decimal.new(1), AUD: Decimal.from_float(0.7345)} {:error, {Money.ExchangeRateError, &quot;No exchange rate is available for currency :CHF&quot;}}"},{"ref":"Money.html#to_currency!/3","title":"Money.to_currency!/3","type":"function","doc":"Convert money from one currency to another and raises on error Arguments money is any Money.t struct returned by Cldr.Currency.new/2 to_currency is a valid currency code into which the money is converted rates is a Map of currency rates where the map key is an upcased atom or string and the value is a Decimal conversion factor. The default is the latest available exchange rates returned from Money.ExchangeRates.latest_rates() Examples iex&gt; Money.to_currency! Money.new(:USD, 100), :AUD, %{USD: Decimal.new(1), AUD: Decimal.from_float(0.7345)} #Money&lt;:AUD, 73.4500&gt; iex&gt; Money.to_currency! Money.new(&quot;USD&quot;, 100), &quot;AUD&quot;, %{&quot;USD&quot; =&gt; Decimal.new(1), &quot;AUD&quot; =&gt; Decimal.from_float(0.7345)} #Money&lt;:AUD, 73.4500&gt; Money.to_currency! Money.new(:USD, 100), :ZZZ, %{USD: Decimal.new(1), AUD: Decimal.from_float(0.7345)} ** (Cldr.UnknownCurrencyError) Currency :ZZZ is not known"},{"ref":"Money.html#to_decimal/1","title":"Money.to_decimal/1","type":"function","doc":"Returns the amount part of a Money type as a Decimal Arguments money is any valid Money.t type returned by Money.new/2 Returns a Decimal.t Example iex&gt; m = Money.new(&quot;USD&quot;, 100) iex&gt; Money.to_decimal(m) #Decimal&lt;100&gt;"},{"ref":"Money.html#to_integer_exp/2","title":"Money.to_integer_exp/2","type":"function","doc":"Returns a tuple comprising the currency code, integer amount, exponent and remainder Some services require submission of money items as an integer with an implied exponent that is appropriate to the currency. Rather than return only the integer, Money.to_integer_exp returns the currency code, integer, exponent and remainder. The remainder is included because to return an integer money with an implied exponent the Money has to be rounded potentially leaving a remainder. Options money is any Money.t struct returned by Cldr.Currency.new/2 Notes Since the returned integer is expected to have the implied fractional digits the Money needs to be rounded which is what this function does. Example iex&gt; m = Money.new(:USD, &quot;200.012356&quot;) #Money&lt;:USD, 200.012356&gt; iex&gt; Money.to_integer_exp(m) {:USD, 20001, -2, Money.new(:USD, &quot;0.002356&quot;)} iex&gt; m = Money.new(:USD, &quot;200.00&quot;) #Money&lt;:USD, 200.00&gt; iex&gt; Money.to_integer_exp(m) {:USD, 20000, -2, Money.new(:USD, &quot;0.00&quot;)}"},{"ref":"Money.html#to_string/2","title":"Money.to_string/2","type":"function","doc":"Returns a formatted string representation of a Money{}. Formatting is performed according to the rules defined by CLDR. See Cldr.Number.to_string/2 for formatting options. The default is to format as a currency which applies the appropriate rounding and fractional digits for the currency. Arguments money is any valid Money.t type returned by Money.new/2 options is a keyword list of options Returns {:ok, string} or {:error, reason} Options :backend is any CLDR backend module. The default is Money.default_backend(). Any other options are passed to Cldr.Number.to_string/3 Examples iex&gt; Money.to_string Money.new(:USD, 1234) {:ok, &quot;$1,234.00&quot;} iex&gt; Money.to_string Money.new(:JPY, 1234) {:ok, &quot;¥1,234&quot;} iex&gt; Money.to_string Money.new(:THB, 1234) {:ok, &quot;THB 1,234.00&quot;} iex&gt; Money.to_string Money.new(:USD, 1234), format: :long {:ok, &quot;1,234 US dollars&quot;}"},{"ref":"Money.html#to_string!/2","title":"Money.to_string!/2","type":"function","doc":"Returns a formatted string representation of a Money.t or raises if there is an error. Formatting is performed according to the rules defined by CLDR. See Cldr.Number.to_string!/2 for formatting options. The default is to format as a currency which applies the appropriate rounding and fractional digits for the currency. Arguments money is any valid Money.t type returned by Money.new/2 options is a keyword list of options Options :backend is any CLDR backend module. The default is Money.default_backend(). Any other options are passed to Cldr.Number.to_string/3 Examples iex&gt; Money.to_string! Money.new(:USD, 1234) &quot;$1,234.00&quot; iex&gt; Money.to_string! Money.new(:JPY, 1234) &quot;¥1,234&quot; iex&gt; Money.to_string! Money.new(:THB, 1234) &quot;THB 1,234.00&quot; iex&gt; Money.to_string! Money.new(:USD, 1234), format: :long &quot;1,234 US dollars&quot;"},{"ref":"Money.html#validate_currency/1","title":"Money.validate_currency/1","type":"function","doc":"See Cldr.validate_currency/1."},{"ref":"Money.html#zero/1","title":"Money.zero/1","type":"function","doc":"Return a zero amount Money.t in the given currency Example iex&gt; Money.zero(:USD) #Money&lt;:USD, 0&gt; iex&gt; money = Money.new(:USD, 200) iex&gt; Money.zero(money) #Money&lt;:USD, 0&gt; iex&gt; Money.zero :ZZZ {:error, {Cldr.UnknownCurrencyError, &quot;The currency :ZZZ is invalid&quot;}}"},{"ref":"Money.html#t:amount/0","title":"Money.amount/0","type":"type","doc":""},{"ref":"Money.html#t:currency_code/0","title":"Money.currency_code/0","type":"type","doc":""},{"ref":"Money.html#t:t/0","title":"Money.t/0","type":"type","doc":"Money is composed of an atom representation of an ISO4217 currency code and a Decimal representation of an amount."},{"ref":"Money.Application.html","title":"Money.Application","type":"module","doc":""},{"ref":"Money.Application.html#start/2","title":"Money.Application.start/2","type":"function","doc":"Called when an application is started. This function is called when an application is started using Application.start/2 (and functions on top of that, such as Application.ensure_started/2). This function should start the top-level process of the application (which should be the top supervisor of the application&#39;s supervision tree if the application follows the OTP design principles around supervision). start_type defines how the application is started: :normal - used if the startup is a normal startup or if the application is distributed and is started on the current node because of a failover from another node and the application specification key :start_phases is :undefined. {:takeover, node} - used if the application is distributed and is started on the current node because of a failover on the node node. {:failover, node} - used if the application is distributed and is started on the current node because of a failover on node node, and the application specification key :start_phases is not :undefined. start_args are the arguments passed to the application in the :mod specification key (e.g., mod: {MyApp, [:my_args]}). This function should either return {:ok, pid} or {:ok, pid, state} if startup is successful. pid should be the PID of the top supervisor. state can be an arbitrary term, and if omitted will default to []; if the application is later stopped, state is passed to the stop/1 callback (see the documentation for the c:stop/1 callback for more information). use Application provides no default implementation for the start/2 callback. Callback implementation for Application.start/2."},{"ref":"Money.Cldr.html","title":"Money.Cldr","type":"module","doc":"Provides the core functions to retrieve and manage the CLDR data that supports formatting and localisation. It provides the core functions to access formatted CLDR data, set and retrieve a current locale and validate certain core data types such as locales, currencies and territories."},{"ref":"Money.Cldr.html#available_locale_name?/1","title":"Money.Cldr.available_locale_name?/1","type":"function","doc":"See Cldr.available_locale_name?/1."},{"ref":"Money.Cldr.html#default_locale/0","title":"Money.Cldr.default_locale/0","type":"function","doc":"Returns the default locale. Example iex&gt; Money.Cldr.default_locale() %Cldr.LanguageTag{ canonical_locale_name: &quot;en-Latn-001&quot;, cldr_locale_name: &quot;en-001&quot;, language_subtags: [], extensions: %{}, gettext_locale_name: &quot;en&quot;, language: &quot;en&quot;, locale: %{}, private_use: [], rbnf_locale_name: &quot;en&quot;, requested_locale_name: &quot;en-001&quot;, script: &quot;Latn&quot;, territory: &quot;001&quot;, transform: %{}, language_variant: nil }"},{"ref":"Money.Cldr.html#default_territory/0","title":"Money.Cldr.default_territory/0","type":"function","doc":"Returns the default territory when a locale does not specify one and none can be inferred. Example iex&gt; Money.Cldr.default_territory() :&quot;001&quot;"},{"ref":"Money.Cldr.html#get_locale/0","title":"Money.Cldr.get_locale/0","type":"function","doc":"Return the current locale to be used for Cldr functions that take an optional locale parameter for which a locale is not supplied. Example iex&gt; Money.Cldr.put_locale(&quot;pl&quot;) iex&gt; Money.Cldr.get_locale %Cldr.LanguageTag{ canonical_locale_name: &quot;pl-Latn-PL&quot;, cldr_locale_name: &quot;pl&quot;, extensions: %{}, language: &quot;pl&quot;, locale: %{}, private_use: [], rbnf_locale_name: &quot;pl&quot;, territory: &quot;PL&quot;, requested_locale_name: &quot;pl&quot;, script: &quot;Latn&quot;, transform: %{}, language_variant: nil }"},{"ref":"Money.Cldr.html#known_calendars/0","title":"Money.Cldr.known_calendars/0","type":"function","doc":"See Cldr.known_calendars/0."},{"ref":"Money.Cldr.html#known_currencies/0","title":"Money.Cldr.known_currencies/0","type":"function","doc":"See Cldr.known_currencies/0."},{"ref":"Money.Cldr.html#known_gettext_locale_name/1","title":"Money.Cldr.known_gettext_locale_name/1","type":"function","doc":"Returns either the Gettext locale_name in Cldr format or false based upon whether the locale name is configured in Gettext. Arguments locale is any valid locale name returned by Money.Cldr.known_gettext_locale_names/0 Examples iex&gt; Money.Cldr.known_gettext_locale_name &quot;en&quot; &quot;en&quot; iex&gt; Money.Cldr.known_gettext_locale_name &quot;en-SA&quot; false"},{"ref":"Money.Cldr.html#known_gettext_locale_name?/1","title":"Money.Cldr.known_gettext_locale_name?/1","type":"function","doc":"Returns a boolean indicating if the specified locale name is configured and available in Gettext. Arguments locale is any valid locale name returned by Money.Cldr.known_locale_names/0 Examples iex&gt; Money.Cldr.known_gettext_locale_name?(&quot;en&quot;) true iex&gt; Money.Cldr.known_gettext_locale_name?(&quot;!!&quot;) false"},{"ref":"Money.Cldr.html#known_gettext_locale_names/0","title":"Money.Cldr.known_gettext_locale_names/0","type":"function","doc":"Returns a list of GetText locale names but in CLDR format with underscore replaced by hyphen in order to facilitate comparisons with Cldr locale names."},{"ref":"Money.Cldr.html#known_locale_name/1","title":"Money.Cldr.known_locale_name/1","type":"function","doc":"Returns either the locale_name or false based upon whether the locale name is configured in Cldr. This is helpful when building a list of or expressions to return the first known locale name from a list. Arguments locale is any valid locale name returned by Money.Cldr.known_locale_names/0 Examples iex&gt; Money.Cldr.known_locale_name &quot;en-AU&quot; &quot;en-AU&quot; iex&gt; Money.Cldr.known_locale_name &quot;en-SA&quot; false"},{"ref":"Money.Cldr.html#known_locale_name?/1","title":"Money.Cldr.known_locale_name?/1","type":"function","doc":"Returns a boolean indicating if the specified locale name is configured and available in Cldr. Arguments locale is any valid locale name returned by Money.Cldr.known_locale_names/0 Examples iex&gt; Money.Cldr.known_locale_name?(&quot;en&quot;) true iex&gt; Money.Cldr.known_locale_name?(&quot;!!&quot;) false"},{"ref":"Money.Cldr.html#known_locale_names/0","title":"Money.Cldr.known_locale_names/0","type":"function","doc":"Returns a list of the known locale names. Known locales are those locales which are the subset of all CLDR locales that have been configured for use either in this module or in Gettext."},{"ref":"Money.Cldr.html#known_number_system_types/0","title":"Money.Cldr.known_number_system_types/0","type":"function","doc":"Returns a list of atoms representing the number systems types known to Cldr. Example iex&gt; Money.Cldr.known_number_system_types [:default, :finance, :native, :traditional]"},{"ref":"Money.Cldr.html#known_number_systems/0","title":"Money.Cldr.known_number_systems/0","type":"function","doc":"See Cldr.known_number_systems/0."},{"ref":"Money.Cldr.html#known_rbnf_locale_name/1","title":"Money.Cldr.known_rbnf_locale_name/1","type":"function","doc":"Returns either the RBNF locale_name or false based upon whether the locale name is configured in Cldr and has RBNF rules defined. Arguments locale is any valid locale name returned by Money.Cldr.known_locale_names/0 Examples iex&gt; Money.Cldr.known_rbnf_locale_name &quot;en&quot; &quot;en&quot; iex&gt; Money.Cldr.known_rbnf_locale_name &quot;en-SA&quot; false"},{"ref":"Money.Cldr.html#known_rbnf_locale_name?/1","title":"Money.Cldr.known_rbnf_locale_name?/1","type":"function","doc":"Returns a boolean indicating if the specified locale name is configured and available in Cldr and supports rules based number formats (RBNF). Arguments locale is any valid locale name returned by Money.Cldr.known_locale_names/0 Examples iex&gt; Money.Cldr.known_rbnf_locale_name?(&quot;en&quot;) true iex&gt; Money.Cldr.known_rbnf_locale_name?(&quot;!!&quot;) false"},{"ref":"Money.Cldr.html#known_rbnf_locale_names/0","title":"Money.Cldr.known_rbnf_locale_names/0","type":"function","doc":"Returns a list of locale names which have rules-based number formats (RBNF)."},{"ref":"Money.Cldr.html#known_territories/0","title":"Money.Cldr.known_territories/0","type":"function","doc":"See Cldr.known_territories/0."},{"ref":"Money.Cldr.html#put_locale/1","title":"Money.Cldr.put_locale/1","type":"function","doc":"Set the current locale to be used for Cldr functions that take an optional locale parameter for which a locale is not supplied. Arguments locale is any valid locale name returned by Money.Cldr.known_locale_names/0 or a Cldr.LanguageTag struct returned by Money.Cldr.Locale.new!/1 See rfc5646 for the specification of a language tag and consult Cldr.Rfc5646.Parser for the specification as implemented that includes the CLDR extensions for &quot;u&quot; (locales) and &quot;t&quot; (transforms). Examples iex&gt; Money.Cldr.put_locale(&quot;en&quot;) {:ok, %Cldr.LanguageTag{ canonical_locale_name: &quot;en-Latn-US&quot;, cldr_locale_name: &quot;en&quot;, language_subtags: [], extensions: %{}, gettext_locale_name: &quot;en&quot;, language: &quot;en&quot;, locale: %{}, private_use: [], rbnf_locale_name: &quot;en&quot;, requested_locale_name: &quot;en&quot;, script: &quot;Latn&quot;, territory: &quot;US&quot;, transform: %{}, language_variant: nil }} iex&gt; Money.Cldr.put_locale(&quot;invalid-locale&quot;) {:error, {Cldr.LanguageTag.ParseError, &quot;Expected a BCP47 language tag. Could not parse the remaining \\&quot;le\\&quot; starting at position 13&quot;}}"},{"ref":"Money.Cldr.html#unknown_locale_names/0","title":"Money.Cldr.unknown_locale_names/0","type":"function","doc":"Returns a list of the locales names that are configured, but not known in CLDR. Since there is a compile-time exception raised if there are any unknown locales this function should always return an empty list."},{"ref":"Money.Cldr.html#validate_calendar/1","title":"Money.Cldr.validate_calendar/1","type":"function","doc":"See Cldr.validate_calendar/1."},{"ref":"Money.Cldr.html#validate_currency/1","title":"Money.Cldr.validate_currency/1","type":"function","doc":"See Cldr.validate_currency/1."},{"ref":"Money.Cldr.html#validate_locale/1","title":"Money.Cldr.validate_locale/1","type":"function","doc":"Normalise and validate a locale name. Arguments locale is any valid locale name returned by Money.Cldr.known_locale_names/0 or a Cldr.LanguageTag struct returned by Money.Cldr.Locale.new!/1 Returns {:ok, language_tag} {:error, reason} Notes See rfc5646 for the specification of a language tag and consult Cldr.Rfc5646.Parser for the specification as implemented that includes the CLDR extensions for &quot;u&quot; (locales) and &quot;t&quot; (transforms). Examples iex&gt; Money.Cldr.validate_locale(&quot;en&quot;) {:ok, %Cldr.LanguageTag{ canonical_locale_name: &quot;en-Latn-US&quot;, cldr_locale_name: &quot;en&quot;, extensions: %{}, gettext_locale_name: &quot;en&quot;, language: &quot;en&quot;, locale: %{}, private_use: [], rbnf_locale_name: &quot;en&quot;, requested_locale_name: &quot;en&quot;, script: &quot;Latn&quot;, territory: &quot;US&quot;, transform: %{}, language_variant: nil }} iex&gt; Money.Cldr.validate_locale Money.Cldr.default_locale() {:ok, %Cldr.LanguageTag{ canonical_locale_name: &quot;en-Latn-001&quot;, cldr_locale_name: &quot;en-001&quot;, extensions: %{}, gettext_locale_name: &quot;en&quot;, language: &quot;en&quot;, locale: %{}, private_use: [], rbnf_locale_name: &quot;en&quot;, requested_locale_name: &quot;en-001&quot;, script: &quot;Latn&quot;, territory: &quot;001&quot;, transform: %{}, language_variant: nil }} iex&gt; Money.Cldr.validate_locale(&quot;zzz&quot;) {:error, {Cldr.UnknownLocaleError, &quot;The locale \\&quot;zzz\\&quot; is not known.&quot;}}"},{"ref":"Money.Cldr.html#validate_number_system/1","title":"Money.Cldr.validate_number_system/1","type":"function","doc":"See Cldr.validate_number_system/1."},{"ref":"Money.Cldr.html#validate_number_system_type/1","title":"Money.Cldr.validate_number_system_type/1","type":"function","doc":"Normalise and validate a number system type. Arguments number_system_type is any number system type returned by Cldr.known_number_system_types/1 Returns {:ok, normalized_number_system_type} or {:error, {exception, message}} Examples iex&gt; Money.Cldr.validate_number_system_type :default {:ok, :default} iex&gt; Money.Cldr.validate_number_system_type :traditional {:ok, :traditional} iex&gt; Money.Cldr.validate_number_system_type :latn { :error, {Cldr.UnknownNumberSystemTypeError, &quot;The number system type :latn is unknown&quot;} } iex&gt; Money.Cldr.validate_number_system_type &quot;bork&quot; { :error, {Cldr.UnknownNumberSystemTypeError, &quot;The number system type \\&quot;bork\\&quot; is invalid&quot;} }"},{"ref":"Money.Cldr.html#validate_territory/1","title":"Money.Cldr.validate_territory/1","type":"function","doc":"See Cldr.validate_territory/1."},{"ref":"Money.Cldr.Currency.html","title":"Money.Cldr.Currency","type":"module","doc":""},{"ref":"Money.Cldr.Currency.html#currencies_for_locale/2","title":"Money.Cldr.Currency.currencies_for_locale/2","type":"function","doc":"Returns a map of the metadata for all currencies for a given locale. Arguments locale is any valid locale name returned by MyApp.Cldr.known_locale_names/0 or a Cldr.LanguageTag struct returned by MyApp.Cldr.Locale.new!/1 currency_status is :all, :current, :historic, unannotated or :tender; or a list of one or more status. The default is :all. See Cldr.Currency.currency_filter/2. Returns {:ok, currency_map} or {:error, {exception, reason}} Example MyApp.Cldr.Currency.currencies_for_locale &quot;en&quot; =&gt; {:ok, %{ FJD: %Cldr.Currency{ cash_digits: 2, cash_rounding: 0, code: &quot;FJD&quot;, count: %{one: &quot;Fijian dollar&quot;, other: &quot;Fijian dollars&quot;}, digits: 2, from: nil, iso_digits: 2, name: &quot;Fijian Dollar&quot;, narrow_symbol: &quot;$&quot;, rounding: 0, symbol: &quot;FJD&quot;, tender: true, to: nil }, SUR: %Cldr.Currency{ cash_digits: 2, cash_rounding: 0, code: &quot;SUR&quot;, count: %{one: &quot;Soviet rouble&quot;, other: &quot;Soviet roubles&quot;}, digits: 2, from: nil, iso_digits: nil, name: &quot;Soviet Rouble&quot;, narrow_symbol: nil, rounding: 0, symbol: &quot;SUR&quot;, tender: true, to: nil }, ... }}"},{"ref":"Money.Cldr.Currency.html#currencies_for_locale!/2","title":"Money.Cldr.Currency.currencies_for_locale!/2","type":"function","doc":"Returns a map of the metadata for all currencies for a given locale and raises on error. Arguments locale is any valid locale name returned by MyApp.Cldr.known_locale_names/0 or a Cldr.LanguageTag struct returned by MyApp.Cldr.Locale.new!/1 currency_status is :all, :current, :historic, unannotated or :tender; or a list of one or more status. The default is :all. See Cldr.Currency.currency_filter/2. Returns {:ok, currency_map} or raises an exception Example MyApp.Cldr.Currency.currencies_for_locale! &quot;en&quot; =&gt; %{ FJD: %Cldr.Currency{ cash_digits: 2, cash_rounding: 0, code: &quot;FJD&quot;, count: %{one: &quot;Fijian dollar&quot;, other: &quot;Fijian dollars&quot;}, digits: 2, from: nil, iso_digits: 2, name: &quot;Fijian Dollar&quot;, narrow_symbol: &quot;$&quot;, rounding: 0, symbol: &quot;FJD&quot;, tender: true, to: nil }, SUR: %Cldr.Currency{ cash_digits: 2, cash_rounding: 0, code: &quot;SUR&quot;, count: %{one: &quot;Soviet rouble&quot;, other: &quot;Soviet roubles&quot;}, digits: 2, from: nil, iso_digits: nil, name: &quot;Soviet Rouble&quot;, narrow_symbol: nil, rounding: 0, symbol: &quot;SUR&quot;, tender: true, to: nil }, ... }"},{"ref":"Money.Cldr.Currency.html#currency_for_code/2","title":"Money.Cldr.Currency.currency_for_code/2","type":"function","doc":"Returns the currency metadata for the requested currency code. Arguments currency_code is a binary or atom representation of an ISO 4217 currency code. Examples iex&gt; Money.Cldr.Currency.currency_for_code(&quot;AUD&quot;) {:ok, %Cldr.Currency{ cash_digits: 2, cash_rounding: 0, code: &quot;AUD&quot;, count: %{one: &quot;Australian dollar&quot;, other: &quot;Australian dollars&quot;}, digits: 2, iso_digits: 2, name: &quot;Australian Dollar&quot;, narrow_symbol: &quot;$&quot;, rounding: 0, symbol: &quot;A$&quot;, tender: true }} iex&gt; Money.Cldr.Currency.currency_for_code(&quot;THB&quot;) {:ok, %Cldr.Currency{ cash_digits: 2, cash_rounding: 0, code: &quot;THB&quot;, count: %{one: &quot;Thai baht&quot;, other: &quot;Thai baht&quot;}, digits: 2, iso_digits: 2, name: &quot;Thai Baht&quot;, narrow_symbol: &quot;฿&quot;, rounding: 0, symbol: &quot;THB&quot;, tender: true }}"},{"ref":"Money.Cldr.Currency.html#currency_history_for_locale/1","title":"Money.Cldr.Currency.currency_history_for_locale/1","type":"function","doc":"Returns a list of historic and the current currency for a given locale. Arguments locale is any valid locale name returned by MyApp.Cldr.known_locale_names/0 or a Cldr.LanguageTag struct returned by MyApp.Cldr.Locale.new!/1 Example iex&gt; MyApp.Cldr.Currency.currency_history_for_locale &quot;en&quot; %{ USD: %{from: ~D[1792-01-01], to: nil}, USN: %{tender: false}, USS: %{from: nil, tender: false, to: ~D[2014-03-01]} }"},{"ref":"Money.Cldr.Currency.html#currency_strings/2","title":"Money.Cldr.Currency.currency_strings/2","type":"function","doc":"Returns a map that matches a currency string to a currency code. A currency string is a localised name or symbol representing a currency in a locale-specific manner. Arguments locale is any valid locale name returned by MyApp.Cldr.known_locale_names/0 or a Cldr.LanguageTag struct returned by MyApp.Cldr.Locale.new!/1 currency_status is :all, :current, :historic, unannotated or :tender; or a list of one or more status. The default is :all. See Cldr.Currency.currency_filter/2. Returns {:ok, currency_string_map} or {:error, {exception, reason}} Example MyApp.Cldr.Currency.currency_strings &quot;en&quot; =&gt; {:ok, %{ &quot;mexican silver pesos&quot; =&gt; :MXP, &quot;sudanese dinar&quot; =&gt; :SDD, &quot;bad&quot; =&gt; :BAD, &quot;rsd&quot; =&gt; :RSD, &quot;swazi lilangeni&quot; =&gt; :SZL, &quot;zairean new zaire&quot; =&gt; :ZRN, &quot;guyanaese dollars&quot; =&gt; :GYD, &quot;equatorial guinean ekwele&quot; =&gt; :GQE, ... }}"},{"ref":"Money.Cldr.Currency.html#currency_strings!/2","title":"Money.Cldr.Currency.currency_strings!/2","type":"function","doc":"Returns a map that matches a currency string to a currency code or raises an exception. A currency string is a localised name or symbol representing a currency in a locale-specific manner. Arguments locale is any valid locale name returned by MyApp.Cldr.known_locale_names/0 or a Cldr.LanguageTag struct returned by MyApp.Cldr.Locale.new!/1 currency_status is :all, :current, :historic, unannotated or :tender; or a list of one or more status. The default is :all. See Cldr.Currency.currency_filter/2. Returns {:ok, currency_string_map} or raises an exception Example MyApp.Cldr.Currency.currency_strings! &quot;en&quot; =&gt; %{ &quot;mexican silver pesos&quot; =&gt; :MXP, &quot;sudanese dinar&quot; =&gt; :SDD, &quot;bad&quot; =&gt; :BAD, &quot;rsd&quot; =&gt; :RSD, &quot;swazi lilangeni&quot; =&gt; :SZL, &quot;zairean new zaire&quot; =&gt; :ZRN, &quot;guyanaese dollars&quot; =&gt; :GYD, &quot;equatorial guinean ekwele&quot; =&gt; :GQE, ... }"},{"ref":"Money.Cldr.Currency.html#current_currency_for_locale/1","title":"Money.Cldr.Currency.current_currency_for_locale/1","type":"function","doc":"Returns the current currency for a given locale. Arguments locale is any valid locale name returned by MyApp.Cldr.known_locale_names/0 or a Cldr.LanguageTag struct returned by MyApp.Cldr.Locale.new!/1 Example iex&gt; MyApp.Cldr.Currency.current_currency_for_locale &quot;en&quot; :USD iex&gt; MyApp.Cldr.Currency.current_currency_for_locale &quot;en-AU&quot; :AUD"},{"ref":"Money.Cldr.Currency.html#known_currencies/0","title":"Money.Cldr.Currency.known_currencies/0","type":"function","doc":"Returns a list of all known currency codes. Example iex&gt; Money.Cldr.Currency.known_currencies |&gt; Enum.count 303"},{"ref":"Money.Cldr.Currency.html#known_currency?/2","title":"Money.Cldr.Currency.known_currency?/2","type":"function","doc":"Returns a boolean indicating if the supplied currency code is known. Arguments currency_code is a binary or atom representing an ISO4217 currency code custom_currencies is an optional list of custom currencies created by the Cldr.Currency.new/2 function Returns true or false Examples iex&gt; Money.Cldr.Currency.known_currency? &quot;AUD&quot; true iex&gt; Money.Cldr.Currency.known_currency? &quot;GGG&quot; false iex&gt; Money.Cldr.Currency.known_currency? :XCV false iex&gt; Money.Cldr.Currency.known_currency? :XCV, [%Cldr.Currency{code: :XCV}] true"},{"ref":"Money.Cldr.Currency.html#make_currency_code/1","title":"Money.Cldr.Currency.make_currency_code/1","type":"function","doc":"Returns a valid normalized ISO4217 format custom currency code or an error. Currency codes conform to the ISO4217 standard which means that any custom currency code must start with an &quot;X&quot; followed by two alphabetic characters. Note that since this function creates atoms but to a maximum of 26 * 26 == 676 since the format permits 2 alphabetic characters only. Arguments currency_code is a String.t or and atom representing the new currency code to be created Returns {:ok, currency_code} or {:error, {exception, message}} Examples iex&gt; Money.Cldr.Currency.make_currency_code(&quot;xzz&quot;) {:ok, :XZZ} iex&gt; Money.Cldr.Currency.make_currency_code(&quot;aaa&quot;) {:error, {Cldr.CurrencyCodeInvalid, &quot;Invalid currency code \\&quot;AAA\\&quot;. Currency codes must start with &#39;X&#39; followed by 2 alphabetic characters only.&quot;}}"},{"ref":"Money.Cldr.Currency.html#new/2","title":"Money.Cldr.Currency.new/2","type":"function","doc":"Returns a Currency struct created from the arguments. Arguments currency is a custom currency code of a format defined in ISO4217 options is a map of options representing the optional elements of the %Cldr.Currency{} struct Returns {:ok, Cldr.Currency.t} or {:error, {exception, message}} Example iex&gt; Money.Cldr.Currency.new(:XAA) {:ok, %Cldr.Currency{cash_digits: 0, cash_rounding: 0, code: :XAA, count: nil, digits: 0, name: &quot;&quot;, narrow_symbol: nil, rounding: 0, symbol: &quot;&quot;, tender: false}} iex&gt; Money.Cldr.Currency.new(:ZAA, name: &quot;Invalid Custom Name&quot;) {:error, {Cldr.UnknownCurrencyError, &quot;The currency :ZAA is invalid&quot;}} iex&gt; Money.Cldr.Currency.new(&quot;xaa&quot;, name: &quot;Custom Name&quot;) {:ok, %Cldr.Currency{cash_digits: 0, cash_rounding: 0, code: :XAA, count: nil, digits: 0, name: &quot;Custom Name&quot;, narrow_symbol: nil, rounding: 0, symbol: &quot;&quot;, tender: false}} iex&gt; Money.Cldr.Currency.new(:XAA, name: &quot;Custom Name&quot;) {:ok, %Cldr.Currency{cash_digits: 0, cash_rounding: 0, code: :XAA, count: nil, digits: 0, name: &quot;Custom Name&quot;, narrow_symbol: nil, rounding: 0, symbol: &quot;&quot;, tender: false}} iex&gt; Money.Cldr.Currency.new(:XBC) {:error, {Cldr.CurrencyAlreadyDefined, &quot;Currency :XBC is already defined&quot;}}"},{"ref":"Money.Cldr.Currency.html#pluralize/3","title":"Money.Cldr.Currency.pluralize/3","type":"function","doc":"Returns the appropriate currency display name for the currency, based on the plural rules in effect for the locale. Arguments number is an integer, float or Decimal currency is any currency returned by Cldr.Currency.known_currencies/0 options is a keyword list of options Options locale is any valid locale name returned by MyApp.Cldr.known_locale_names/0 or a Cldr.LanguageTag struct returned by MyApp.Cldr.Locale.new!/1. The default is Cldr.get_current_locale/1 Returns {:ok, plural_string} or {:error, {exception, message}} Examples iex&gt; Money.Cldr.Currency.pluralize 1, :USD {:ok, &quot;US dollar&quot;} iex&gt; Money.Cldr.Currency.pluralize 3, :USD {:ok, &quot;US dollars&quot;} iex&gt; Money.Cldr.Currency.pluralize 12, :USD, locale: &quot;zh&quot; {:ok, &quot;美元&quot;} iex&gt; Money.Cldr.Currency.pluralize 12, :USD, locale: &quot;fr&quot; {:ok, &quot;dollars des États-Unis&quot;} iex&gt; Money.Cldr.Currency.pluralize 1, :USD, locale: &quot;fr&quot; {:ok, &quot;dollar des États-Unis&quot;}"},{"ref":"Money.Cldr.Currency.html#strings_for_currency/2","title":"Money.Cldr.Currency.strings_for_currency/2","type":"function","doc":"Returns the strings associated with a currency in a given locale. Arguments currency is an ISO4217 currency code locale is any valid locale name returned by MyApp.Cldr.known_locale_names/0 or a Cldr.LanguageTag struct returned by MyApp.Cldr.Locale.new!/1 Returns A list of strings or {:error, {exception, reason}} Example iex&gt; MyApp.Cldr.Currency.strings_for_currency :AUD, &quot;en&quot; [&quot;a$&quot;, &quot;australian dollars&quot;, &quot;aud&quot;, &quot;australian dollar&quot;]"},{"ref":"Money.Cldr.Locale.html","title":"Money.Cldr.Locale","type":"module","doc":"Backend module that provides functions to define new locales."},{"ref":"Money.Cldr.Locale.html#new/1","title":"Money.Cldr.Locale.new/1","type":"function","doc":""},{"ref":"Money.Cldr.Locale.html#new!/1","title":"Money.Cldr.Locale.new!/1","type":"function","doc":""},{"ref":"Money.Cldr.Number.html","title":"Money.Cldr.Number","type":"module","doc":"Formats numbers and currencies based upon CLDR&#39;s decimal formats specification. The format specification is documentated in Unicode TR35. There are several classes of formatting including non-scientific, scientific, rules based (for spelling and ordinal formats), compact formats that display 1k rather than 1,000 and so on. See Cldr.Number.to_string/2 for specific formatting options. Non-Scientific Notation Formatting The following description applies to formats that do not use scientific notation or significant digits: If the number of actual integer digits exceeds the maximum integer digits, then only the least significant digits are shown. For example, 1997 is formatted as &quot;97&quot; if the maximum integer digits is set to 2. If the number of actual integer digits is less than the minimum integer digits, then leading zeros are added. For example, 1997 is formatted as &quot;01997&quot; if the minimum integer digits is set to 5. If the number of actual fraction digits exceeds the maximum fraction digits, then half-even rounding it performed to the maximum fraction digits. For example, 0.125 is formatted as &quot;0.12&quot; if the maximum fraction digits is 2. This behavior can be changed by specifying a rounding increment and a rounding mode. If the number of actual fraction digits is less than the minimum fraction digits, then trailing zeros are added. For example, 0.125 is formatted as &quot;0.1250&quot; if the minimum fraction digits is set to 4. Trailing fractional zeros are not displayed if they occur j positions after the decimal, where j is less than the maximum fraction digits. For example, 0.10004 is formatted as &quot;0.1&quot; if the maximum fraction digits is four or less. Scientific Notation Formatting Numbers in scientific notation are expressed as the product of a mantissa and a power of ten, for example, 1234 can be expressed as 1.234 x 10^3. The mantissa is typically in the half-open interval [1.0, 10.0) or sometimes [0.0, 1.0), but it need not be. In a pattern, the exponent character immediately followed by one or more digit characters indicates scientific notation. Example: &quot;0.###E0&quot; formats the number 1234 as &quot;1.234E3&quot;. The number of digit characters after the exponent character gives the minimum exponent digit count. There is no maximum. Negative exponents are formatted using the localized minus sign, not the prefix and suffix from the pattern. This allows patterns such as &quot;0.###E0 m/s&quot;. To prefix positive exponents with a localized plus sign, specify &#39;+&#39; between the exponent and the digits: &quot;0.###E+0&quot; will produce formats &quot;1E+1&quot;, &quot;1E+0&quot;, &quot;1E-1&quot;, and so on. (In localized patterns, use the localized plus sign rather than &#39;+&#39;.) The minimum number of integer digits is achieved by adjusting the exponent. Example: 0.00123 formatted with &quot;00.###E0&quot; yields &quot;12.3E-4&quot;. This only happens if there is no maximum number of integer digits. If there is a maximum, then the minimum number of integer digits is fixed at one. The maximum number of integer digits, if present, specifies the exponent grouping. The most common use of this is to generate engineering notation, in which the exponent is a multiple of three, for example, &quot;##0.###E0&quot;. The number 12345 is formatted using &quot;##0.####E0&quot; as &quot;12.345E3&quot;. When using scientific notation, the formatter controls the digit counts using significant digits logic. The maximum number of significant digits limits the total number of integer and fraction digits that will be shown in the mantissa; it does not affect parsing. For example, 12345 formatted with &quot;##0.##E0&quot; is &quot;12.3E3&quot;. Exponential patterns may not contain grouping separators. Significant Digits There are two ways of controlling how many digits are shows: (a) significant digits counts, or (b) integer and fraction digit counts. Integer and fraction digit counts are described above. When a formatter is using significant digits counts, it uses however many integer and fraction digits are required to display the specified number of significant digits. It may ignore min/max integer/fraction digits, or it may use them to the extent possible."},{"ref":"Money.Cldr.Number.html#to_approx_string/2","title":"Money.Cldr.Number.to_approx_string/2","type":"function","doc":"Formats a number and applies the :approximately format for a locale and number system. Arguments number is an integer, float or Decimal to be formatted options is a keyword list defining how the number is to be formatted. See Cldr.Number.to_string/3 for a description of the available options. Example iex&gt; Money.Cldr.Number.to_approx_string 1234 {:ok, &quot;~1,234&quot;}"},{"ref":"Money.Cldr.Number.html#to_at_least_string/2","title":"Money.Cldr.Number.to_at_least_string/2","type":"function","doc":"Formats a number and applies the :at_least format for a locale and number system. Arguments number is an integer, float or Decimal to be formatted options is a keyword list defining how the number is to be formatted. See Money.Cldr.Number.to_string/2 for a description of the available options. Example iex&gt; Money.Cldr.Number.to_at_least_string 1234 {:ok, &quot;1,234+&quot;}"},{"ref":"Money.Cldr.Number.html#to_at_most_string/2","title":"Money.Cldr.Number.to_at_most_string/2","type":"function","doc":"Formats a number and applies the :at_most format for a locale and number system. Arguments number is an integer, float or Decimal to be formatted options is a keyword list defining how the number is to be formatted. See Cldr.Number.to_string/3 for a description of the available options. Example iex&gt; Money.Cldr.Number.to_at_most_string 1234 {:ok, &quot;≤1,234&quot;}"},{"ref":"Money.Cldr.Number.html#to_range_string/2","title":"Money.Cldr.Number.to_range_string/2","type":"function","doc":"Formats the first and last numbers of a range and applies the :range format for a locale and number system. Arguments number is an integer, float or Decimal to be formatted options is a keyword list defining how the number is to be formatted. See Cldr.Number.to_string/3 for a description of the available options. Example iex&gt; Money.Cldr.Number.to_range_string 1234..5678 {:ok, &quot;1,234–5,678&quot;}"},{"ref":"Money.Cldr.Number.html#to_string/2","title":"Money.Cldr.Number.to_string/2","type":"function","doc":"Returns a number formatted into a string according to a format pattern and options. Arguments number is an integer, float or Decimal to be formatted options is a keyword list defining how the number is to be formatted. Options format: the format style or a format string defining how the number is formatted. See Cldr.Number.Format for how format strings can be constructed. See Cldr.Number.Format.format_styles_for/3 to return available format styles for a locale. The default format is :standard. If :format is set to :long or :short then the formatting depends on whether :currency is specified. If not specified then the number is formatted as :decimal_long or :decimal_short. If :currency is specified the number is formatted as :currency_long or :currency_short and :fractional_digits is set to 0 as a default. :format may also be a format defined by CLDR&#39;s Rules Based Number Formats (RBNF). Further information is found in the module Cldr.Rbnf. The most commonly used formats in this category are to spell out the number in a the locales language. The applicable formats are :spellout, :spellout_year, :ordinal. A number can also be formatted as roman numbers by using the format :roman or :roman_lower. currency: is the currency for which the number is formatted. For available currencies see Cldr.Currency.known_currencies/0. This option is required if :format is set to :currency. If currency is set and no :format is set, :format will be set to :currency as well. :cash: a boolean which indicates whether a number being formatted as a :currency is to be considered a cash value or not. Currencies can be rounded differently depending on whether :cash is true or false. *This option is deprecated in favour of currency_digits: :cash. :currency_digits indicates which of the rounding and digits should be used. The options are :accounting which is the default, :cash or :iso :rounding_mode: determines how a number is rounded to meet the precision of the format requested. The available rounding modes are :down, :half_up, :half_even, :ceiling, :floor, :half_down, :up. The default is :half_even. :number_system: determines which of the number systems for a locale should be used to define the separators and digits for the formatted number. If number_system is an atom then number_system is interpreted as a number system. If the :number_system is binary then it is interpreted as a number system name. See Cldr.Number.System.number_system_names_for/2. The default is :default. :locale: determines the locale in which the number is formatted. See Cldr.known_locale_names/0. The default isCldr.get_locale/0 which is the locale currently in affect for this Process and which is set by Cldr.put_locale/1. :fractional_digits is set to a positive integer value then the number will be rounded to that number of digits and displayed accordingly overriding settings that would be applied by default. For example, currencies have fractional digits defined reflecting each currencies minor unit. Setting :fractional_digits will override that setting. :minimum_grouping_digits overrides the CLDR definition of minimum grouping digits. For example in the locale es the number 1234 is formatted by default as 1345 because the locale defines the minimium_grouping_digits as 2. If minimum_grouping_digits: 1 is set as an option the number is formatting as 1.345. The :minimum_grouping_digits is added to the grouping defined by the number format. If the sum of these two digits is greater than the number of digits in the integer (or fractional) part of the number then no grouping is performed. Returns {:ok, string} or {:error, {exception, message}} Examples iex&gt; Money.Cldr.Number.to_string 12345 {:ok, &quot;12,345&quot;} iex&gt; Money.Cldr.Number.to_string 12345, locale: &quot;fr&quot; {:ok, &quot;12 345&quot;} iex&gt; Money.Cldr.Number.to_string 1345.32, currency: :EUR, locale: &quot;es&quot;, minimum_grouping_digits: 1 {:ok, &quot;1.345,32 €&quot;} iex&gt; Money.Cldr.Number.to_string 1345.32, currency: :EUR, locale: &quot;es&quot; {:ok, &quot;1345,32 €&quot;} iex&gt; Money.Cldr.Number.to_string 12345, locale: &quot;fr&quot;, currency: &quot;USD&quot; {:ok, &quot;12 345,00 $US&quot;} iex&gt; Money.Cldr.Number.to_string 12345, format: &quot;#E0&quot; {:ok, &quot;1.2345E4&quot;} iex&gt; Money.Cldr.Number.to_string 12345, format: :accounting, currency: &quot;THB&quot; {:ok, &quot;THB 12,345.00&quot;} iex&gt; Money.Cldr.Number.to_string -12345, format: :accounting, currency: &quot;THB&quot; {:ok, &quot;(THB 12,345.00)&quot;} iex&gt; Money.Cldr.Number.to_string 12345, format: :accounting, currency: &quot;THB&quot;, ...&gt; locale: &quot;th&quot; {:ok, &quot;฿12,345.00&quot;} iex&gt; Money.Cldr.Number.to_string 12345, format: :accounting, currency: &quot;THB&quot;, ...&gt; locale: &quot;th&quot;, number_system: :native {:ok, &quot;฿๑๒,๓๔๕.๐๐&quot;} iex&gt; Money.Cldr.Number.to_string 1244.30, format: :long {:ok, &quot;1 thousand&quot;} iex&gt; Money.Cldr.Number.to_string 1244.30, format: :long, currency: &quot;USD&quot; {:ok, &quot;1,244 US dollars&quot;} iex&gt; Money.Cldr.Number.to_string 1244.30, format: :short {:ok, &quot;1K&quot;} iex&gt; Money.Cldr.Number.to_string 1244.30, format: :short, currency: &quot;EUR&quot; {:ok, &quot;€1K&quot;} iex&gt; Money.Cldr.Number.to_string 1234, format: :spellout {:ok, &quot;one thousand two hundred thirty-four&quot;} iex&gt; Money.Cldr.Number.to_string 1234, format: :spellout_verbose {:ok, &quot;one thousand two hundred and thirty-four&quot;} iex&gt; Money.Cldr.Number.to_string 1989, format: :spellout_year {:ok, &quot;nineteen eighty-nine&quot;} iex&gt; Money.Cldr.Number.to_string 123, format: :ordinal {:ok, &quot;123rd&quot;} iex(4)&gt; Money.Cldr.Number.to_string 123, format: :roman {:ok, &quot;CXXIII&quot;} Errors An error tuple {:error, reason} will be returned if an error is detected. The two most likely causes of an error return are: A format cannot be compiled. In this case the error tuple will look like: iex&gt; Money.Cldr.Number.to_string(12345, format: &quot;0#&quot;) {:error, {Cldr.FormatCompileError, &quot;Decimal format compiler: syntax error before: \\&quot;#\\&quot;&quot;}} A currency was not specific for a format type of format: :currency or format: :accounting or any other format that specifies a currency symbol placeholder. In this case the error return looks like: iex&gt; Money.Cldr.Number.to_string(12345, format: :accounting) {:error, {Cldr.FormatError, &quot;currency format \\&quot;¤#,##0.00;(¤#,##0.00)\\&quot; requires that &quot; &lt;&gt; &quot;options[:currency] be specified&quot;}} The format style requested is not defined for the locale and number_system. This happens typically when the number system is :algorithmic rather than the more common :numeric. In this case the error return looks like: iex&gt; Money.Cldr.Number.to_string(1234, locale: &quot;he&quot;, number_system: &quot;hebr&quot;) {:error, {Cldr.UnknownFormatError, &quot;The locale \\&quot;he\\&quot; with number system :hebr does not define a format :standard.&quot;}}"},{"ref":"Money.Cldr.Number.html#to_string!/2","title":"Money.Cldr.Number.to_string!/2","type":"function","doc":"Same as the execution of to_string/2 but raises an exception if an error would be returned. Arguments number is an integer, float or Decimal to be formatted options is a keyword list defining how the number is to be formatted. See Money.Cldr.Number.to_string/2 Returns a formatted number as a string or raises an exception Examples iex&gt; Money.Cldr.Number.to_string! 12345 &quot;12,345&quot; iex&gt; Money.Cldr.Number.to_string! 12345, locale: &quot;fr&quot; &quot;12 345&quot;"},{"ref":"Money.Cldr.Number.html#validate_number_system/2","title":"Money.Cldr.Number.validate_number_system/2","type":"function","doc":"Return a valid number system from a provided locale and number system name or type. The number system or number system type must be valid for the given locale. If a number system type is provided, the underlying number system is returned. Arguments locale is any valid locale name returned by Cldr.known_locale_names/1 or a Cldr.LanguageTag struct returned by Cldr.Locale.new!/2 system_name is any number system name returned by Cldr.known_number_systems/0 or a number system type returned by Cldr.known_number_system_types/0 Examples iex&gt; Money.Cldr.Number.validate_number_system &quot;en&quot;, :latn {:ok, :latn} iex&gt; Money.Cldr.Number.validate_number_system &quot;en&quot;, :default {:ok, :latn} iex&gt; Money.Cldr.Number.validate_number_system &quot;en&quot;, :unknown {:error, {Cldr.UnknownNumberSystemError, &quot;The number system :unknown is unknown&quot;}} iex&gt; Money.Cldr.Number.validate_number_system &quot;zz&quot;, :default {:error, {Cldr.UnknownLocaleError, &quot;The locale \\&quot;zz\\&quot; is not known.&quot;}}"},{"ref":"Money.Cldr.Number.Cardinal.html","title":"Money.Cldr.Number.Cardinal","type":"module","doc":"Implements cardinal plural rules for numbers."},{"ref":"Money.Cldr.Number.Cardinal.html#available_locale_names/0","title":"Money.Cldr.Number.Cardinal.available_locale_names/0","type":"function","doc":"The locale names for which plural rules are defined."},{"ref":"Money.Cldr.Number.Cardinal.html#known_locale_names/0","title":"Money.Cldr.Number.Cardinal.known_locale_names/0","type":"function","doc":"The configured locales for which plural rules are defined. Returns the intersection of Money.Cldr.known_locale_names/0 and the locales for which Cardinal plural rules are defined. There are many Cldr locales which don&#39;t have their own plural rules so this list is the intersection of Cldr&#39;s configured locales and those that have rules."},{"ref":"Money.Cldr.Number.Cardinal.html#plural_rule/3","title":"Money.Cldr.Number.Cardinal.plural_rule/3","type":"function","doc":"Return the plural key for a given number in a given locale Returns which plural key (:zero, :one, :two, :few, :many or :other) a given number fits into within the context of a given locale. Note that these key names should not be interpreted literally. For example, the key returned from Cldr.Number.Ordinal.plural_rule(0, &quot;en&quot;) is actually :other, not :zero. This key can then be used to format a number, date, time, unit, list or other content in a plural-sensitive way. Arguments number is any integer, float or Decimal locale is any locale returned by Cldr.Locale.new!/2 or any locale_name returned by Money.Cldr.known_locale_names/0 rounding is one of [:down, :up, :ceiling, :floor, :half_even, :half_up, :half_down]. The default is :half_even. Examples iex&gt; Money.Cldr.Number.Cardinal.plural_rule 0, &quot;fr&quot; :one iex&gt; Money.Cldr.Number.Cardinal.plural_rule 0, &quot;en&quot; :other"},{"ref":"Money.Cldr.Number.Cardinal.html#plural_rules/0","title":"Money.Cldr.Number.Cardinal.plural_rules/0","type":"function","doc":"Returns all the plural rules defined in CLDR."},{"ref":"Money.Cldr.Number.Cardinal.html#plural_rules_for/1","title":"Money.Cldr.Number.Cardinal.plural_rules_for/1","type":"function","doc":"Return the plural rules for a locale. Arguments locale is any locale returned by Money.Cldr.Locale.new!/1 or any locale_name returned by Money.Cldr.known_locale_names/0 The rules are returned in AST form after parsing."},{"ref":"Money.Cldr.Number.Cardinal.html#pluralize/3","title":"Money.Cldr.Number.Cardinal.pluralize/3","type":"function","doc":"Pluralize a number using cardinal plural rules and a substition map. Arguments number is an integer, float or Decimal locale is any locale returned by Money.Cldr.Locale.new!/1 or any locale_name returned by Money.Cldr.known_locale_names/0 substitutions is a map that maps plural keys to a string. The valid substitution keys are :zero, :one, :two, :few, :many and :other. See also Money.Cldr.Number.Cardinal.Cardinal.plural_rule/3. Examples iex&gt; Money.Cldr.Number.Cardinal.pluralize 1, &quot;en&quot;, %{one: &quot;one&quot;} &quot;one&quot; iex&gt; Money.Cldr.Number.Cardinal.pluralize 2, &quot;en&quot;, %{one: &quot;one&quot;} nil iex&gt; Money.Cldr.Number.Cardinal.pluralize 2, &quot;en&quot;, %{one: &quot;one&quot;, two: &quot;two&quot;, other: &quot;other&quot;} &quot;other&quot; iex&gt; Money.Cldr.Number.Cardinal.pluralize 22, &quot;en&quot;, %{one: &quot;one&quot;, two: &quot;two&quot;, other: &quot;other&quot;} &quot;other&quot; iex&gt; Money.Cldr.Number.Cardinal.pluralize Decimal.new(1), &quot;en&quot;, %{one: &quot;one&quot;} &quot;one&quot; iex&gt; Money.Cldr.Number.Cardinal.pluralize Decimal.new(2), &quot;en&quot;, %{one: &quot;one&quot;} nil iex&gt; Money.Cldr.Number.Cardinal.pluralize Decimal.new(2), &quot;en&quot;, %{one: &quot;one&quot;, two: &quot;two&quot;} nil iex&gt; Money.Cldr.Number.Cardinal.pluralize 1..10, &quot;ar&quot;, %{one: &quot;one&quot;, few: &quot;few&quot;, other: &quot;other&quot;} &quot;few&quot; iex&gt; Money.Cldr.Number.Cardinal.pluralize 1..10, &quot;en&quot;, %{one: &quot;one&quot;, few: &quot;few&quot;, other: &quot;other&quot;} &quot;other&quot;"},{"ref":"Money.Cldr.Number.Format.html","title":"Money.Cldr.Number.Format","type":"module","doc":"Functions to manage the collection of number patterns defined in Cldr. Number patterns affect how numbers are interpreted in a localized context. Here are some examples, based on the French locale. The &quot;.&quot; shows where the decimal point should go. The &quot;,&quot; shows where the thousands separator should go. A &quot;0&quot; indicates zero-padding: if the number is too short, a zero (in the locale&#39;s numeric set) will go there. A &quot;#&quot; indicates no padding: if the number is too short, nothing goes there. A &quot;¤&quot; shows where the currency sign will go. The following illustrates the effects of different patterns for the French locale, with the number &quot;1234.567&quot;. Notice how the pattern characters &#39;,&#39; and &#39;.&#39; are replaced by the characters appropriate for the locale. Number Pattern Examples PatternCurrencyText #,##0.##n/a1 234,57 #,##0.###n/a1 234,567 ###0.#####n/a1234,567 ###0.0000#n/a1234,5670 00000.0000n/a01234,5670 #,##0.00 ¤EUR1 234,57 € The number of # placeholder characters before the decimal do not matter, since no limit is placed on the maximum number of digits. There should, however, be at least one zero some place in the pattern. In currency formats, the number of digits after the decimal also do not matter, since the information in the supplemental data (see Supplemental Currency Data) is used to override the number of decimal places — and the rounding — according to the currency that is being formatted. That can be seen in the above chart, with the difference between Yen and Euro formatting. Details of the number formats are described in the Unicode documentation"},{"ref":"Money.Cldr.Number.Format.html#all_formats_for/1","title":"Money.Cldr.Number.Format.all_formats_for/1","type":"function","doc":"Returns the decimal formats defined for a given locale. Arguments locale is any valid locale name returned by Money.Cldr.known_locale_names/0 or a Cldr.LanguageTag struct returned by Money.Cldr.Locale.new!/1. The default is Money.Cldr.get_locale/0 Returns {:ok, map} where map is a map of decimal formats keyed by number system or {:error, {exception, message}}"},{"ref":"Money.Cldr.Number.Format.html#all_formats_for!/1","title":"Money.Cldr.Number.Format.all_formats_for!/1","type":"function","doc":"Returns the decimal formats defined for a given locale. Arguments locale is any valid locale name returned by Money.Cldr.known_locale_names/0 or a Cldr.LanguageTag struct returned by Money.Cldr.Locale.new!/1. The default is Money.Cldr.get_locale/0 Returns {:ok, map} where map is a map of decimal formats keyed by number system or raises an exception See Money.Cldr.Number.Format.Number.Format.all_formats_for/1 for further information."},{"ref":"Money.Cldr.Number.Format.html#currency_spacing/2","title":"Money.Cldr.Number.Format.currency_spacing/2","type":"function","doc":"Returns the currency space for a given locale and number system."},{"ref":"Money.Cldr.Number.Format.html#decimal_format_list/0","title":"Money.Cldr.Number.Format.decimal_format_list/0","type":"function","doc":"Returns the list of decimal formats in the configured locales including the list of locales configured for precompilation in config.exs. This function exists to allow the decimal formatter to precompile all the known formats at compile time. Example #=&gt; Money.Cldr.Number.Format.Format.decimal_format_list [&quot;#&quot;, &quot;#,##,##0%&quot;, &quot;#,##,##0.###&quot;, &quot;#,##,##0.00¤&quot;, &quot;#,##,##0.00¤;(#,##,##0.00¤)&quot;, &quot;#,##,##0 %&quot;, &quot;#,##0%&quot;, &quot;#,##0.###&quot;, &quot;#,##0.00 ¤&quot;, &quot;#,##0.00 ¤;(#,##0.00 ¤)&quot;, &quot;#,##0.00¤&quot;, &quot;#,##0.00¤;(#,##0.00¤)&quot;, &quot;#,##0 %&quot;, &quot;#0%&quot;, &quot;#0.######&quot;, &quot;#0.00 ¤&quot;, &quot;#E0&quot;, &quot;%#,##0&quot;, &quot;% #,##0&quot;, &quot;0&quot;, &quot;0.000000E+000&quot;, &quot;0000 M ¤&quot;, &quot;0000¤&quot;, &quot;000G ¤&quot;, &quot;000K ¤&quot;, &quot;000M ¤&quot;, &quot;000T ¤&quot;, &quot;000mM ¤&quot;, &quot;000m ¤&quot;, &quot;000 Bio&#39;.&#39; ¤&quot;, &quot;000 Bln ¤&quot;, &quot;000 Bn ¤&quot;, &quot;000 B ¤&quot;, &quot;000 E ¤&quot;, &quot;000 K ¤&quot;, &quot;000 MRD ¤&quot;, &quot;000 Md ¤&quot;, &quot;000 Mio&#39;.&#39; ¤&quot;, &quot;000 Mio ¤&quot;, &quot;000 Mld ¤&quot;, &quot;000 Mln ¤&quot;, &quot;000 Mn ¤&quot;, &quot;000 Mrd&#39;.&#39; ¤&quot;, &quot;000 Mrd ¤&quot;, &quot;000 Mr ¤&quot;, &quot;000 M ¤&quot;, &quot;000 NT ¤&quot;, &quot;000 N ¤&quot;, &quot;000 Tn ¤&quot;, &quot;000 Tr ¤&quot;, ...]"},{"ref":"Money.Cldr.Number.Format.html#decimal_format_list_for/1","title":"Money.Cldr.Number.Format.decimal_format_list_for/1","type":"function","doc":"Returns the list of decimal formats for a configured locale. Arguments locale is any valid locale name returned by Cldr.known_locale_names/0 or a Cldr.LanguageTag struct returned by Cldr.Locale.new!/1. The default is Cldr.get_locale/0 This function exists to allow the decimal formatter to precompile all the known formats at compile time. Its use is not otherwise recommended. Example iex&gt; Money.Cldr.Number.Format.decimal_format_list_for(&quot;en&quot;) {:ok, [&quot;#,##0%&quot;, &quot;#,##0.###&quot;, &quot;#E0&quot;, &quot;0 billion&quot;, &quot;0 million&quot;, &quot;0 thousand&quot;, &quot;0 trillion&quot;, &quot;00 billion&quot;, &quot;00 million&quot;, &quot;00 thousand&quot;, &quot;00 trillion&quot;, &quot;000 billion&quot;, &quot;000 million&quot;, &quot;000 thousand&quot;, &quot;000 trillion&quot;, &quot;000B&quot;, &quot;000K&quot;, &quot;000M&quot;, &quot;000T&quot;, &quot;00B&quot;, &quot;00K&quot;, &quot;00M&quot;, &quot;00T&quot;, &quot;0B&quot;, &quot;0K&quot;, &quot;0M&quot;, &quot;0T&quot;, &quot;¤#,##0.00&quot;, &quot;¤#,##0.00;(¤#,##0.00)&quot;, &quot;¤000B&quot;, &quot;¤000K&quot;, &quot;¤000M&quot;, &quot;¤000T&quot;, &quot;¤00B&quot;, &quot;¤00K&quot;, &quot;¤00M&quot;, &quot;¤00T&quot;, &quot;¤0B&quot;, &quot;¤0K&quot;, &quot;¤0M&quot;, &quot;¤0T&quot;]}"},{"ref":"Money.Cldr.Number.Format.html#default_grouping_for/1","title":"Money.Cldr.Number.Format.default_grouping_for/1","type":"function","doc":"Returns the default grouping for a locale as a map. Arguments locale is any valid locale name returned by Cldr.known_locale_names/0 or a Cldr.LanguageTag struct returned by Money.Cldr.Locale.new!/1. The default is Money.Cldr.get_locale/0 Returns {:ok, grouping} or {:error, {exception, message}} Examples iex&gt; Money.Cldr.Number.Format.default_grouping_for(&quot;en&quot;) {:ok, %{fraction: %{first: 0, rest: 0}, integer: %{first: 3, rest: 3}}}"},{"ref":"Money.Cldr.Number.Format.html#default_grouping_for!/1","title":"Money.Cldr.Number.Format.default_grouping_for!/1","type":"function","doc":"Returns the default grouping for a locale or raises on error. Arguments locale is any valid locale name returned by Money.Cldr.known_locale_names/0 or a Cldr.LanguageTag struct returned by Money.Cldr.Locale.new!/1. The default is Money.Cldr.get_locale/0 Returns grouping as a map or raises an exception Examples iex&gt; Money.Cldr.Number.Format.default_grouping_for!(&quot;en&quot;) %{fraction: %{first: 0, rest: 0}, integer: %{first: 3, rest: 3}}"},{"ref":"Money.Cldr.Number.Format.html#formats_for/2","title":"Money.Cldr.Number.Format.formats_for/2","type":"function","doc":"Return the predfined formats for a given locale and number_system. Arguments locale is any valid locale name returned by Money.Cldr.known_locale_names/0 or a Cldr.LanguageTag struct returned by Cldr.Locale.new!/1. The default is Money.Cldr.get_locale/0 number_system is any valid number system or number system type returned by Money.Cldr.Number.System.number_systems_for/1 Example Money.Cldr.Number.Format.formats_for &quot;fr&quot;, :native #=&gt; %Cldr.Number.Format{ accounting: &quot;#,##0.00 ¤;(#,##0.00 ¤)&quot;, currency: &quot;#,##0.00 ¤&quot;, percent: &quot;#,##0 %&quot;, scientific: &quot;#E0&quot;, standard: &quot;#,##0.###&quot; currency_short: [{&quot;1000&quot;, [one: &quot;0 k ¤&quot;, other: &quot;0 k ¤&quot;]}, {&quot;10000&quot;, [one: &quot;00 k ¤&quot;, other: &quot;00 k ¤&quot;]}, {&quot;100000&quot;, [one: &quot;000 k ¤&quot;, other: &quot;000 k ¤&quot;]}, {&quot;1000000&quot;, [one: &quot;0 M ¤&quot;, other: &quot;0 M ¤&quot;]}, {&quot;10000000&quot;, [one: &quot;00 M ¤&quot;, other: &quot;00 M ¤&quot;]}, {&quot;100000000&quot;, [one: &quot;000 M ¤&quot;, other: &quot;000 M ¤&quot;]}, {&quot;1000000000&quot;, [one: &quot;0 Md ¤&quot;, other: &quot;0 Md ¤&quot;]}, {&quot;10000000000&quot;, [one: &quot;00 Md ¤&quot;, other: &quot;00 Md ¤&quot;]}, {&quot;100000000000&quot;, [one: &quot;000 Md ¤&quot;, other: &quot;000 Md ¤&quot;]}, {&quot;1000000000000&quot;, [one: &quot;0 Bn ¤&quot;, other: &quot;0 Bn ¤&quot;]}, {&quot;10000000000000&quot;, [one: &quot;00 Bn ¤&quot;, other: &quot;00 Bn ¤&quot;]}, {&quot;100000000000000&quot;, [one: &quot;000 Bn ¤&quot;, other: &quot;000 Bn ¤&quot;]}], ... }"},{"ref":"Money.Cldr.Number.Format.html#formats_for!/2","title":"Money.Cldr.Number.Format.formats_for!/2","type":"function","doc":""},{"ref":"Money.Cldr.Number.Format.html#minimum_grouping_digits_for/1","title":"Money.Cldr.Number.Format.minimum_grouping_digits_for/1","type":"function","doc":"Returns the minimum grouping digits for a locale. Arguments locale is any valid locale name returned by Cldr.known_locale_names/0 or a Cldr.LanguageTag struct returned by Money.Cldr.Locale.new!/1. The default is Money.Cldr.get_locale/0 Returns {:ok, minumum_digits} or {:error, {exception, message}} Examples iex&gt; Money.Cldr.Number.Format.minimum_grouping_digits_for(&quot;en&quot;) {:ok, 1}"},{"ref":"Money.Cldr.Number.Format.html#minimum_grouping_digits_for!/1","title":"Money.Cldr.Number.Format.minimum_grouping_digits_for!/1","type":"function","doc":"Returns the minium grouping digits for a locale or raises on error. Arguments locale is any valid locale name returned by Money.Cldr.known_locale_names/0 or a Cldr.LanguageTag struct returned by Money.Cldr.Locale.new!/1. The default is Money.Cldr.get_locale/0 Returns minumum_digits or raises an exception Examples iex&gt; Money.Cldr.Number.Format.minimum_grouping_digits_for!(&quot;en&quot;) 1"},{"ref":"Money.Cldr.Number.Formatter.Decimal.html","title":"Money.Cldr.Number.Formatter.Decimal","type":"module","doc":""},{"ref":"Money.Cldr.Number.Formatter.Decimal.html#to_string/3","title":"Money.Cldr.Number.Formatter.Decimal.to_string/3","type":"function","doc":"Formats a number according to a decimal format string. Arguments number is an integer, float or Decimal format is a format string. See Money.Cldr.Number for further information. options is a map of options. See Money.Cldr.Number.to_string/2 for further information."},{"ref":"Money.Cldr.Number.Ordinal.html","title":"Money.Cldr.Number.Ordinal","type":"module","doc":"Implements ordinal plural rules for numbers."},{"ref":"Money.Cldr.Number.Ordinal.html#available_locale_names/0","title":"Money.Cldr.Number.Ordinal.available_locale_names/0","type":"function","doc":"The locale names for which plural rules are defined."},{"ref":"Money.Cldr.Number.Ordinal.html#known_locale_names/0","title":"Money.Cldr.Number.Ordinal.known_locale_names/0","type":"function","doc":"The configured locales for which plural rules are defined. Returns the intersection of Money.Cldr.known_locale_names/0 and the locales for which Ordinal plural rules are defined. There are many Cldr locales which don&#39;t have their own plural rules so this list is the intersection of Cldr&#39;s configured locales and those that have rules."},{"ref":"Money.Cldr.Number.Ordinal.html#plural_rule/3","title":"Money.Cldr.Number.Ordinal.plural_rule/3","type":"function","doc":"Return the plural key for a given number in a given locale Returns which plural key (:zero, :one, :two, :few, :many or :other) a given number fits into within the context of a given locale. Note that these key names should not be interpreted literally. For example, the key returned from Cldr.Number.Ordinal.plural_rule(0, &quot;en&quot;) is actually :other, not :zero. This key can then be used to format a number, date, time, unit, list or other content in a plural-sensitive way. Arguments number is any integer, float or Decimal locale is any locale returned by Cldr.Locale.new!/2 or any locale_name returned by Money.Cldr.known_locale_names/0 rounding is one of [:down, :up, :ceiling, :floor, :half_even, :half_up, :half_down]. The default is :half_even. Examples iex&gt; Money.Cldr.Number.Ordinal.plural_rule 0, &quot;fr&quot; :other iex&gt; Money.Cldr.Number.Ordinal.plural_rule 1, &quot;en&quot; :one"},{"ref":"Money.Cldr.Number.Ordinal.html#plural_rules/0","title":"Money.Cldr.Number.Ordinal.plural_rules/0","type":"function","doc":"Returns all the plural rules defined in CLDR."},{"ref":"Money.Cldr.Number.Ordinal.html#plural_rules_for/1","title":"Money.Cldr.Number.Ordinal.plural_rules_for/1","type":"function","doc":"Return the plural rules for a locale. Arguments locale is any locale returned by Money.Cldr.Locale.new!/1 or any locale_name returned by Money.Cldr.known_locale_names/0 The rules are returned in AST form after parsing."},{"ref":"Money.Cldr.Number.Ordinal.html#pluralize/3","title":"Money.Cldr.Number.Ordinal.pluralize/3","type":"function","doc":"Pluralize a number using ordinal plural rules and a substition map. Arguments number is an integer, float or Decimal or a Range.t{}. When a range, The is that in any usage, the start value is strictly less than the end value, and that no values are negative. Results for any cases that do not meet these criteria are undefined. locale is any locale returned by Money.Cldr.Locale.new!/1 or any locale_name returned by Money.Cldr.known_locale_names/0 substitutions is a map that maps plural keys to a string. The valid substitution keys are :zero, :one, :two, :few, :many and :other. See also Money.Cldr.Number.Ordinal.Ordinal.plural_rule/3. Examples iex&gt; Money.Cldr.Number.Ordinal.pluralize 1, &quot;en&quot;, %{one: &quot;one&quot;} &quot;one&quot; iex&gt; Money.Cldr.Number.Ordinal.pluralize 2, &quot;en&quot;, %{one: &quot;one&quot;} nil iex&gt; Money.Cldr.Number.Ordinal.pluralize 2, &quot;en&quot;, %{one: &quot;one&quot;, two: &quot;two&quot;} &quot;two&quot; iex&gt; Money.Cldr.Number.Ordinal.pluralize 22, &quot;en&quot;, %{one: &quot;one&quot;, two: &quot;two&quot;, other: &quot;other&quot;} &quot;two&quot; iex&gt; Money.Cldr.Number.Ordinal.pluralize Decimal.new(1), &quot;en&quot;, %{one: &quot;one&quot;} &quot;one&quot; iex&gt; Money.Cldr.Number.Ordinal.pluralize Decimal.new(2), &quot;en&quot;, %{one: &quot;one&quot;} nil iex&gt; Money.Cldr.Number.Ordinal.pluralize Decimal.new(2), &quot;en&quot;, %{one: &quot;one&quot;, two: &quot;two&quot;} &quot;two&quot; iex&gt; Money.Cldr.Number.Ordinal.pluralize 1..10, &quot;ar&quot;, %{one: &quot;one&quot;, few: &quot;few&quot;, other: &quot;other&quot;} &quot;other&quot; iex&gt; Money.Cldr.Number.Ordinal.pluralize 1..10, &quot;en&quot;, %{one: &quot;one&quot;, few: &quot;few&quot;, other: &quot;other&quot;} &quot;other&quot;"},{"ref":"Money.Cldr.Number.PluralRule.Range.html","title":"Money.Cldr.Number.PluralRule.Range","type":"module","doc":"Implements plural rules for ranges"},{"ref":"Money.Cldr.Number.PluralRule.Range.html#plural_rule/3","title":"Money.Cldr.Number.PluralRule.Range.plural_rule/3","type":"function","doc":"Returns a final plural type for a start-of-range plural type, an end-of-range plural type and a locale. Arguments first is a plural type for the start of a range last is a plural type for the end of a range locale is any Cldr.LanguageTag.t or a language name (not locale name) Example iex&gt; Money.Cldr.Number.PluralRule.Range.plural_rule :other, :few, &quot;ar&quot; :few"},{"ref":"Money.Cldr.Number.Symbol.html","title":"Money.Cldr.Number.Symbol","type":"module","doc":""},{"ref":"Money.Cldr.Number.Symbol.html#all_decimal_symbols/0","title":"Money.Cldr.Number.Symbol.all_decimal_symbols/0","type":"function","doc":"Returns a list of all decimal symbols defined by the locales configured in this backend as a list."},{"ref":"Money.Cldr.Number.Symbol.html#all_decimal_symbols_class/0","title":"Money.Cldr.Number.Symbol.all_decimal_symbols_class/0","type":"function","doc":"Returns a list of all decimal symbols defined by the locales configured in this backend as a string. This string can be used as a character class when builing a regular expression."},{"ref":"Money.Cldr.Number.Symbol.html#all_grouping_symbols/0","title":"Money.Cldr.Number.Symbol.all_grouping_symbols/0","type":"function","doc":"Returns a list of all grouping symbols defined by the locales configured in this backend as a list."},{"ref":"Money.Cldr.Number.Symbol.html#all_grouping_symbols_class/0","title":"Money.Cldr.Number.Symbol.all_grouping_symbols_class/0","type":"function","doc":"Returns a list of all grouping symbols defined by the locales configured in this backend as a string. This string can be used as a character class when builing a regular expression."},{"ref":"Money.Cldr.Number.Symbol.html#number_symbols_for/1","title":"Money.Cldr.Number.Symbol.number_symbols_for/1","type":"function","doc":"Returns a map of Cldr.Number.Symbol.t structs of the number symbols for each of the number systems of a locale. Options locale is any valid locale name returned by Money.Cldr.known_locale_names/0 or a Cldr.LanguageTag struct returned by Money.Cldr.Locale.new!/. The default is Money.Cldr.get_locale/0. Example: iex&gt; Money.Cldr.Number.Symbol.number_symbols_for(&quot;th&quot;) {:ok, %{ latn: %Cldr.Number.Symbol{ decimal: &quot;.&quot;, exponential: &quot;E&quot;, group: &quot;,&quot;, infinity: &quot;∞&quot;, list: &quot;;&quot;, minus_sign: &quot;-&quot;, nan: &quot;NaN&quot;, per_mille: &quot;‰&quot;, percent_sign: &quot;%&quot;, plus_sign: &quot;+&quot;, superscripting_exponent: &quot;×&quot;, time_separator: &quot;:&quot; }, thai: %Cldr.Number.Symbol{ decimal: &quot;.&quot;, exponential: &quot;E&quot;, group: &quot;,&quot;, infinity: &quot;∞&quot;, list: &quot;;&quot;, minus_sign: &quot;-&quot;, nan: &quot;NaN&quot;, per_mille: &quot;‰&quot;, percent_sign: &quot;%&quot;, plus_sign: &quot;+&quot;, superscripting_exponent: &quot;×&quot;, time_separator: &quot;:&quot; } }}"},{"ref":"Money.Cldr.Number.Symbol.html#number_symbols_for/2","title":"Money.Cldr.Number.Symbol.number_symbols_for/2","type":"function","doc":""},{"ref":"Money.Cldr.Number.System.html","title":"Money.Cldr.Number.System","type":"module","doc":""},{"ref":"Money.Cldr.Number.System.html#number_system_for/2","title":"Money.Cldr.Number.System.number_system_for/2","type":"function","doc":"Returns the actual number system from a number system type. locale is any valid locale name returned by Cldr.known_locale_names/0 or a Cldr.LanguageTag struct returned by Cldr.Locale.new!/1 system_name is any number system name returned by Cldr.known_number_systems/0 or a number system type returned by Cldr.known_number_system_types/0 This function will decode a number system type into the actual number system. If the number system provided can&#39;t be decoded it is returned as is. Examples iex&gt; Cldr.Number.System.number_system_for &quot;th&quot;, :latn, TestBackend.Cldr {:ok, %{digits: &quot;0123456789&quot;, type: :numeric}} iex&gt; Cldr.Number.System.number_system_for &quot;en&quot;, :default, TestBackend.Cldr {:ok, %{digits: &quot;0123456789&quot;, type: :numeric}} iex&gt; Cldr.Number.System.number_system_for &quot;he&quot;, :traditional, TestBackend.Cldr {:ok, %{rules: &quot;hebrew&quot;, type: :algorithmic}} iex&gt; Cldr.Number.System.number_system_for &quot;en&quot;, :finance, TestBackend.Cldr { :error, {Cldr.UnknownNumberSystemError, &quot;The number system :finance is unknown for the locale named \\&quot;en\\&quot;. Valid number systems are %{default: :latn, native: :latn}&quot;} } iex&gt; Cldr.Number.System.number_system_for &quot;en&quot;, :native, TestBackend.Cldr {:ok, %{digits: &quot;0123456789&quot;, type: :numeric}}"},{"ref":"Money.Cldr.Number.System.html#number_system_names_for/1","title":"Money.Cldr.Number.System.number_system_names_for/1","type":"function","doc":"Returns the number systems available for a locale or {:error, message} if the locale is not known. locale is any valid locale name returned by Money.Cldr.known_locale_names/0 or a Cldr.LanguageTag struct returned by Money.Cldr.Locale.new!/1 Examples iex&gt; Money.Cldr.Number.System.number_system_names_for &quot;en&quot; {:ok, [:latn]} iex&gt; Money.Cldr.Number.System.number_system_names_for &quot;zz&quot; {:error, {Cldr.UnknownLocaleError, &quot;The locale \\&quot;zz\\&quot; is not known.&quot;}}"},{"ref":"Money.Cldr.Number.System.html#number_system_names_for!/1","title":"Money.Cldr.Number.System.number_system_names_for!/1","type":"function","doc":""},{"ref":"Money.Cldr.Number.System.html#number_system_types_for/1","title":"Money.Cldr.Number.System.number_system_types_for/1","type":"function","doc":""},{"ref":"Money.Cldr.Number.System.html#number_systems_for/1","title":"Money.Cldr.Number.System.number_systems_for/1","type":"function","doc":"Returns the number systems available for a locale or {:error, message} if the locale is not known. locale is any valid locale name returned by Money.Cldr.known_locale_names/0 or a Cldr.LanguageTag struct returned by Money.Cldr.Locale.new!/1 Examples iex&gt; Money.Cldr.Number.System.number_systems_for &quot;en&quot; {:ok, %{default: :latn, native: :latn}} iex&gt; Money.Cldr.Number.System.number_systems_for &quot;th&quot; {:ok, %{default: :latn, native: :thai}} iex&gt; Money.Cldr.Number.System.number_systems_for &quot;zz&quot; {:error, {Cldr.UnknownLocaleError, &quot;The locale \\&quot;zz\\&quot; is not known.&quot;}}"},{"ref":"Money.Cldr.Number.System.html#number_systems_for!/1","title":"Money.Cldr.Number.System.number_systems_for!/1","type":"function","doc":""},{"ref":"Money.Cldr.Number.System.html#number_systems_like/2","title":"Money.Cldr.Number.System.number_systems_like/2","type":"function","doc":""},{"ref":"Money.Cldr.Number.System.html#system_name_from/2","title":"Money.Cldr.Number.System.system_name_from/2","type":"function","doc":"Returns a number system name for a given locale and number system reference. system_name is any number system name returned by Money.Cldr.known_number_systems/0 or a number system type returned by Money.Cldr.known_number_system_types/0 locale is any valid locale name returned by Money.Cldr.known_locale_names/0 or a Cldr.LanguageTag struct returned by Money.Cldr.Locale.new!/1 Number systems can be references in one of two ways: As a number system type such as :default, :native, :traditional and :finance. This allows references to a number system for a locale in a consistent fashion for a given use WIth the number system name directly, such as :latn, :arab or any of the other 70 or so This function dereferences the supplied system_name and returns the actual system name. Examples ex&gt; Money.Cldr.Number.System.system_name_from(:default, &quot;en&quot;) {:ok, :latn} iex&gt; Money.Cldr.Number.System.system_name_from(&quot;latn&quot;, &quot;en&quot;) {:ok, :latn} iex&gt; Money.Cldr.Number.System.system_name_from(:native, &quot;en&quot;) {:ok, :latn} iex&gt; Money.Cldr.Number.System.system_name_from(:nope, &quot;en&quot;) { :error, {Cldr.UnknownNumberSystemError, &quot;The number system :nope is unknown&quot;} } Note that return value is not guaranteed to be a valid number system for the given locale as demonstrated in the third example."},{"ref":"Money.Cldr.Number.System.html#to_system/2","title":"Money.Cldr.Number.System.to_system/2","type":"function","doc":"Converts a number into the representation of a non-latin number system. This function converts numbers to a known number system only, it does not provide number formatting. number is a float, integer or Decimal system_name is any number system name returned by Cldr.known_number_systems/0 or a number system type returned by Cldr.known_number_system_types/0 There are two types of number systems in CLDR: :numeric in which the number system defines a direct mapping between the latin digits 0..9 into a the number system equivalent. In this case, to_system/2 invokes Cldr.Number.Transliterate.transliterate_digits/3 for the given number. :algorithmic in which the number system does not have the same structure as the :latn number system and therefore the conversion is done algorithmically. For CLDR the algorithm is implemented through Cldr.Rbnf rulesets. These rulesets are considered by CLDR to be less rigorous than the :numeric number systems and caution and testing for a specific use case is recommended. Examples iex&gt; Money.Cldr.Number.System.to_system 123456, :hebr {:ok, &quot;ק׳׳ת׳&quot;} iex&gt; Money.Cldr.Number.System.to_system 123, :hans {:ok, &quot;一百二十三&quot;} iex&gt; Money.Cldr.Number.System.to_system 123, :hant {:ok, &quot;一百二十三&quot;} iex&gt; Money.Cldr.Number.System.to_system 123, :hansfin {:ok, &quot;壹佰贰拾叁&quot;}"},{"ref":"Money.Cldr.Number.System.html#to_system!/2","title":"Money.Cldr.Number.System.to_system!/2","type":"function","doc":"Converts a number into the representation of a non-latin number system. Returns a converted string or raises on error. number is a float, integer or Decimal system_name is any number system name returned by Money.Cldr.known_number_systems/0 or a number system type returned by Money.Cldr.known_number_system_types/0 See Money.Cldr.Number.System.to_system/2 for further information. Examples iex&gt; Money.Cldr.Number.System.to_system! 123, :hans &quot;一百二十三&quot; iex&gt; Money.Cldr.Number.System.to_system! 123, :hant &quot;一百二十三&quot; iex&gt; Money.Cldr.Number.System.to_system! 123, :hansfin &quot;壹佰贰拾叁&quot;"},{"ref":"Money.Cldr.Number.Transliterate.html","title":"Money.Cldr.Number.Transliterate","type":"module","doc":"Transliteration for digits and separators. Transliterating a string is an expensive business. First the string has to be exploded into its component graphemes. Then for each grapheme we have to map to the equivalent in the other {locale, number_system}. Then we have to reassemble the string. Effort is made to short circuit where possible. Transliteration is not required for any {locale, number_system} that is the same as {&quot;en&quot;, &quot;latn&quot;} since the implementation uses this combination for the placeholders during formatting already. When short circuiting is possible (typically the en-* locales with &quot;latn&quot; number_system - the total number of short circuited locales is 211 of the 537 in CLDR) the overall number formatting is twice as fast than when formal transliteration is required. Configuring precompilation of digit transliterations This module includes Cldr.Number.Transliterate.transliterate_digits/3 which transliterates digits between number systems. For example from :arabic to :latn. Since generating a transliteration map is slow, pairs of transliterations can be configured so that the transliteration map is created at compile time and therefore speeding up transliteration at run time. To configure these transliteration pairs, add the to the use Cldr configuration in a backend module: defmodule MyApp.Cldr do use Cldr, precompile_transliterations: [{:latn, :arab}, {:arab, :thai}] end end Where each tuple in the list configures one transliteration map. In this example, two maps are configured: from :latn to :thai and from :arab to :thai. A list of configurable number systems is returned by Cldr.Number.System.systems_with_digits/0. If a transliteration is requested between two number pairs that have not been configured for precompilation, a warning is logged."},{"ref":"Money.Cldr.Number.Transliterate.html#transliterate/3","title":"Money.Cldr.Number.Transliterate.transliterate/3","type":"function","doc":"Transliterates from latin digits to another number system&#39;s digits. Transliterates the latin digits 0..9 to their equivalents in another number system. Also transliterates the decimal and grouping separators as well as the plus, minus and exponent symbols. Any other character in the string will be returned &quot;as is&quot;. Arguments sequence is the string to be transliterated. locale is any known locale, defaulting to Money.Cldr.get_locale/0. number_system is any known number system. If expressed as a string it is the actual name of a known number system. If epressed as an atom it is used as a key to look up a number system for the locale (the usual keys are :default and :native but :traditional and :finance are also part of the standard). See Money.Cldr.Number.System.number_systems_for/1 for a locale to see what number system types are defined. The default is :default. For available number systems see Cldr.Number.System.number_systems/0 and Money.Cldr.Number.System.number_systems_for/1. Also see Money.Cldr.Number.Symbol.number_symbols_for/1. Examples iex&gt; Money.Cldr.Number.Transliterate.transliterate(&quot;123556&quot;) &quot;123556&quot; iex&gt; Money.Cldr.Number.Transliterate.transliterate(&quot;123,556.000&quot;, &quot;fr&quot;, :default) &quot;123 556,000&quot; iex&gt; Money.Cldr.Number.Transliterate.transliterate(&quot;123556&quot;, &quot;th&quot;, :default) &quot;123556&quot; iex&gt; Money.Cldr.Number.Transliterate.transliterate(&quot;123556&quot;, &quot;th&quot;, &quot;thai&quot;) &quot;๑๒๓๕๕๖&quot; iex&gt; Money.Cldr.Number.Transliterate.transliterate(&quot;123556&quot;, &quot;th&quot;, :native) &quot;๑๒๓๕๕๖&quot; iex&gt; Money.Cldr.Number.Transliterate.transliterate(&quot;Some number is: 123556&quot;, &quot;th&quot;, &quot;thai&quot;) &quot;Some number is: ๑๒๓๕๕๖&quot;"},{"ref":"Money.Cldr.Number.Transliterate.html#transliterate!/3","title":"Money.Cldr.Number.Transliterate.transliterate!/3","type":"function","doc":""},{"ref":"Money.Cldr.Number.Transliterate.html#transliterate_digits/3","title":"Money.Cldr.Number.Transliterate.transliterate_digits/3","type":"function","doc":"Transliterates digits from one number system to another number system digits is binary representation of a number from_system and to_system are number system names in atom form. See Cldr.Number.System.systems_with_digits/0 for available number systems. Example iex&gt; Money.Cldr.Number.Transliterate.transliterate_digits &quot;٠١٢٣٤٥٦٧٨٩&quot;, :arab, :latn &quot;0123456789&quot;"},{"ref":"Money.Cldr.Rbnf.NumberSystem.html","title":"Money.Cldr.Rbnf.NumberSystem","type":"module","doc":"Functions to implement the number system rule-based-number-format rules of CLDR. These rules are defined only on the &quot;root&quot; locale and represent specialised number formatting. The standard public API for RBNF is via the Cldr.Number.to_string/2 function. The functions on this module are defined at compile time based upon the RBNF rules defined in the Unicode CLDR data repository. Available rules are identified by: iex&gt; Money.Cldr.Rbnf.NumberSystem.rule_sets(&quot;root&quot;) [:tamil, :roman_upper, :roman_lower, :hebrew_item, :hebrew_0_99, :hebrew, :greek_upper, :greek_lower, :georgian, :ethiopic_p1, :ethiopic, :cyrillic_lower_1_10, :cyrillic_lower, :armenian_upper, :armenian_lower] A rule can then be invoked on an available rule_set. For example iex&gt; Money.Cldr.Rbnf.NumberSystem.roman_upper(123, &quot;root&quot;) &quot;CXXIII&quot; This call is equivalent to the call through the public API of: iex&gt; Money.Cldr.Number.to_string(123, format: :roman) {:ok, &quot;CXXIII&quot;}"},{"ref":"Money.Cldr.Rbnf.NumberSystem.html#all_rule_sets/0","title":"Money.Cldr.Rbnf.NumberSystem.all_rule_sets/0","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.NumberSystem.html#armenian_lower/2","title":"Money.Cldr.Rbnf.NumberSystem.armenian_lower/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.NumberSystem.html#armenian_upper/2","title":"Money.Cldr.Rbnf.NumberSystem.armenian_upper/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.NumberSystem.html#cyrillic_lower/2","title":"Money.Cldr.Rbnf.NumberSystem.cyrillic_lower/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.NumberSystem.html#cyrillic_lower_1_10/2","title":"Money.Cldr.Rbnf.NumberSystem.cyrillic_lower_1_10/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.NumberSystem.html#cyrillic_lower_final/2","title":"Money.Cldr.Rbnf.NumberSystem.cyrillic_lower_final/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.NumberSystem.html#cyrillic_lower_post/2","title":"Money.Cldr.Rbnf.NumberSystem.cyrillic_lower_post/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.NumberSystem.html#cyrillic_lower_thousands/2","title":"Money.Cldr.Rbnf.NumberSystem.cyrillic_lower_thousands/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.NumberSystem.html#ethiopic/2","title":"Money.Cldr.Rbnf.NumberSystem.ethiopic/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.NumberSystem.html#ethiopic_p/2","title":"Money.Cldr.Rbnf.NumberSystem.ethiopic_p/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.NumberSystem.html#ethiopic_p1/2","title":"Money.Cldr.Rbnf.NumberSystem.ethiopic_p1/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.NumberSystem.html#ethiopic_p2/2","title":"Money.Cldr.Rbnf.NumberSystem.ethiopic_p2/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.NumberSystem.html#ethiopic_p3/2","title":"Money.Cldr.Rbnf.NumberSystem.ethiopic_p3/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.NumberSystem.html#georgian/2","title":"Money.Cldr.Rbnf.NumberSystem.georgian/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.NumberSystem.html#greek_lower/2","title":"Money.Cldr.Rbnf.NumberSystem.greek_lower/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.NumberSystem.html#greek_numeral_majuscules/2","title":"Money.Cldr.Rbnf.NumberSystem.greek_numeral_majuscules/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.NumberSystem.html#greek_numeral_minuscules/2","title":"Money.Cldr.Rbnf.NumberSystem.greek_numeral_minuscules/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.NumberSystem.html#greek_upper/2","title":"Money.Cldr.Rbnf.NumberSystem.greek_upper/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.NumberSystem.html#hebrew/2","title":"Money.Cldr.Rbnf.NumberSystem.hebrew/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.NumberSystem.html#hebrew_0_99/2","title":"Money.Cldr.Rbnf.NumberSystem.hebrew_0_99/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.NumberSystem.html#hebrew_item/2","title":"Money.Cldr.Rbnf.NumberSystem.hebrew_item/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.NumberSystem.html#hebrew_thousands/2","title":"Money.Cldr.Rbnf.NumberSystem.hebrew_thousands/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.NumberSystem.html#roman_lower/2","title":"Money.Cldr.Rbnf.NumberSystem.roman_lower/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.NumberSystem.html#roman_upper/2","title":"Money.Cldr.Rbnf.NumberSystem.roman_upper/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.NumberSystem.html#rule_sets/1","title":"Money.Cldr.Rbnf.NumberSystem.rule_sets/1","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.NumberSystem.html#tamil/2","title":"Money.Cldr.Rbnf.NumberSystem.tamil/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.NumberSystem.html#tamil_thousands/2","title":"Money.Cldr.Rbnf.NumberSystem.tamil_thousands/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Ordinal.html","title":"Money.Cldr.Rbnf.Ordinal","type":"module","doc":"Functions to implement the ordinal rule-based-number-format rules of CLDR. As CLDR notes, the data is incomplete or non-existent for many languages. It is considered complete for English however. The standard public API for RBNF is via the Cldr.Number.to_string/2 function. The functions on this module are defined at compile time based upon the RBNF rules defined in the Unicode CLDR data repository. Available rules are identified by: iex&gt; Money.Cldr.Rbnf.Ordinal.rule_sets(&quot;en&quot;) [:digits_ordinal] A rule can then be invoked on an available rule_set. For example iex&gt; Money.Cldr.Rbnf.Ordinal.digits_ordinal(123, &quot;en&quot;) &quot;123rd&quot; This call is equivalent to the call through the public API of: iex&gt; Money.Cldr.Number.to_string(123, format: :ordinal) {:ok, &quot;123rd&quot;}"},{"ref":"Money.Cldr.Rbnf.Ordinal.html#all_rule_sets/0","title":"Money.Cldr.Rbnf.Ordinal.all_rule_sets/0","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Ordinal.html#digits_ordinal/2","title":"Money.Cldr.Rbnf.Ordinal.digits_ordinal/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Ordinal.html#digits_ordinal_feminine/2","title":"Money.Cldr.Rbnf.Ordinal.digits_ordinal_feminine/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Ordinal.html#digits_ordinal_feminine_plural/2","title":"Money.Cldr.Rbnf.Ordinal.digits_ordinal_feminine_plural/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Ordinal.html#digits_ordinal_masculine/2","title":"Money.Cldr.Rbnf.Ordinal.digits_ordinal_masculine/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Ordinal.html#digits_ordinal_masculine_adjective/2","title":"Money.Cldr.Rbnf.Ordinal.digits_ordinal_masculine_adjective/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Ordinal.html#digits_ordinal_masculine_plural/2","title":"Money.Cldr.Rbnf.Ordinal.digits_ordinal_masculine_plural/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Ordinal.html#dord_femabbrev/2","title":"Money.Cldr.Rbnf.Ordinal.dord_femabbrev/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Ordinal.html#dord_mascabbrev/2","title":"Money.Cldr.Rbnf.Ordinal.dord_mascabbrev/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Ordinal.html#rule_sets/1","title":"Money.Cldr.Rbnf.Ordinal.rule_sets/1","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html","title":"Money.Cldr.Rbnf.Spellout","type":"module","doc":"Functions to implement the spellout rule-based-number-format rules of CLDR. As CLDR notes, the data is incomplete or non-existent for many languages. It is considered complete for English however. The standard public API for RBNF is via the Cldr.Number.to_string/2 function. The functions on this module are defined at compile time based upon the RBNF rules defined in the Unicode CLDR data repository. Available rules are identified by: iex&gt; Money.Cldr.Rbnf.Spellout.rule_sets(&quot;en&quot;) [:spellout_ordinal_verbose, :spellout_ordinal, :spellout_numbering_year, :spellout_numbering_verbose, :spellout_numbering, :spellout_cardinal_verbose, :spellout_cardinal] A rule can then be invoked on an available rule_set. For example: iex&gt; Money.Cldr.Rbnf.Spellout.spellout_ordinal(123, &quot;en&quot;) &quot;one hundred twenty-third&quot; This call is equivalent to the call through the public API of: iex&gt; Money.Cldr.Number.to_string(123, format: :spellout) {:ok, &quot;one hundred twenty-three&quot;}"},{"ref":"Money.Cldr.Rbnf.Spellout.html#2d_year/2","title":"Money.Cldr.Rbnf.Spellout.2d_year/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#all_rule_sets/0","title":"Money.Cldr.Rbnf.Spellout.all_rule_sets/0","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#and/2","title":"Money.Cldr.Rbnf.Spellout.and/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#and_o/2","title":"Money.Cldr.Rbnf.Spellout.and_o/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#cents_f/2","title":"Money.Cldr.Rbnf.Spellout.cents_f/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#cents_m/2","title":"Money.Cldr.Rbnf.Spellout.cents_m/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#cents_o/2","title":"Money.Cldr.Rbnf.Spellout.cents_o/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#commas/2","title":"Money.Cldr.Rbnf.Spellout.commas/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#commas_o/2","title":"Money.Cldr.Rbnf.Spellout.commas_o/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#et_un/2","title":"Money.Cldr.Rbnf.Spellout.et_un/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#et_une/2","title":"Money.Cldr.Rbnf.Spellout.et_une/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#et_unieme/2","title":"Money.Cldr.Rbnf.Spellout.et_unieme/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#fem_with_a/2","title":"Money.Cldr.Rbnf.Spellout.fem_with_a/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#fem_with_i/2","title":"Money.Cldr.Rbnf.Spellout.fem_with_i/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#fem_with_o/2","title":"Money.Cldr.Rbnf.Spellout.fem_with_o/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#lenient_parse/2","title":"Money.Cldr.Rbnf.Spellout.lenient_parse/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#mille_o/2","title":"Money.Cldr.Rbnf.Spellout.mille_o/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#msc_no_final/2","title":"Money.Cldr.Rbnf.Spellout.msc_no_final/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#msc_with_a/2","title":"Money.Cldr.Rbnf.Spellout.msc_with_a/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#msc_with_a_nofinal/2","title":"Money.Cldr.Rbnf.Spellout.msc_with_a_nofinal/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#msc_with_i/2","title":"Money.Cldr.Rbnf.Spellout.msc_with_i/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#msc_with_i_nofinal/2","title":"Money.Cldr.Rbnf.Spellout.msc_with_i_nofinal/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#msc_with_o/2","title":"Money.Cldr.Rbnf.Spellout.msc_with_o/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#msc_with_o_nofinal/2","title":"Money.Cldr.Rbnf.Spellout.msc_with_o_nofinal/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#msco_with_a/2","title":"Money.Cldr.Rbnf.Spellout.msco_with_a/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#msco_with_i/2","title":"Money.Cldr.Rbnf.Spellout.msco_with_i/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#msco_with_o/2","title":"Money.Cldr.Rbnf.Spellout.msco_with_o/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#ordinal_esima/2","title":"Money.Cldr.Rbnf.Spellout.ordinal_esima/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#ordinal_esima_with_a/2","title":"Money.Cldr.Rbnf.Spellout.ordinal_esima_with_a/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#ordinal_esima_with_i/2","title":"Money.Cldr.Rbnf.Spellout.ordinal_esima_with_i/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#ordinal_esima_with_o/2","title":"Money.Cldr.Rbnf.Spellout.ordinal_esima_with_o/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#ordinal_esimo/2","title":"Money.Cldr.Rbnf.Spellout.ordinal_esimo/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#ordinal_esimo_with_a/2","title":"Money.Cldr.Rbnf.Spellout.ordinal_esimo_with_a/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#ordinal_esimo_with_i/2","title":"Money.Cldr.Rbnf.Spellout.ordinal_esimo_with_i/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#ordinal_esimo_with_o/2","title":"Money.Cldr.Rbnf.Spellout.ordinal_esimo_with_o/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#rule_sets/1","title":"Money.Cldr.Rbnf.Spellout.rule_sets/1","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#spellout_cardinal/2","title":"Money.Cldr.Rbnf.Spellout.spellout_cardinal/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#spellout_cardinal_feminine/2","title":"Money.Cldr.Rbnf.Spellout.spellout_cardinal_feminine/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#spellout_cardinal_masculine/2","title":"Money.Cldr.Rbnf.Spellout.spellout_cardinal_masculine/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#spellout_cardinal_n/2","title":"Money.Cldr.Rbnf.Spellout.spellout_cardinal_n/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#spellout_cardinal_neuter/2","title":"Money.Cldr.Rbnf.Spellout.spellout_cardinal_neuter/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#spellout_cardinal_r/2","title":"Money.Cldr.Rbnf.Spellout.spellout_cardinal_r/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#spellout_cardinal_s/2","title":"Money.Cldr.Rbnf.Spellout.spellout_cardinal_s/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#spellout_cardinal_verbose/2","title":"Money.Cldr.Rbnf.Spellout.spellout_cardinal_verbose/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#spellout_leading/2","title":"Money.Cldr.Rbnf.Spellout.spellout_leading/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#spellout_numbering/2","title":"Money.Cldr.Rbnf.Spellout.spellout_numbering/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#spellout_numbering_verbose/2","title":"Money.Cldr.Rbnf.Spellout.spellout_numbering_verbose/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#spellout_numbering_year/2","title":"Money.Cldr.Rbnf.Spellout.spellout_numbering_year/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#spellout_ordinal/2","title":"Money.Cldr.Rbnf.Spellout.spellout_ordinal/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#spellout_ordinal_feminine/2","title":"Money.Cldr.Rbnf.Spellout.spellout_ordinal_feminine/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#spellout_ordinal_feminine_plural/2","title":"Money.Cldr.Rbnf.Spellout.spellout_ordinal_feminine_plural/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#spellout_ordinal_masculine/2","title":"Money.Cldr.Rbnf.Spellout.spellout_ordinal_masculine/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#spellout_ordinal_masculine_adjective/2","title":"Money.Cldr.Rbnf.Spellout.spellout_ordinal_masculine_adjective/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#spellout_ordinal_masculine_plural/2","title":"Money.Cldr.Rbnf.Spellout.spellout_ordinal_masculine_plural/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#spellout_ordinal_n/2","title":"Money.Cldr.Rbnf.Spellout.spellout_ordinal_n/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#spellout_ordinal_r/2","title":"Money.Cldr.Rbnf.Spellout.spellout_ordinal_r/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#spellout_ordinal_s/2","title":"Money.Cldr.Rbnf.Spellout.spellout_ordinal_s/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#spellout_ordinal_verbose/2","title":"Money.Cldr.Rbnf.Spellout.spellout_ordinal_verbose/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#ste/2","title":"Money.Cldr.Rbnf.Spellout.ste/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#ste2/2","title":"Money.Cldr.Rbnf.Spellout.ste2/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#subcents_f/2","title":"Money.Cldr.Rbnf.Spellout.subcents_f/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#subcents_m/2","title":"Money.Cldr.Rbnf.Spellout.subcents_m/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#subcents_o/2","title":"Money.Cldr.Rbnf.Spellout.subcents_o/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#th/2","title":"Money.Cldr.Rbnf.Spellout.th/2","type":"function","doc":""},{"ref":"Money.Cldr.Rbnf.Spellout.html#tieth/2","title":"Money.Cldr.Rbnf.Spellout.tieth/2","type":"function","doc":""},{"ref":"Money.Currency.html","title":"Money.Currency","type":"module","doc":"Functions to return lists of known, historic and legal tender currencies."},{"ref":"Money.Currency.html#currency_for_code/1","title":"Money.Currency.currency_for_code/1","type":"function","doc":""},{"ref":"Money.Currency.html#known_current_currencies/0","title":"Money.Currency.known_current_currencies/0","type":"function","doc":"Returns the list of currently active ISO 4217 currency codes. Example: iex&gt; Money.Currency.known_current_currencies [:AED, :AFN, :ALL, :AMD, :ANG, :AOA, :ARS, :AUD, :AWG, :AZN, :BAM, :BBD, :BDT, :BGN, :BHD, :BIF, :BMD, :BND, :BOB, :BOV, :BRL, :BSD, :BTN, :BWP, :BYN, :BZD, :CAD, :CDF, :CHE, :CHF, :CHW, :CLF, :CLP, :CNY, :COP, :COU, :CRC, :CUC, :CUP, :CVE, :CZK, :DJF, :DKK, :DOP, :DZD, :EGP, :ERN, :ETB, :EUR, :FJD, :FKP, :GBP, :GEL, :GHS, :GIP, :GMD, :GNF, :GTQ, :GYD, :HKD, :HNL, :HRK, :HTG, :HUF, :IDR, :ILS, :INR, :IQD, :IRR, :ISK, :JMD, :JOD, :JPY, :KES, :KGS, :KHR, :KMF, :KPW, :KRW, :KWD, :KYD, :KZT, :LAK, :LBP, :LKR, :LRD, :LSL, :LYD, :MAD, :MDL, :MGA, :MKD, :MMK, :MNT, :MOP, :MRU, :MUR, :MVR, :MWK, :MXN, :MXV, :MYR, :MZN, :NAD, :NGN, :NIO, :NOK, :NPR, :NZD, :OMR, :PAB, :PEN, :PGK, :PHP, :PKR, :PLN, :PYG, :QAR, :RON, :RSD, :RUB, :RWF, :SAR, :SBD, :SCR, :SDG, :SEK, :SGD, :SHP, :SLL, :SOS, :SRD, :SSP, :STN, :SVC, :SYP, :SZL, :THB, :TJS, :TMT, :TND, :TOP, :TRY, :TTD, :TWD, :TZS, :UAH, :UGX, :USD, :USN, :UYI, :UYU, :UYW, :UZS, :VES, :VND, :VUV, :WST, :XAF, :XAG, :XAU, :XBA, :XBB, :XBC, :XBD, :XCD, :XDR, :XOF, :XPD, :XPF, :XPT, :XSU, :XTS, :XUA, :XXX, :YER, :ZAR, :ZMW, :ZWL]"},{"ref":"Money.Currency.html#known_historic_currencies/0","title":"Money.Currency.known_historic_currencies/0","type":"function","doc":"Returns the list of historic ISO 4217 currency codes. Example: iex&gt; Money.Currency.known_historic_currencies [:ADP, :AFA, :ALK, :AOK, :AON, :AOR, :ARA, :ARL, :ARM, :ARP, :ATS, :AZM, :BAD, :BAN, :BEC, :BEF, :BEL, :BGL, :BGM, :BGO, :BOL, :BOP, :BRB, :BRC, :BRE, :BRN, :BRR, :BRZ, :BUK, :BYB, :BYR, :CLE, :CNH, :CNX, :CSD, :CSK, :CYP, :DDM, :DEM, :ECS, :ECV, :EEK, :ESA, :ESB, :ESP, :FIM, :FRF, :GEK, :GHC, :GNS, :GQE, :GRD, :GWE, :GWP, :HRD, :IEP, :ILP, :ILR, :ISJ, :ITL, :KRH, :KRO, :LTL, :LTT, :LUC, :LUF, :LUL, :LVL, :LVR, :MAF, :MCF, :MDC, :MGF, :MKN, :MLF, :MRO, :MTL, :MTP, :MVP, :MXP, :MZE, :MZM, :NIC, :NLG, :PEI, :PES, :PLZ, :PTE, :RHD, :ROL, :RUR, :SDD, :SDP, :SIT, :SKK, :SRG, :STD, :SUR, :TJR, :TMM, :TPE, :TRL, :UAK, :UGS, :USS, :UYP, :VEB, :VEF, :VNN, :XEU, :XFO, :XFU, :XRE, :YDD, :YUD, :YUM, :YUN, :YUR, :ZAL, :ZMK, :ZRN, :ZRZ, :ZWD, :ZWR]"},{"ref":"Money.Currency.html#known_tender_currencies/0","title":"Money.Currency.known_tender_currencies/0","type":"function","doc":"Returns the list of legal tender ISO 4217 currency codes. Example: iex&gt; Money.Currency.known_tender_currencies [:ADP, :AED, :AFA, :AFN, :ALK, :ALL, :AMD, :ANG, :AOA, :AOK, :AON, :AOR, :ARA, :ARL, :ARM, :ARP, :ARS, :ATS, :AUD, :AWG, :AZM, :AZN, :BAD, :BAM, :BAN, :BBD, :BDT, :BEC, :BEF, :BEL, :BGL, :BGM, :BGN, :BGO, :BHD, :BIF, :BMD, :BND, :BOB, :BOL, :BOP, :BOV, :BRB, :BRC, :BRE, :BRL, :BRN, :BRR, :BRZ, :BSD, :BTN, :BUK, :BWP, :BYB, :BYN, :BYR, :BZD, :CAD, :CDF, :CHE, :CHF, :CHW, :CLE, :CLF, :CLP, :CNH, :CNX, :CNY, :COP, :COU, :CRC, :CSD, :CSK, :CUC, :CUP, :CVE, :CYP, :CZK, :DDM, :DEM, :DJF, :DKK, :DOP, :DZD, :ECS, :ECV, :EEK, :EGP, :ERN, :ESA, :ESB, :ESP, :ETB, :EUR, :FIM, :FJD, :FKP, :FRF, :GBP, :GEK, :GEL, :GHC, :GHS, :GIP, :GMD, :GNF, :GNS, :GQE, :GRD, :GTQ, :GWE, :GWP, :GYD, :HKD, :HNL, :HRD, :HRK, :HTG, :HUF, :IDR, :IEP, :ILP, :ILR, :ILS, :INR, :IQD, :IRR, :ISJ, :ISK, :ITL, :JMD, :JOD, :JPY, :KES, :KGS, :KHR, :KMF, :KPW, :KRH, :KRO, :KRW, :KWD, :KYD, :KZT, :LAK, :LBP, :LKR, :LRD, :LSL, :LTL, :LTT, :LUC, :LUF, :LUL, :LVL, :LVR, :LYD, :MAD, :MAF, :MCF, :MDC, :MDL, :MGA, :MGF, :MKD, :MKN, :MLF, :MMK, :MNT, :MOP, :MRO, :MRU, :MTL, :MTP, :MUR, :MVP, :MVR, :MWK, :MXN, :MXP, :MXV, :MYR, :MZE, :MZM, :MZN, :NAD, :NGN, :NIC, :NIO, :NLG, :NOK, :NPR, :NZD, :OMR, :PAB, :PEI, :PEN, :PES, :PGK, :PHP, :PKR, :PLN, :PLZ, :PTE, :PYG, :QAR, :RHD, :ROL, :RON, :RSD, :RUB, :RUR, :RWF, :SAR, :SBD, :SCR, :SDD, :SDG, :SDP, :SEK, :SGD, :SHP, :SIT, :SKK, :SLL, :SOS, :SRD, :SRG, :SSP, :STD, :STN, :SUR, :SVC, :SYP, :SZL, :THB, :TJR, :TJS, :TMM, :TMT, :TND, :TOP, :TPE, :TRL, :TRY, :TTD, :TWD, :TZS, :UAH, :UAK, :UGS, :UGX, :USD, :USN, :USS, :UYI, :UYP, :UYU, :UYW, :UZS, :VEB, :VEF, :VES, :VND, :VNN, :VUV, :WST, :XAF, :XAG, :XAU, :XBA, :XBB, :XBC, :XBD, :XCD, :XDR, :XEU, :XFO, :XFU, :XOF, :XPD, :XPF, :XPT, :XRE, :XSU, :XTS, :XUA, :XXX, :YDD, :YER, :YUD, :YUM, :YUN, :YUR, :ZAL, :ZAR, :ZMK, :ZMW, :ZRN, :ZRZ, :ZWD, :ZWL, :ZWR]"},{"ref":"Money.ExchangeRateError.html","title":"Money.ExchangeRateError","type":"exception","doc":""},{"ref":"Money.Financial.html","title":"Money.Financial","type":"module","doc":"A set of financial functions, primarily related to discounted cash flows. Some of the algorithms are from http://www.financeformulas.net"},{"ref":"Money.Financial.html#future_value/2","title":"Money.Financial.future_value/2","type":"function","doc":"Calculates the future value for a list of cash flows and an interest rate. flows is a list of tuples representing a cash flow. Each flow is represented as a tuple of the form {period, %Money{}} interest_rate is a float representation of an interest rate. For example, 12% would be represented as 0.12 Example iex&gt; Money.Financial.future_value([{4, Money.new(:USD, 10000)}, {5, Money.new(:USD, 10000)}, {6, Money.new(:USD, 10000)}], 0.13) #Money&lt;:USD, 34068.99999999999999999999999&gt; iex&gt; Money.Financial.future_value [{0, Money.new(:USD, 5000)},{1, Money.new(:USD, 2000)}], 0.12 #Money&lt;:USD, 7600.000000000000000000000000&gt;"},{"ref":"Money.Financial.html#future_value/3","title":"Money.Financial.future_value/3","type":"function","doc":"Calculates the future value for a present value, an interest rate and a number of periods. present_value is a %Money{} representation of the present value interest_rate is a float representation of an interest rate. For example, 12% would be represented as 0.12 periods in an integer number of periods Examples iex&gt; Money.Financial.future_value Money.new(:USD, 10000), 0.08, 1 #Money&lt;:USD, 10800.00&gt; iex&gt; Money.Financial.future_value Money.new(:USD, 10000), 0.04, 2 #Money&lt;:USD, 10816.0000&gt; iex&gt; Money.Financial.future_value Money.new(:USD, 10000), 0.02, 4 #Money&lt;:USD, 10824.32160000&gt;"},{"ref":"Money.Financial.html#interest_rate/3","title":"Money.Financial.interest_rate/3","type":"function","doc":"Calculates the effective interest rate for a given present value, a future value and a number of periods. present_value is a %Money{} representation of the present value future_value is a %Money{} representation of the future value periods is an integer number of a period Examples iex&gt; Money.Financial.interest_rate Money.new(:USD, 10000), Money.new(:USD, 10816), 2 #Decimal&lt;0.04&gt; iex&gt; Money.Financial.interest_rate Money.new(:USD, 10000), Money.new(:USD, &quot;10824.3216&quot;), 4 #Decimal&lt;0.02&gt;"},{"ref":"Money.Financial.html#internal_rate_of_return/1","title":"Money.Financial.internal_rate_of_return/1","type":"function","doc":"Calculates the interal rate of return for a given list of cash flows. flows is a list of tuples representing a cash flow. Each flow is represented as a tuple of the form {period, %Money{}}"},{"ref":"Money.Financial.html#net_present_value/2","title":"Money.Financial.net_present_value/2","type":"function","doc":"Calculates the net present value of an initial investment, a list of cash flows and an interest rate. flows is a list of tuples representing a cash flow. Each flow is represented as a tuple of the form {period, %Money{}} interest_rate is a float representation of an interest rate. For example, 12% would be represented as 0.12 investment is a %Money{} struct representing the initial investment Example iex&gt; flows = [{0, Money.new(:USD, 5000)},{1, Money.new(:USD, 2000)},{2, Money.new(:USD, 500)},{3, Money.new(:USD,10_000)}] iex&gt; Money.Financial.net_present_value flows, 0.08, Money.new(:USD, 100) #Money&lt;:USD, 15118.84367220444038002337042&gt; iex&gt; Money.Financial.net_present_value flows, 0.08 #Money&lt;:USD, 15218.84367220444038002337042&gt;"},{"ref":"Money.Financial.html#net_present_value/3","title":"Money.Financial.net_present_value/3","type":"function","doc":"Calculates the net present value of an initial investment, a recurring payment, an interest rate and a number of periods investment is a %Money{} struct representing the initial investment future_value is a %Money{} representation of the future value interest_rate is a float representation of an interest rate. For example, 12% would be represented as 0.12 periods in an integer number of a period Example iex&gt; Money.Financial.net_present_value Money.new(:USD, 10000), 0.13, 2 #Money&lt;:USD, 7831.466833737959119743127888&gt; iex&gt; Money.Financial.net_present_value Money.new(:USD, 10000), 0.13, 2, Money.new(:USD, 100) #Money&lt;:USD, 7731.466833737959119743127888&gt;"},{"ref":"Money.Financial.html#net_present_value/4","title":"Money.Financial.net_present_value/4","type":"function","doc":""},{"ref":"Money.Financial.html#payment/3","title":"Money.Financial.payment/3","type":"function","doc":"Calculates the payment for a given loan or annuity given a present value, an interest rate and a number of periods. present_value is a %Money{} representation of the present value interest_rate is a float representation of an interest rate. For example, 12% would be represented as 0.12 periods is an integer number of periods Example iex&gt; Money.Financial.payment Money.new(:USD, 100), 0.12, 20 #Money&lt;:USD, 13.38787800396606622792492299&gt;"},{"ref":"Money.Financial.html#periods/3","title":"Money.Financial.periods/3","type":"function","doc":"Calculates the number of periods between a present value and a future value with a given interest rate. present_value is a %Money{} representation of the present value future_value is a %Money{} representation of the future value interest_rate is a float representation of an interest rate. For example, 12% would be represented as 0.12 Example iex&gt; Money.Financial.periods Money.new(:USD, 1500), Money.new(:USD, 2000), 0.005 #Decimal&lt;57.68013595323872502502238648&gt;"},{"ref":"Money.Financial.html#present_value/2","title":"Money.Financial.present_value/2","type":"function","doc":"Calculates the present value for a list of cash flows and an interest rate. flows is a list of tuples representing a cash flow. Each flow is represented as a tuple of the form {period, %Money{}} interest_rate is a float representation of an interest rate. For example, 12% would be represented as 0.12 Example iex&gt; Money.Financial.present_value([{4, Money.new(:USD, 10000)}, {5, Money.new(:USD, 10000)}, {6, Money.new(:USD, 10000)}], 0.13) #Money&lt;:USD, 16363.97191111964880256655144&gt; iex&gt; Money.Financial.present_value [{0, Money.new(:USD, -1000)},{1, Money.new(:USD, -4000)}], 0.1 #Money&lt;:USD, -4636.363636363636363636363636&gt;"},{"ref":"Money.Financial.html#present_value/3","title":"Money.Financial.present_value/3","type":"function","doc":"Calculates the present value for future value, an interest rate and a number of periods future_value is a %Money{} representation of the future value interest_rate is a float representation of an interest rate. For example, 12% would be represented as 0.12 periods in an integer number of periods Examples iex&gt; Money.Financial.present_value Money.new(:USD, 100), 0.08, 2 #Money&lt;:USD, 85.73388203017832647462277092&gt; iex&gt; Money.Financial.present_value Money.new(:USD, 1000), 0.10, 20 #Money&lt;:USD, 148.6436280241436864020760472&gt;"},{"ref":"Money.Invalid.html","title":"Money.Invalid","type":"exception","doc":""},{"ref":"Money.InvalidAmountError.html","title":"Money.InvalidAmountError","type":"exception","doc":""},{"ref":"Money.Sigil.html","title":"Money.Sigil","type":"module","doc":""},{"ref":"Money.Sigil.html#sigil_M/2","title":"Money.Sigil.sigil_M/2","type":"function","doc":"Implements the sigil ~M for Money The lower case ~m variant does not exist as interpolation and excape characters are not useful for Money sigils. Example iex&gt; import Money.Sigil iex&gt; ~M[1000]usd #Money&lt;:USD, 1000&gt; iex&gt; ~M[1000.34]usd #Money&lt;:USD, 1000.34&gt;"},{"ref":"Money.Sigil.html#validate_currency!/1","title":"Money.Sigil.validate_currency!/1","type":"function","doc":""},{"ref":"Money.UnknownCurrencyError.html","title":"Money.UnknownCurrencyError","type":"exception","doc":""},{"ref":"Money.ExchangeRates.html","title":"Money.ExchangeRates","type":"behaviour","doc":"Implements a behaviour and functions to retrieve exchange rates from an exchange rate service. Configuration for the exchange rate service is defined in a Money.ExchangeRates.Config struct. A default configuration is returned by Money.ExchangeRates.default_config/0. The default configuration is: config :ex_money, exchange_rate_service: false, exchange_rates_retrieve_every: 300_000, api_module: Money.ExchangeRates.OpenExchangeRates, callback_module: Money.ExchangeRates.Callback, preload_historic_rates: nil log_failure: :warn, log_info: :info, log_success: nil These keys are are defined as follows: :exchange_rate_service is a boolean that determines whether to automatically start the exchange rate retrieval service. The default it false. :exchange_rates_retrieve_every defines how often the exchange rates are retrieved in milliseconds. The default is 5 minutes (300,000 milliseconds) :api_module identifies the module that does the retrieval of exchange rates. This is any module that implements the Money.ExchangeRates behaviour. The default is Money.ExchangeRates.OpenExchangeRates :callback_module defines a module that follows the Money.ExchangeRates.Callback behaviour whereby the function rates_retrieved/2 is invoked after every successful retrieval of exchange rates. The default is Money.ExchangeRates.Callback. :preload_historic_rates defines a date or a date range, that will be requested when the exchange rate service starts up. The date or date range should be specified as either a Date.t or a Date.Range.t or a tuple of {Date.t, Date.t} representing the from and to dates for the rates to be retrieved. The default is nil meaning no historic rates are preloaded. :log_failure defines the log level at which api retrieval errors are logged. The default is :warn :log_success defines the log level at which successful api retrieval notifications are logged. The default is nil which means no logging. :log_info defines the log level at which service startup messages are logged. The default is :info. :retriever_options is available for exchange rate retriever module developers as a place to add retriever-specific configuration information. This information should be added in the init/1 callback in the retriever module. See Money.ExchangeRates.OpenExchangeRates.init/1 for an example. Keys can also be configured to retrieve values from environment variables. This lookup is done at runtime to facilitate deployment strategies. If the value of a configuration key is {:system, &quot;some_string&quot;} then &quot;some_string&quot; is interpreted as an environment variable name which is passed to System.get_env/2. An example configuration might be: config :ex_money, exchange_rate_service: {:system, &quot;RATE_SERVICE&quot;}, exchange_rates_retrieve_every: {:system, &quot;RETRIEVE_EVERY&quot;}, Open Exchange Rates If you plan to use the provided Open Exchange Rates module to retrieve exchange rates then you should also provide the additional configuration key for app_id: config :ex_money, open_exchange_rates_app_id: &quot;your_app_id&quot; or configure it via environment variable: config :ex_money, open_exchange_rates_app_id: {:system, &quot;OPEN_EXCHANGE_RATES_APP_ID&quot;} The default exchange rate retrieval module is provided in Money.ExchangeRates.OpenExchangeRates which can be used as a example to implement your own retrieval module for other services. Managing the configuration at runtime During exchange rate service startup, the function init/1 is called on the configuration exchange rate retrieval module. This module is expected to return an updated configuration allowing a developer to customise how the configuration is to be managed. See the implementation at Money.ExchangeRates.OpenExchangeRates.init/1 for an example."},{"ref":"Money.ExchangeRates.html#config/0","title":"Money.ExchangeRates.config/0","type":"function","doc":"Returns the configuration for ex_money including the configuration merged from the configured exchange rates retriever module."},{"ref":"Money.ExchangeRates.html#c:decode_rates/1","title":"Money.ExchangeRates.decode_rates/1","type":"callback","doc":"Decode the body returned from the API request and return a map of rates. THe map of rates must have an upcased atom key representing an ISO 4217 currency code and the value must be a Decimal number."},{"ref":"Money.ExchangeRates.html#default_config/0","title":"Money.ExchangeRates.default_config/0","type":"function","doc":"Returns the default configuration for the exchange rates retriever."},{"ref":"Money.ExchangeRates.html#c:get_historic_rates/2","title":"Money.ExchangeRates.get_historic_rates/2","type":"callback","doc":"Invoked to return the historic exchange rates from the configured exchange rate retrieval service. config is an %Money.ExchangeRataes.Config{} struct Returns {:ok, map_of_historic_rates} or {:error, reason}"},{"ref":"Money.ExchangeRates.html#c:get_latest_rates/1","title":"Money.ExchangeRates.get_latest_rates/1","type":"callback","doc":"Invoked to return the latest exchange rates from the configured exchange rate retrieval service. config is an %Money.ExchangeRataes.Config{} struct Returns {:ok, map_of_rates} or {:error, reason}"},{"ref":"Money.ExchangeRates.html#historic_rates/1","title":"Money.ExchangeRates.historic_rates/1","type":"function","doc":"Return historic exchange rates. date is a date returned by Date.new/3 or any struct with the elements :year, :month and :day. Returns: {:ok, rates} if exchange rates are successfully retrieved. rates is a map of exchange rates. {:error, reason} if no exchange rates can be returned. Note; all dates are expected to be in the Calendar.ISO calendar This function looks up the historic exchange rates in a an ETS table called :exchange_rates. The actual retrieval of rates is requested through Money.ExchangeRates.Retriever.historic_rates/1."},{"ref":"Money.ExchangeRates.html#c:init/1","title":"Money.ExchangeRates.init/1","type":"callback","doc":"Given the default configuration, returns an updated configuration at runtime during exchange rates service startup. This callback is optional. If the callback is not defined, the default configuration returned by Money.ExchangeRates.default_config/0 is used. config is the configuration returned by Money.ExchangeRates.default_config/0 The callback is expected to return a %Money.ExchangeRates.Config.t() struct which may have been updated. The configuration key :retriever_options is available for any service-specific configuration."},{"ref":"Money.ExchangeRates.html#last_updated/0","title":"Money.ExchangeRates.last_updated/0","type":"function","doc":"Return the timestamp of the last successful retrieval of exchange rates or {:error, reason} if no timestamp is known. Example Money.ExchangeRates.last_updated #&gt; {:ok, %DateTime{calendar: Calendar.ISO, day: 20, hour: 12, microsecond: {731942, 6}, minute: 36, month: 11, second: 6, std_offset: 0, time_zone: &quot;Etc/UTC&quot;, utc_offset: 0, year: 2016, zone_abbr: &quot;UTC&quot;}}"},{"ref":"Money.ExchangeRates.html#latest_rates/0","title":"Money.ExchangeRates.latest_rates/0","type":"function","doc":"Return the latest exchange rates. Returns: {:ok, rates} if exchange rates are successfully retrieved. rates is a map of exchange rates. {:error, reason} if no exchange rates can be returned. This function looks up the latest exchange rates in a an ETS table called :exchange_rates. The actual retrieval of rates is requested through Money.ExchangeRates.Retriever.latest_rates/0."},{"ref":"Money.ExchangeRates.html#latest_rates_available?/0","title":"Money.ExchangeRates.latest_rates_available?/0","type":"function","doc":"Returns true if the latest exchange rates are available and false otherwise."},{"ref":"Money.ExchangeRates.html#t:t/0","title":"Money.ExchangeRates.t/0","type":"type","doc":""},{"ref":"Money.ExchangeRates.Cache.html","title":"Money.ExchangeRates.Cache","type":"behaviour","doc":"Defines a cache behaviour and default inplementation of a cache for exchange rates"},{"ref":"Money.ExchangeRates.Cache.html#cache/0","title":"Money.ExchangeRates.Cache.cache/0","type":"function","doc":""},{"ref":"Money.ExchangeRates.Cache.html#historic_rates/1","title":"Money.ExchangeRates.Cache.historic_rates/1","type":"function","doc":""},{"ref":"Money.ExchangeRates.Cache.html#c:historic_rates/1","title":"Money.ExchangeRates.Cache.historic_rates/1","type":"callback","doc":"Returns the exchange rates for a given date."},{"ref":"Money.ExchangeRates.Cache.html#c:init/0","title":"Money.ExchangeRates.Cache.init/0","type":"callback","doc":"Initialize the cache when the exchange rates retriever is started"},{"ref":"Money.ExchangeRates.Cache.html#latest_rates/0","title":"Money.ExchangeRates.Cache.latest_rates/0","type":"function","doc":""},{"ref":"Money.ExchangeRates.Cache.html#c:latest_rates/0","title":"Money.ExchangeRates.Cache.latest_rates/0","type":"callback","doc":"Retrieve the latest exchange rates from the cache."},{"ref":"Money.ExchangeRates.Cache.html#c:store_historic_rates/2","title":"Money.ExchangeRates.Cache.store_historic_rates/2","type":"callback","doc":"Store the historic exchange rates for a given date in the cache."},{"ref":"Money.ExchangeRates.Cache.html#c:store_latest_rates/2","title":"Money.ExchangeRates.Cache.store_latest_rates/2","type":"callback","doc":"Store the latest exchange rates in the cache."},{"ref":"Money.ExchangeRates.Cache.html#c:terminate/0","title":"Money.ExchangeRates.Cache.terminate/0","type":"callback","doc":"Terminate the cache when the retriver process stops normally"},{"ref":"Money.ExchangeRates.Cache.Dets.html","title":"Money.ExchangeRates.Cache.Dets","type":"module","doc":"Money.ExchangeRates.Cache implementation for :dets"},{"ref":"Money.ExchangeRates.Cache.Dets.html#get/1","title":"Money.ExchangeRates.Cache.Dets.get/1","type":"function","doc":""},{"ref":"Money.ExchangeRates.Cache.Dets.html#historic_rates/1","title":"Money.ExchangeRates.Cache.Dets.historic_rates/1","type":"function","doc":"Returns the exchange rates for a given date. Callback implementation for Money.ExchangeRates.Cache.historic_rates/1."},{"ref":"Money.ExchangeRates.Cache.Dets.html#init/0","title":"Money.ExchangeRates.Cache.Dets.init/0","type":"function","doc":"Initialize the cache when the exchange rates retriever is started Callback implementation for Money.ExchangeRates.Cache.init/0."},{"ref":"Money.ExchangeRates.Cache.Dets.html#last_updated/0","title":"Money.ExchangeRates.Cache.Dets.last_updated/0","type":"function","doc":""},{"ref":"Money.ExchangeRates.Cache.Dets.html#latest_rates/0","title":"Money.ExchangeRates.Cache.Dets.latest_rates/0","type":"function","doc":"Retrieve the latest exchange rates from the cache. Callback implementation for Money.ExchangeRates.Cache.latest_rates/0."},{"ref":"Money.ExchangeRates.Cache.Dets.html#put/2","title":"Money.ExchangeRates.Cache.Dets.put/2","type":"function","doc":""},{"ref":"Money.ExchangeRates.Cache.Dets.html#store_historic_rates/2","title":"Money.ExchangeRates.Cache.Dets.store_historic_rates/2","type":"function","doc":"Store the historic exchange rates for a given date in the cache. Callback implementation for Money.ExchangeRates.Cache.store_historic_rates/2."},{"ref":"Money.ExchangeRates.Cache.Dets.html#store_latest_rates/2","title":"Money.ExchangeRates.Cache.Dets.store_latest_rates/2","type":"function","doc":"Store the latest exchange rates in the cache. Callback implementation for Money.ExchangeRates.Cache.store_latest_rates/2."},{"ref":"Money.ExchangeRates.Cache.Dets.html#terminate/0","title":"Money.ExchangeRates.Cache.Dets.terminate/0","type":"function","doc":"Terminate the cache when the retriver process stops normally Callback implementation for Money.ExchangeRates.Cache.terminate/0."},{"ref":"Money.ExchangeRates.Cache.Ets.html","title":"Money.ExchangeRates.Cache.Ets","type":"module","doc":"Money.ExchangeRates.Cache implementation for :ets and :dets"},{"ref":"Money.ExchangeRates.Cache.Ets.html#get/1","title":"Money.ExchangeRates.Cache.Ets.get/1","type":"function","doc":""},{"ref":"Money.ExchangeRates.Cache.Ets.html#historic_rates/1","title":"Money.ExchangeRates.Cache.Ets.historic_rates/1","type":"function","doc":"Returns the exchange rates for a given date. Callback implementation for Money.ExchangeRates.Cache.historic_rates/1."},{"ref":"Money.ExchangeRates.Cache.Ets.html#init/0","title":"Money.ExchangeRates.Cache.Ets.init/0","type":"function","doc":"Initialize the cache when the exchange rates retriever is started Callback implementation for Money.ExchangeRates.Cache.init/0."},{"ref":"Money.ExchangeRates.Cache.Ets.html#last_updated/0","title":"Money.ExchangeRates.Cache.Ets.last_updated/0","type":"function","doc":""},{"ref":"Money.ExchangeRates.Cache.Ets.html#latest_rates/0","title":"Money.ExchangeRates.Cache.Ets.latest_rates/0","type":"function","doc":"Retrieve the latest exchange rates from the cache. Callback implementation for Money.ExchangeRates.Cache.latest_rates/0."},{"ref":"Money.ExchangeRates.Cache.Ets.html#put/2","title":"Money.ExchangeRates.Cache.Ets.put/2","type":"function","doc":""},{"ref":"Money.ExchangeRates.Cache.Ets.html#store_historic_rates/2","title":"Money.ExchangeRates.Cache.Ets.store_historic_rates/2","type":"function","doc":"Store the historic exchange rates for a given date in the cache. Callback implementation for Money.ExchangeRates.Cache.store_historic_rates/2."},{"ref":"Money.ExchangeRates.Cache.Ets.html#store_latest_rates/2","title":"Money.ExchangeRates.Cache.Ets.store_latest_rates/2","type":"function","doc":"Store the latest exchange rates in the cache. Callback implementation for Money.ExchangeRates.Cache.store_latest_rates/2."},{"ref":"Money.ExchangeRates.Cache.Ets.html#terminate/0","title":"Money.ExchangeRates.Cache.Ets.terminate/0","type":"function","doc":"Terminate the cache when the retriver process stops normally Callback implementation for Money.ExchangeRates.Cache.terminate/0."},{"ref":"Money.ExchangeRates.Cache.EtsDets.html","title":"Money.ExchangeRates.Cache.EtsDets","type":"module","doc":""},{"ref":"Money.ExchangeRates.Cache.EtsDets.html#define_common_functions/0","title":"Money.ExchangeRates.Cache.EtsDets.define_common_functions/0","type":"macro","doc":""},{"ref":"Money.ExchangeRates.Callback.html","title":"Money.ExchangeRates.Callback","type":"behaviour","doc":"Default exchange rates retrieval callback module. When exchange rates are successfully retrieved, the function latest_rates_retrieved/2 or historic_rates_retrieved/2 is called to perform any desired serialization or proocessing."},{"ref":"Money.ExchangeRates.Callback.html#historic_rates_retrieved/2","title":"Money.ExchangeRates.Callback.historic_rates_retrieved/2","type":"function","doc":"Callback function invoked when historic exchange rates are retrieved."},{"ref":"Money.ExchangeRates.Callback.html#c:historic_rates_retrieved/2","title":"Money.ExchangeRates.Callback.historic_rates_retrieved/2","type":"callback","doc":"Defines the behaviour to retrieve historic exchange rates from an external data source."},{"ref":"Money.ExchangeRates.Callback.html#latest_rates_retrieved/2","title":"Money.ExchangeRates.Callback.latest_rates_retrieved/2","type":"function","doc":"Callback function invoked when the latest exchange rates are retrieved."},{"ref":"Money.ExchangeRates.Callback.html#c:latest_rates_retrieved/2","title":"Money.ExchangeRates.Callback.latest_rates_retrieved/2","type":"callback","doc":"Defines the behaviour to retrieve the latest exchange rates from an external data source."},{"ref":"Money.ExchangeRates.Config.html","title":"Money.ExchangeRates.Config","type":"module","doc":""},{"ref":"Money.ExchangeRates.Config.html#t:t/0","title":"Money.ExchangeRates.Config.t/0","type":"type","doc":""},{"ref":"Money.ExchangeRates.OpenExchangeRates.html","title":"Money.ExchangeRates.OpenExchangeRates","type":"module","doc":"Implements the Money.ExchangeRates for the Open Exchange Rates service. Required configuration: The configuration key :open_exchange_rates_app_id should be set to your app_id. for example: config :ex_money, open_exchange_rates_app_id: &quot;your_app_id&quot; or configure it via environment variable: config :ex_money, open_exchange_rates_app_id: {:system, &quot;OPEN_EXCHANGE_RATES_APP_ID&quot;} It is also possible to configure an alternative base url for this service in case it changes in the future. For example: config :ex_money, open_exchange_rates_app_id: &quot;your_app_id&quot; open_exchange_rates_url: &quot;https://openexchangerates.org/alternative_api&quot;"},{"ref":"Money.ExchangeRates.OpenExchangeRates.html#decode_rates/1","title":"Money.ExchangeRates.OpenExchangeRates.decode_rates/1","type":"function","doc":"Decode the body returned from the API request and return a map of rates. THe map of rates must have an upcased atom key representing an ISO 4217 currency code and the value must be a Decimal number. Callback implementation for Money.ExchangeRates.decode_rates/1."},{"ref":"Money.ExchangeRates.OpenExchangeRates.html#get_historic_rates/2","title":"Money.ExchangeRates.OpenExchangeRates.get_historic_rates/2","type":"function","doc":"Retrieves the historic exchange rates from Open Exchange Rates site. date is a date returned by Date.new/3 or any struct with the elements :year, :month and :day. config is the retrieval configuration. When invoked from the exchange rates services this will be the config returned from Money.ExchangeRates.config/0 Returns: {:ok, rates} if the rates can be retrieved {:error, reason} if rates cannot be retrieved Typically this function is called by the exchange rates retrieval service although it can be called outside that context as required."},{"ref":"Money.ExchangeRates.OpenExchangeRates.html#get_latest_rates/1","title":"Money.ExchangeRates.OpenExchangeRates.get_latest_rates/1","type":"function","doc":"Retrieves the latest exchange rates from Open Exchange Rates site. config is the retrieval configuration. When invoked from the exchange rates services this will be the config returned from Money.ExchangeRates.config/0 Returns: {:ok, rates} if the rates can be retrieved {:error, reason} if rates cannot be retrieved Typically this function is called by the exchange rates retrieval service although it can be called outside that context as required."},{"ref":"Money.ExchangeRates.OpenExchangeRates.html#init/1","title":"Money.ExchangeRates.OpenExchangeRates.init/1","type":"function","doc":"Update the retriever configuration to include the requirements for Open Exchange Rates. This function is invoked when the exchange rate service starts up, just after the ets table :exchange_rates is created. default_config is the configuration returned by Money.ExchangeRates.default_config/0 Returns the configuration either unchanged or updated with additional configuration specific to this exchange rates retrieval module."},{"ref":"Money.ExchangeRates.Retriever.html","title":"Money.ExchangeRates.Retriever","type":"module","doc":"Implements a GenServer to retrieve exchange rates from a configured retrieveal module on a periodic or on demand basis. By default exchange rates are retrieved from Open Exchange Rates. The default period of execution is 5 minutes (300_000 milliseconds). The period of retrieval is configured in config.exs or the appropriate environment configuration. For example: config :ex_money, retrieve_every: 300_000"},{"ref":"Money.ExchangeRates.Retriever.html#child_spec/1","title":"Money.ExchangeRates.Retriever.child_spec/1","type":"function","doc":"Returns a specification to start this module under a supervisor. See Supervisor."},{"ref":"Money.ExchangeRates.Retriever.html#config/0","title":"Money.ExchangeRates.Retriever.config/0","type":"function","doc":"Return the current configuration of the Exchange Rates Retrieval service"},{"ref":"Money.ExchangeRates.Retriever.html#delete/0","title":"Money.ExchangeRates.Retriever.delete/0","type":"function","doc":"Delete the exchange rates retrieval service The service can be started again with start/1"},{"ref":"Money.ExchangeRates.Retriever.html#historic_rates/1","title":"Money.ExchangeRates.Retriever.historic_rates/1","type":"function","doc":"Forces retrieval of historic exchange rates for a single date date is a date returned by Date.new/3 or any struct with the elements :year, :month and :day or a Date.Range.t created by Date.range/2 that specifies a range of dates to retrieve Returns: {:ok, rates} if exchange rates request is successfully sent. {:error, reason} if the request cannot be send. Sends a message ot the exchange rate retrieval worker to request historic rates for a specified date or range be retrieved and stored. This function does not return exchange rates, for that see Money.ExchangeRates.latest_rates/0 or Money.ExchangeRates.historic_rates/1."},{"ref":"Money.ExchangeRates.Retriever.html#historic_rates/2","title":"Money.ExchangeRates.Retriever.historic_rates/2","type":"function","doc":"Forces retrieval of historic exchange rates for a range of dates from is a date returned by Date.new/3 or any struct with the elements :year, :month and :day. to is a date returned by Date.new/3 or any struct with the elements :year, :month and :day. Returns: {:ok, rates} if exchange rates request is successfully sent. {:error, reason} if the request cannot be send. Sends a message to the exchange rate retrieval process for each date in the range from..to to request historic rates be retrieved."},{"ref":"Money.ExchangeRates.Retriever.html#latest_rates/0","title":"Money.ExchangeRates.Retriever.latest_rates/0","type":"function","doc":"Forces retrieval of the latest exchange rates Sends a message ot the exchange rate retrieval worker to request current rates be retrieved and stored. Returns: {:ok, rates} if exchange rates request is successfully sent. {:error, reason} if the request cannot be send. This function does not return exchange rates, for that see Money.ExchangeRates.latest_rates/0 or Money.ExchangeRates.historic_rates/1."},{"ref":"Money.ExchangeRates.Retriever.html#reconfigure/1","title":"Money.ExchangeRates.Retriever.reconfigure/1","type":"function","doc":"Updated the configuration for the Exchange Rate Service"},{"ref":"Money.ExchangeRates.Retriever.html#restart/0","title":"Money.ExchangeRates.Retriever.restart/0","type":"function","doc":"Restart the exchange rates retrieval service"},{"ref":"Money.ExchangeRates.Retriever.html#retrieve_rates/2","title":"Money.ExchangeRates.Retriever.retrieve_rates/2","type":"function","doc":"Retrieve exchange rates from an external HTTP service. This function is primarily intended for use by an exchange rates api module."},{"ref":"Money.ExchangeRates.Retriever.html#start/1","title":"Money.ExchangeRates.Retriever.start/1","type":"function","doc":"Starts the exchange rates retrieval service"},{"ref":"Money.ExchangeRates.Retriever.html#stop/0","title":"Money.ExchangeRates.Retriever.stop/0","type":"function","doc":"Stop the exchange rates retrieval service. The service can be restarted with restart/0."},{"ref":"Money.ExchangeRates.Supervisor.html","title":"Money.ExchangeRates.Supervisor","type":"module","doc":"Functions to manage the starting, stopping, deleting and restarting of the Exchange Rates Retriever."},{"ref":"Money.ExchangeRates.Supervisor.html#child_spec/1","title":"Money.ExchangeRates.Supervisor.child_spec/1","type":"function","doc":"Returns a specification to start this module under a supervisor. See Supervisor."},{"ref":"Money.ExchangeRates.Supervisor.html#delete_retriever/0","title":"Money.ExchangeRates.Supervisor.delete_retriever/0","type":"function","doc":"Deleted the retriever child specification from the exchange rates supervisor. This is primarily of use if you want to change the configuration of the retriever after it is stopped and before it is restarted. In this situation the sequence of calls would be: iex&gt; Money.ExchangeRates.Retriever.stop iex&gt; Money.ExchangeRates.Retriever.delete iex&gt; Money.ExchangeRates.Retriever.start(config)"},{"ref":"Money.ExchangeRates.Supervisor.html#restart_retriever/0","title":"Money.ExchangeRates.Supervisor.restart_retriever/0","type":"function","doc":"Restarts a stopped retriever. See also Money.ExchangeRates.Retriever.stop/0"},{"ref":"Money.ExchangeRates.Supervisor.html#retriever_running?/0","title":"Money.ExchangeRates.Supervisor.retriever_running?/0","type":"function","doc":"Returns a boolean indicating of the retriever process is configured and running"},{"ref":"Money.ExchangeRates.Supervisor.html#retriever_status/0","title":"Money.ExchangeRates.Supervisor.retriever_status/0","type":"function","doc":"Returns the status of the exchange rates retriever. The returned value is one of: :running if the service is running. In this state the valid action is Money.ExchangeRates.Service.stop/0 :stopped if it is stopped. In this state the valid actions are Money.ExchangeRates.Supervisor.restart_retriever/0 or Money.ExchangeRates.Supervisor.delete_retriever/0 :not_started if it is not configured in the supervisor and is not running. In this state the only valid action is Money.ExchangeRates.Supervisor.start_retriever/1"},{"ref":"Money.ExchangeRates.Supervisor.html#start_link/0","title":"Money.ExchangeRates.Supervisor.start_link/0","type":"function","doc":"Starts the Exchange Rates supervisor and optionally starts the exchange rates retrieval service as well. Options :restart is a boolean value indicating if the supervisor is to be restarted. This is typically used to move the supervisor from its default position under the ex_money supervision tree to a different supervision tree. The default is false :start_retriever is a boolean indicating if the exchange rates retriever is to be started when the supervisor is started. The default is defined by the configuration key :auto_start_exchange_rate_service"},{"ref":"Money.ExchangeRates.Supervisor.html#start_link/1","title":"Money.ExchangeRates.Supervisor.start_link/1","type":"function","doc":""},{"ref":"Money.ExchangeRates.Supervisor.html#start_retriever/1","title":"Money.ExchangeRates.Supervisor.start_retriever/1","type":"function","doc":"Starts the exchange rates retriever Arguments config is a %Money.ExchangeRages.Config{} struct returned by Money.ExchangeRates.config/0 and adjusted as required. The default is Money.ExchangeRates.config/0"},{"ref":"Money.ExchangeRates.Supervisor.html#stop/0","title":"Money.ExchangeRates.Supervisor.stop/0","type":"function","doc":"Stop the Money.ExchangeRates.Supervisor. The Money.ExchangeRates.Supervisor is always started with ex_money starts even if the config key :auto_start_exchange_rates_service is set to false. In some instances an application may require the Money.ExchangeRates.Supervisor to be started under a different supervision tree. In this case it is required to call this function first before a new configuration is started. One use case is when the Exchange Rates service is configured with either an API module, a Callback module or a Cache module which uses Ecto and therefore its a requirement that Ecto is started first. See the README section on &quot;Using Ecto or other applications from within the callback module&quot; for an eanple of how to configure the supervisor in this case."},{"ref":"Money.ExchangeRates.Supervisor.html#stop_retriever/0","title":"Money.ExchangeRates.Supervisor.stop_retriever/0","type":"function","doc":"Stop the exchange rates retriever."},{"ref":"Money.Subscription.html","title":"Money.Subscription","type":"module","doc":"Provides functions to create, upgrade and downgrade subscriptions from one plan to another. Since moving from one plan to another may require prorating the payment stream at the point of transition, this module is introduced to provide a single point of calculation of the proration in order to give clear focus to the issues of calculating the carry-over amount or the carry-over period at the point of plan change. Defining a subscription A subscription records this current state and history of all plans assigned over time to a subscriber. The definition is deliberately minimal to simplify integration into applications that have a specific implementation of a subscription. A new subscription is created with Money.Subscription.new/3 which has the following attributes: plan which defines the initial plan for the subscription. This option is required. effective_date which determines the effective date of the inital plan. This option is required. options which include :created_at and :id with which a subscription may be annotated Changing a subscription plan Changing a subscription plan requires the following information be provided: A Subscription or the definition of the current plan The definition of the new plan The strategy for changing the plan which is either: to have the effective date of the new plan be after the current interval of the current plan To change the plan immediately in which case there will be a credit on the current plan which needs to be applied to the new plan. See Money.Subscription.change_plan/3 When the new plan is effective at the end of the current billing period The first strategy simply finishes the current billing period before the new plan is introduced and therefore no proration is required. This is the default strategy. When the new plan is effective immediately If the new plan is to be effective immediately then any credit balance remaining on the old plan needs to be applied to the new plan. There are two options of applying the credit: Reduce the billing amount of the first period of the new plan be the amount of the credit left on the old plan. This means that the billing amount for the first period of the new plan will be different (less) than the billing amount for subsequent periods on the new plan. Extend the first period of the new plan by the interval amount that can be funded by the credit amount left on the old plan. In the situation where the credit amount does not fully fund an integral interval the additional interval can be truncated or rounded up to the next integral period. Plan definition This module, and Money in general, does not provide a full billing or subscription solution - its focus is to support a reliable means of calcuating the accounting outcome of a plan change only. Therefore the plan definition required by Money.Subscription can be any Map.t that includes the following fields: interval which defines the time interval for a plan. The value can be one of day, week, month or year. interval_count which defines the number of intervals for the current plan interval. This must be a positive integer. price which is a Money.t representing the price of the plan to be paid each interval count. Billing in advance This module calculates all subscription changes on the basis that billing is done in advance. This primarily affects the calculation of plan credit when a plan changes. The assumption is that the period from the start of the current interval to the point of change has been consumed and therefore the credit is based upon that period of the plan that has not yet been consumed. If the calculation was based upon &quot;payment in arrears&quot; then the credit would actually be a debit since the part of the current period consumed has not yet been paid."},{"ref":"Money.Subscription.html#__struct__/0","title":"Money.Subscription.__struct__/0","type":"function","doc":"A struct defining a subscription :id any term that uniquely identifies this subscription :plans is a list of {change, plan} tuples that record the history of plans assigned to this subscription :created_at records the DateTime.t when the subscription was created"},{"ref":"Money.Subscription.html#cancel_pending_plan/2","title":"Money.Subscription.cancel_pending_plan/2","type":"function","doc":"Cancel a subscription&#39;s pending plan. A pending plan arise when a a Subscription.change_plan/3 has been executed but the effective date is in the future. Only one plan may be pending at any one time so that if Subscription.change_plan/3 is attemtped a second time an error tuple will be returned. Subscription.cancel_pending_plan/2 can be used to roll back the pending plan change. Arguments :subscription is any Money.Subscription.t :options is a Keyword.t Options :today is a Date.t that represents today. The default is Date.utc_today Returns An updated Money.Subscription.t which may or may not have had a pending plan. If it did have a pending plan that plan is deleted. If there was no pending plan then the subscription is returned unchanged."},{"ref":"Money.Subscription.html#change_plan/3","title":"Money.Subscription.change_plan/3","type":"function","doc":"Change plan from the current plan to a new plan. Arguments subscription_or_plan is either a Money.Subscription.t or Money.Subscription.Plan.t or a map with the same fields new_plan is a Money.Subscription.Plan.t or a map with at least the fields interval, interval_count and price current_interval_started is a Date.t or other map with the fields year, month, day and calendar options is a keyword list of options the define how the change is to be made Options :effective defines when the new plan comes into effect. The values are :immediately, a Date.t or :next_period. The default is :next_period. Note that the date applied in the case of :immediately is the date returned by Date.utc_today. :prorate which determines how to prorate the current plan into the new plan. The options are :price which will reduce the price of the first period of the new plan by the credit amount left on the old plan (this is the default). Or :period in which case the first period of the new plan is extended by the interval amount of the new plan that the credit on the old plan will fund. :round determines whether when prorating the :period it is truncated or rounded up to the next nearest full interval_count. Valid values are :down, :half_up, :half_even, :ceiling, :floor, :half_down, :up. The default is :up. :first_interval_started determines the anchor day for monthly billing. For example if a monthly plan starts on January 31st then the next period will start on February 28th (or 29th). The period following that should, however, be March 31st. If subscription_or_plan is a Money.Subscription.t then the :first_interval_started is automatically populated from the subscription. If :first_interval_started is nil then the date defined by :effective is used. Returns A Money.Subscription.Change.t with the following elements: :first_interval_starts which is the start date of the first interval for the new plan :first_billing_amount is the amount to be billed, net of any credit, at the :first_interval_starts :next_interval_starts is the start date of the next interval after the first intervalincluding any credit_days_applied :credit_amount is the amount of unconsumed credit of the current plan :credit_amount_applied is the amount of credit applied to the new plan. If the :prorate option is :price (the default) then :first_billing_amount is the plan :price reduced by the :credit_amount_applied. If the :prorate option is :period then the :first_billing_amount is the plan price and the :next_interval_date is extended by the :credit_days_applied instead. :credit_days_applied is the number of days credit applied to the first interval by adding days to the :first_interval_starts date. :credit_period_ends is the date on which any applied credit is consumed or nil :carry_forward is any amount of credit carried forward to a subsequent period. If non-zero, this amount is a negative Money.t. It is non-zero when the credit amount for the current plan is greater than the :price of the new plan. In this case the :first_billing_amount is zero. Returns {:ok, updated_subscription} or {:error, {exception, message}} Examples # Change at end of the current period so no proration iex&gt; current = Money.Subscription.Plan.new!(Money.new(:USD, 10), :month, 1) iex&gt; new = Money.Subscription.Plan.new!(Money.new(:USD, 10), :month, 3) iex&gt; Money.Subscription.change_plan current, new, current_interval_started: ~D[2018-01-01] {:ok, %Money.Subscription.Change{ carry_forward: Money.zero(:USD), credit_amount: Money.zero(:USD), credit_amount_applied: Money.zero(:USD), credit_days_applied: 0, credit_period_ends: nil, next_interval_starts: ~D[2018-05-01], first_billing_amount: Money.new(:USD, 10), first_interval_starts: ~D[2018-02-01] }} # Change during the current plan generates a credit amount iex&gt; current = Money.Subscription.Plan.new!(Money.new(:USD, 10), :month, 1) iex&gt; new = Money.Subscription.Plan.new!(Money.new(:USD, 10), :month, 3) iex&gt; Money.Subscription.change_plan current, new, current_interval_started: ~D[2018-01-01], effective: ~D[2018-01-15] {:ok, %Money.Subscription.Change{ carry_forward: Money.zero(:USD), credit_amount: Money.new(:USD, &quot;5.49&quot;), credit_amount_applied: Money.new(:USD, &quot;5.49&quot;), credit_days_applied: 0, credit_period_ends: nil, next_interval_starts: ~D[2018-04-15], first_billing_amount: Money.new(:USD, &quot;4.51&quot;), first_interval_starts: ~D[2018-01-15] }} # Change during the current plan generates a credit period iex&gt; current = Money.Subscription.Plan.new!(Money.new(:USD, 10), :month, 1) iex&gt; new = Money.Subscription.Plan.new!(Money.new(:USD, 10), :month, 3) iex&gt; Money.Subscription.change_plan current, new, current_interval_started: ~D[2018-01-01], effective: ~D[2018-01-15], prorate: :period {:ok, %Money.Subscription.Change{ carry_forward: Money.zero(:USD), credit_amount: Money.new(:USD, &quot;5.49&quot;), credit_amount_applied: Money.zero(:USD), credit_days_applied: 50, credit_period_ends: ~D[2018-03-05], next_interval_starts: ~D[2018-06-04], first_billing_amount: Money.new(:USD, 10), first_interval_starts: ~D[2018-01-15] }}"},{"ref":"Money.Subscription.html#change_plan!/3","title":"Money.Subscription.change_plan!/3","type":"function","doc":"Change plan from the current plan to a new plan. Retuns the plan or raises an exception on error. See Money.Subscription.change_plan/3 for the description of arguments, options and return."},{"ref":"Money.Subscription.html#current_interval_start_date/2","title":"Money.Subscription.current_interval_start_date/2","type":"function","doc":"Returns the first date of the current interval of a plan. Arguments :subscription_or_changeset is anyMoney.Subscription.t or a {Change.t, Plan.t} tuple :options is a keyword list of options Options :today is a Date.t that represents today. The default is Date.utc_today Returns The Date.t that is the first date of the current interval"},{"ref":"Money.Subscription.html#current_plan/2","title":"Money.Subscription.current_plan/2","type":"function","doc":"Retrieve the plan that is currently in affect. The plan in affect is not necessarily the first plan in the list. We may have upgraded plans to be in affect at some later time. Arguments subscription is a Money.Subscription.t or any map that provides the field :plans Returns The Money.Subscription.Plan.t that is the plan currently in affect or nil"},{"ref":"Money.Subscription.html#current_plan_start_date/1","title":"Money.Subscription.current_plan_start_date/1","type":"function","doc":"Returns the start date of the current plan. Arguments subscription is a Money.Subscription.t or any map that provides the field :plans Returns The start Date.t of the current plan"},{"ref":"Money.Subscription.html#days_remaining/4","title":"Money.Subscription.days_remaining/4","type":"function","doc":"Returns number of days remaining in a plan interval. Arguments plan is any Money.Subscription.Plan.t current_interval_started is a Date.t effective_date is a Date.t after the current_interval_started and before the end of the plan_days Returns The number of days remaining in a plan interval Examples iex&gt; plan = Money.Subscription.Plan.new! Money.new!(:USD, 100), :month, 1 iex&gt; Money.Subscription.days_remaining plan, ~D[2018-01-01], ~D[2018-01-02] 30 iex&gt; Money.Subscription.days_remaining plan, ~D[2018-02-01], ~D[2018-02-02] 27"},{"ref":"Money.Subscription.html#latest_plan/1","title":"Money.Subscription.latest_plan/1","type":"function","doc":"Returns the latest plan for a subscription. The latest plan may not be in affect since its start date may be in the future. Arguments subscription is a Money.Subscription.t or any map that provides the field :plans Returns The Money.Subscription.Plan.t that is the most recent plan - whether or not it is the currently active plan."},{"ref":"Money.Subscription.html#new/3","title":"Money.Subscription.new/3","type":"function","doc":"Creates a new subscription. Arguments plan is any Money.Subscription.Plan.t the defines the initial plan effective_date is a Date.t that represents the effective date of the initial plan. This defines the start of the first interval options is a keyword list of options Options :id is any term that an application can use to uniquely identify this subscription. It is not used in any function in this module. :created_at is a DateTime.t that records the timestamp when the subscription was created. The default is DateTime.utc_now/0 Returns {:ok, Money.Subscription.t} or {:error, {exception, message}}"},{"ref":"Money.Subscription.html#new!/3","title":"Money.Subscription.new!/3","type":"function","doc":"Creates a new subscription or raises an exception. Arguments plan is any Money.Subscription.Plan.t the defines the initial plan effective_date is a Date.t that represents the effective date of the initial plan. This defines the start of the first interval :options is a keyword list of options Options :id is any term that an application can use to uniquely identify this subscription. It is not used in any function in this module. :created_at is a DateTime.t that records the timestamp when the subscription was created. The default is DateTime.utc_now/0 Returns A Money.Subscription.t or raises an exception"},{"ref":"Money.Subscription.html#next_interval_starts/3","title":"Money.Subscription.next_interval_starts/3","type":"function","doc":"Returns the next interval start date for a plan. Arguments plan is any Money.Subscription.Plan.t :current_interval_started is the Date.t that represents the start of the current interval Returns The next interval start date as a Date.t. Example iex&gt; plan = Money.Subscription.Plan.new!(Money.new!(:USD, 100), :month) iex&gt; Money.Subscription.next_interval_starts(plan, ~D[2018-03-01]) ~D[2018-04-01] iex&gt; plan = Money.Subscription.Plan.new!(Money.new!(:USD, 100), :day, 30) iex&gt; Money.Subscription.next_interval_starts(plan, ~D[2018-02-01]) ~D[2018-03-03]"},{"ref":"Money.Subscription.html#plan_days/3","title":"Money.Subscription.plan_days/3","type":"function","doc":"Returns number of days in a plan interval. Arguments plan is any Money.Subscription.Plan.t current_interval_started is any Date.t Returns The number of days in a plan interval. Examples iex&gt; plan = Money.Subscription.Plan.new! Money.new!(:USD, 100), :month, 1 iex&gt; Money.Subscription.plan_days plan, ~D[2018-01-01] 31 iex&gt; Money.Subscription.plan_days plan, ~D[2018-02-01] 28 iex&gt; Money.Subscription.plan_days plan, ~D[2018-04-01] 30"},{"ref":"Money.Subscription.html#plan_pending?/2","title":"Money.Subscription.plan_pending?/2","type":"function","doc":"Returns a boolean indicating if there is a pending plan. A pending plan is one where the subscription has changed plans but the plan is not yet in effect. There can only be one pending plan. Arguments :subscription is any Money.Subscription.t :options is a keyword list of options Options :today is a Date.t that represents the effective date used to determine is there is a pending plan. The default is Date.utc_today/1. Returns Either true or false"},{"ref":"Money.Subscription.html#t:id/0","title":"Money.Subscription.id/0","type":"type","doc":"An id that uniquely identifies a subscription"},{"ref":"Money.Subscription.html#t:t/0","title":"Money.Subscription.t/0","type":"type","doc":"A Money.Subscription type"},{"ref":"Money.Subscription.Change.html","title":"Money.Subscription.Change","type":"module","doc":"Defines the structure of a plan changeset. :first_interval_starts which is the start date of the first interval for the new plan :first_billing_amount is the amount to be billed, net of any credit, at the :first_interval_starts :next_interval_starts is the start date of the next interval after the first interval including anycredit_days_applied*:credit_amountis the amount of unconsumed credit of the current plan *:credit_amount_appliedis the amount of credit applied to the new plan. If the:prorateoption is:price(the default) the:first_billing_amountis the plan:pricereduced by the:credit_amount_applied. If the:prorateoption is:periodthen the:first_billing_amountis the planprice and the :next_interval_date is extended by the :credit_days_applied instead. :credit_days_applied is the number of days credit applied to the first interval by adding days to the :first_interval_starts date. :credit_period_ends is the date on which any applied credit is consumed or nil :carry_forward is any amount of credit carried forward to a subsequent period. If non-zero this amount is a negative Money.t. It is non-zero when the credit amount for the current plan is greater than the price of the new plan. In this case the :first_billing_amount is zero."},{"ref":"Money.Subscription.Change.html#__struct__/0","title":"Money.Subscription.Change.__struct__/0","type":"function","doc":"A struct defining the changes between two plans."},{"ref":"Money.Subscription.Change.html#t:t/0","title":"Money.Subscription.Change.t/0","type":"type","doc":"A plan change record struct."},{"ref":"Money.Subscription.DateError.html","title":"Money.Subscription.DateError","type":"exception","doc":""},{"ref":"Money.Subscription.NoCurrentPlan.html","title":"Money.Subscription.NoCurrentPlan","type":"exception","doc":""},{"ref":"Money.Subscription.Plan.html","title":"Money.Subscription.Plan","type":"module","doc":"Defines a standard subscription plan data structure."},{"ref":"Money.Subscription.Plan.html#__struct__/0","title":"Money.Subscription.Plan.__struct__/0","type":"function","doc":"Defines the structure of a subscription plan."},{"ref":"Money.Subscription.Plan.html#new/3","title":"Money.Subscription.Plan.new/3","type":"function","doc":"Returns {:ok, Money.Subscription.Plan.t} or an {:error, reason} tuple. Arguments :price is any Money.t :interval is the period of the plan. The valid intervals are :day,:week,:monthor &#39;:year. :interval_count is an integer count of the number of :intervals of the plan. The default is 1 Returns A Money.Subscription.Plan.t Examples iex&gt; Money.Subscription.Plan.new Money.new(:USD, 100), :month, 1 {:ok, %Money.Subscription.Plan{ interval: :month, interval_count: 1, price: Money.new(:USD, 100) }} iex&gt; Money.Subscription.Plan.new Money.new(:USD, 100), :month {:ok, %Money.Subscription.Plan{ interval: :month, interval_count: 1, price: Money.new(:USD, 100) }} iex&gt; Money.Subscription.Plan.new Money.new(:USD, 100), :day, 30 {:ok, %Money.Subscription.Plan{ interval: :day, interval_count: 30, price: Money.new(:USD, 100) }} iex&gt; Money.Subscription.Plan.new 23, :day, 30 {:error, {Money.Invalid, &quot;Invalid subscription plan definition&quot;}}"},{"ref":"Money.Subscription.Plan.html#new!/3","title":"Money.Subscription.Plan.new!/3","type":"function","doc":"Returns {:ok, Money.Subscription.Plan.t} or raises an exception. Takes the same arguments as Money.Subscription.Plan.new/3. Example iex&gt; Money.Subscription.Plan.new! Money.new(:USD, 100), :day, 30 %Money.Subscription.Plan{ interval: :day, interval_count: 30, price: Money.new(:USD, 100) }"},{"ref":"Money.Subscription.Plan.html#t:interval/0","title":"Money.Subscription.Plan.interval/0","type":"type","doc":"A plan interval type."},{"ref":"Money.Subscription.Plan.html#t:interval_count/0","title":"Money.Subscription.Plan.interval_count/0","type":"type","doc":"A integer interval count for a plan."},{"ref":"Money.Subscription.Plan.html#t:t/0","title":"Money.Subscription.Plan.t/0","type":"type","doc":"A Subscription Plan"},{"ref":"Money.Subscription.PlanError.html","title":"Money.Subscription.PlanError","type":"exception","doc":""},{"ref":"Money.Subscription.PlanPending.html","title":"Money.Subscription.PlanPending","type":"exception","doc":""},{"ref":"readme.html","title":"Introduction to Money","type":"extras","doc":"Introduction to Money Money implements a set of functions to store, retrieve, convert and perform arithmetic on a %Money{} type that is composed of an ISO 4217 currency code and a currency amount. Money is opinionated in the interests of serving as a dependable library that can underpin accounting and financial applications. How is this opinion expressed? Money must always have both a amount and a currency code. The currency code must always be a valid ISO4217 code. Current and historical currency codes can be used. See the ISO Currency for more information. You can also identify the relevant codes by: Money.known_currencies/0 returns all the currency codes known to Money Money.known_current_currencies/0 returns the currency codes currently in use Money.known_historic_currencies/0 returns the list of historic currency codes Money.known_tender_currencies/0 returns the list of currencies known to be legal tender Money arithmetic can only be performed when both operands are of the same currency. Money amounts are represented as a Decimal. Money can be serialised to the database as a composite Postgres type that includes both the amount and the currency. For MySQL, money is serialized into a json column with the amount converted to a string to preserve precision since json does not have a decimal type. Serialization is entirely optional. All arithmetic functions work on a Decimal. No rounding occurs automatically (unless expressly called out for a function, as is the case for Money.split/2). Explicit rounding obeys the rounding rules for a given currency. The rounding rules are defined by the Unicode consortium in its CLDR repository as implemented by the hex package ex_cldr. These rules define the number of fractional digits for a currency and the rounding increment where appropriate. Money output string formatting output using the hex package ex_cldr that correctly rounds to the appropriate number of fractional digits and to the correct rounding increment for currencies that have minimum cash increments (like the Swiss Franc and Australian Dollar)"},{"ref":"readme.html#prerequisities","title":"Introduction to Money - Prerequisities","type":"extras","doc":"Money is supported on Elixir 1.6 and later only"},{"ref":"readme.html#exchange-rates-and-currency-conversion","title":"Introduction to Money - Exchange rates and currency conversion","type":"extras","doc":"Money includes a process to retrieve exchange rates on a periodic basis. These exchange rates can then be used to support currency conversion. This service is not started by default. If started it will attempt to retrieve exchange rates every 5 minutes by default. By default, exchange rates are retrieved from Open Exchange Rates however any module that conforms to the Money.ExchangeRates behaviour can be configured. An optional callback module can also be defined. This module defines a rates_retrieved/2 function that is invoked upon every successful retrieval of exchange rates. This might be used to serialize exchange rate to a data store or to stream rates to other applications or systems."},{"ref":"readme.html#configuration","title":"Introduction to Money - Configuration","type":"extras","doc":"Money provides a set of configuration keys to customize behaviour. The default configuration is: config :ex_money, exchange_rates_retrieve_every: 300_000, api_module: Money.ExchangeRates.OpenExchangeRates, callback_module: Money.ExchangeRates.Callback, exchange_rates_cache_module: Money.ExchangeRates.Cache.Ets, preload_historic_rates: nil, retriever_options: nil, log_failure: :warn, log_info: :info, log_success: nil, json_library: Cldr.Config.json_library(), default_cldr_backend: MyApp.Cldr Configuration key definitions :exchange_rates_retrieve_every defines how often the exchange rates are retrieved in milliseconds. The default is :never. An atom value is interpreted to mean that there should be no periodic retrieval. :api_module identifies the module that does the retrieval of exchange rates. This is any module that implements the Money.ExchangeRates behaviour. The default is Money.ExchangeRates.OpenExchangeRates. :exchange_rates_cache_module defines the module that provides an exchange rates cache. Any module that implements the Money.ExchangeRates.Cache behaviour. Two alternative strategies are provided: Money.ExchangeRates.Cache.Ets which is also the default. Money.ExchangeRates.Cache.Dets :preload_historic_rates defines a date or a date range that will be requested when the exchange rate service starts up. The date or date range should be specified as either a Date.t or a Date.Range.t or a tuple of {Date.t, Date.t} representing the from and to dates for the rates to be retrieved. The default is nil meaning no historic rates are preloaded. Some examples: callback_module defines a module that follows the Money.ExchangeRates.Callback behaviour whereby the function rates_retrieved/2 is invoked after every successful retrieval of exchange rates. The default is Money.ExchangeRates.Callback. log_failure defines the log level at which api retrieval errors are logged. The default is :warn. log_success defines the log level at which successful api retrieval notifications are logged. The default is nil which means no logging. log_info defines the log level at which service startup messages are logged. The default is info. :retriever_options is available for exchange rate retriever module developers as a place to add retriever-specific configuration information. This information should be added in the init/1 callback in the retriever module. See Money.ExchangeRates.OpenExchangeRates.init/1 for an example. :json_library determines which json library to be used for decoding. Two common options are Poison and Jason. The default is Cldr.Config.json_library/0 which is currently configured by default as Jason. :default_cldr_backend defines the Cldr backend module that is default for Money. See the ex_cldr documentation for further information on how to define this module. This is a required option. JSON library configuration Note that ex_money does not define a json library dependency and therefore it is the users responsibility to configure the required json library as a dependency in the application&#39;s mix.exs. The recommended library is jason which would be configured as: defp deps do [ {:jason, &quot;~&gt; 1.0&quot;}, ... ] end ex_money depends on ex_cldr which provides currency and localisation data. The default configuration of ex_money uses the default json_library from ex_cldr. This can be configured as follows in config.exs: config :ex_cldr, json_library: Jason In most cases this is not required since the presence of Jason (or Poison) is automatic. Configuring locales to support localised formatting Money uses ex_cldr and ex_cldr_numbers to support configuring locales and providing locale formatting. These packages are also the source of currency definitions, names, formats and so on. To use Cldr and therefore Money, a backend module must be defined. This module will host the Cldr data and public API used by Money. A simple example would be: defmodule MyApp.Cldr do use Cldr, locales: [&quot;en&quot;, &quot;fr&quot;, &quot;zh&quot;], default_locale: &quot;en&quot; end Preloading historic exchange rates The current implementation will call the api_module to retrieve the historic rates once for each date in the :preload_exchange_rates range. Some exchange rate services, like Open Exchange Rates, provides a bulk retrieval api that can retrieve multiple dates in a single call. However this endpoint is only available for premium subscribers and it is still charged on a &quot;per date retrieved&quot; basis. So while there is a network/performance/efficiency benefit there is no economic benefit. Please file an issue on github if implementing a bulk api is important to you. Some examples of configuring the :preload_exchange_rates key follow: preload_exchange_rates: ~D[2017-01-01] preload_exchange_rates: Date.range(~D[2017-01-01], ~D[2017-10-01]) preload_exchange_rates: {~D[2017-01-01], ~D[2017-10-01]} Open Exchange Rates configuration If you plan to use the provided Open Exchange Rates module to retrieve exchange rates then you should also provide the addition configuration key for app_id: config :ex_money, open_exchange_rates_app_id: &quot;your_app_id&quot; or configure it via environment variable, for example: config :ex_money, open_exchange_rates_app_id: {:system, &quot;OPEN_EXCHANGE_RATES_APP_ID&quot;} The default exchange rate retrieval module is provided in Money.ExchangeRates.OpenExchangeRates which can be used as a example to implement your own retrieval module for other services. Managing the configuration at runtime During exchange rate service startup, the function init/1 is called on the configured exchange rate retrieval module. This module is expected to return an updated configuration allowing a developer to customise how the configuration is to be managed. See the implementation at Money.ExchangeRates.OpenExchangeRates.init/1 for an example. To support runtime (re-)configuration the following functions are provided: Money.ExchangeRates.Retriever.config/0 returns the current configuration of the exchange rates retrieval service. Money.ExchangeRates.Retriever.stop/0 and Money.ExchangeRates.Retriever.start/0 stop and start the exchange rates retrieval service respectively. Money.ExchangeRates.Retriever.reconfigure/1 reconfigures the exchange rates retrieval service. It does not restart the service, the service remains active during the recongiguration. Using Environment Variables in the configuration Keys can also be configured to retrieve values from environment variables. This lookup is done at runtime to facilitate deployment strategies. If the value of a configuration key is {:system, &quot;some_string&quot;} then &quot;some_string&quot; is interpreted as an environment variable name which is passed to System.get_env/2. An example configuration might be: config :ex_money, auto_start_exchange_rate_service: {:system, &quot;RATE_SERVICE&quot;}, exchange_rates_retrieve_every: {:system, &quot;RETRIEVE_EVERY&quot;}, open_exchange_rates_app_id: {:system, &quot;OPEN_EXCHANGE_RATES_APP_ID&quot;} Note that the {:system, &quot;ENV KEY&quot;} approach is not currently supported for the :preload_historic_rates configuration key."},{"ref":"readme.html#the-exchange-rates-service-process-supervision-and-startup","title":"Introduction to Money - The Exchange rates service process supervision and startup","type":"extras","doc":"If the exchange rate service is configured to automatically start up (because the config key auto_start_exchange_rate_service is set to true) then a supervisor process named Money.ExchangeRates.Supervisor is started which in turns starts a child GenServer called Money.ExchangeRates.Retriever. It is Money.ExchangeRates.Retriever which will call the configured api_module to retrieve the rates. It is also responsible for calling the configured callback_module after a successfull retrieval. +-----------------+ | | +-------------+ +-----------+ | api_module |-&gt; External Service | | | |---&gt; | | | Supervisor |---&gt;| Retriever | +-----------------+ | | | |---&gt; +-----------------+ +-------------+ +-----------+ | | | callback_module | | | +-----------------+ On application start (or manual start if :auto_start_exchange_rate_service is set to false), Money.ExchangeRates.Retriever will schedule the first retrieval to be executed after immediately and then each :exchange_rates_retrieve_every milliseconds thereafter."},{"ref":"readme.html#using-ecto-or-other-applications-from-within-the-callback-module","title":"Introduction to Money - Using Ecto or other applications from within the callback module","type":"extras","doc":"If you provide your own callback module and that module depends on some other applications, like Ecto, already being started then automatically starting Money.ExchangeRates.Supervisor may not work since your Ecto.Repo is unlikely to have already been started. In this situation the appropriate way to configure the exchange rates retrieval service is the following: Set the configuration key auto_start_exchange_rate_service to false to prevent automatic startup of the service. Configure your api_module, callback_module and any other required configuration as appropriate In your client application code, add the Money.ExchangeRates.Supervisor to the children configuration of your application. For example, in an application that uses Ecto and where your callback_module is designed to save exchange rates to a database, your application may would look something like: defmodule Application do use Application def start(_type, _args) do import Supervisor.Spec children = [ # Start your repo first so that it is running before your # exchange rates callback module is called supervisor(MoneyTest.Repo, []), # Include the Money.ExchangeRates.Supervisor in your application&#39;s # supervision tree. This supervisor will start the child process # Money.ExchangeRates.Retriever. # Note the use of double `[]` around # the parameters which are required to ensure that the supervisor # is stopped before including in your supervisor tree. # The `start_retriever: true` is optional. The default value is `false`. supervisor(Money.ExchangeRates.Supervisor, [[restart: true, start_retriever: true]]) ] opts = [strategy: :one_for_one, name: Application.Supervisor] Supervisor.start_link(children, opts) end end"},{"ref":"readme.html#api-usage-examples","title":"Introduction to Money - API Usage Examples","type":"extras","doc":"Creating a %Money{} struct iex&gt; Money.new(:USD, 100) #Money&lt;:USD, 100&gt; iex&gt; Money.new(100, :USD) #Money&lt;:USD, 100&gt; iex&gt; Money.new(&quot;CHF&quot;, &quot;130.02&quot;) #Money&lt;:CHF, 130.02&gt; iex&gt; Money.new(&quot;thb&quot;, 11) #Money&lt;:THB, 11&gt; iex&gt; Money.new(&quot;1.000,99&quot;, :EUR, locale: &quot;de&quot;) #Money&lt;:EUR, 1000.99&gt; iex&gt; Money.parse(&quot;USD 100&quot;) #Money&lt;:USD, 100&gt; iex&gt; Money.parse(&quot;USD 100,00&quot;, locale: &quot;de&quot;) #Money&lt;:USD, 100.00&gt; The canonical representation of a currency code is an atom that is a valid ISO4217 currency code. The amount of a %Money{} is represented by a Decimal. Note that the amount and currency code arguments to Money.new/3 can be supplied in either order. Parsing money strings Money provides an ability to parse strings that contain a currency and an amount. The currency can be represented in different ways depending on the locale. See Money.parse/2 for further information. Some examples are: # These are the strings available for a given currency # and locale that are recognised during parsing iex&gt; Cldr.Currency.strings_for_currency :AUD, &quot;de&quot; [&quot;aud&quot;, &quot;au$&quot;, &quot;australischer dollar&quot;, &quot;australische dollar&quot;] iex&gt; Money.parse &quot;$au 12 346&quot;, locale: &quot;fr&quot; #Money&lt;:AUD, 12346&gt; iex&gt; Money.parse &quot;12 346 dollar australien&quot;, locale: &quot;fr&quot; #Money&lt;:AUD, 12346&gt; iex&gt; Money.parse &quot;A$ 12346&quot;, locale: &quot;en&quot; #Money&lt;:AUD, 12346&gt; iex&gt; Money.parse &quot;australian dollar 12346.45&quot;, locale: &quot;en&quot; #Money&lt;:AUD, 12346.45&gt; # Note that the decimal separator in the &quot;de&quot; locale # is a `.` iex&gt; Money.parse &quot;AU$ 12346,45&quot;, locale: &quot;de&quot; #Money&lt;:AUD, 12346.45&gt; # Round trip formatting is supported iex&gt; {:ok, string} = Cldr.Number.to_string 1234, Money.Cldr, currency: :AUD {:ok, &quot;A$1,234.00&quot;} iex&gt; Money.parse string #Money&lt;:AUD, 1234.00&gt; # Fuzzy matching is possible iex&gt; Money.parse(&quot;100 eurosports&quot;, fuzzy: 0.8) #Money&lt;:EUR, 100&gt; iex&gt; Money.parse(&quot;100 eurosports&quot;, fuzzy: 0.9) {:error, {Money.Invalid, &quot;Unable to create money from \\&quot;eurosports\\&quot; and \\&quot;100\\&quot;&quot;}} # Eligible currencies can be filtered iex&gt; Money.parse(&quot;100 eurosports&quot;, fuzzy: 0.8, currency_filter: [:current, :tender]) #Money&lt;:EUR, 100&gt; iex&gt; Money.parse &quot;100 afghan afghanis&quot; #Money&lt;:AFA, 100&gt; iex&gt; Money.parse &quot;100 afghan afghanis&quot;, currency_filter: [:current, :tender] {:error, {Money.Invalid, &quot;Unable to create money from \\&quot;afghan afghanis\\&quot; and \\&quot;100\\&quot;&quot;}} Casting a money type (basic support for HTML forms) Money supports form field inputs that are a single string combining both a currency code and an amount. When a form field (or other data) is cast then Money will attempt to parse a string field into a Money.t using Money.parse/2. Therefore simple money form input can be supported with a single input field of type=text. Note that when parsing the input text, the amount is interpreted in the context of the current locale set on the default backend configured for ex_money. This affects how separator characters are interpreted in exactly the same way as is done for Money.new/3. Float amounts cannot be provided to Money.new/2 Float have well-known issues in computing due to issues of rounding and potential precision loss. Internally Money uses Decimal to store the amount which allows arbitrary precision arithmetic. Money also uses the numeric type in Postgres to preserve precision and even goes to far as to store the amount as a string in MySQL for the same reason. Therefore an error is returned if an attempt is made to use Money.new/2 with a float amount: {:error, {Money.InvalidAmountError, &quot;Float amounts are not supported in new/2 due to potenial rounding &quot; &lt;&gt; &quot;and precision issues. If absolutely required, use Money.from_float/2&quot;}} If the use of floats is require then the function Money.from_float/2 is provided with the same arguments as those for Money.new/2. Money.from_float/2 provides an addition check and will return an error if the precision (number of digits) of the provided float is more than 15 (the number of digits guaranteed to round-trip between a 64-bit float and a string). Optional ~M sigil An optional sigil module is available to aid in creating %Money{} structs. It needs to be imported before use: import Money.Sigil ~M[100]USD #&gt; #Money&lt;:USD, 100&gt; Localised Money formatting Money provides locale-specific formatted output that is controlled be either the locale that has been set for this process or by the :locale parameter supplied to Money.to_string/2. Configuring your localised environment requires configuring ex_cldr which is a dependency to Money. See the Configuration section of the ex_cldr readme for more information. The main API for formatting Money is Money.to_string/2. Additionally formatting options are passed to Cldr.Number.to_string/2. Those options are described in the readme for ex_cldr_numbers which is also a dependency to Money. iex&gt; Money.to_string Money.new(&quot;thb&quot;, 11) {:ok, &quot;THB11.00&quot;} # The default locale is &quot;en-001&quot; which is # &quot;global english&quot; iex&gt; Money.to_string Money.new(&quot;USD&quot;, &quot;234.467&quot;) {:ok, &quot;$US234.47&quot;} # The locale &quot;en&quot; is &quot;American English&quot;. For # UK English use the locale &quot;en-GB&quot;. Australian # English is &quot;en-AU&quot; and so on. iex&gt; Money.to_string Money.new(&quot;USD&quot;, &quot;234.467&quot;), locale: &quot;en&quot; {:ok, &quot;$234.47&quot;} iex&gt; Money.to_string Money.new(&quot;USD&quot;, &quot;234.467&quot;), format: :long {:ok, &quot;234.47 US dollars&quot;} iex&gt; Money.to_string Money.new(&quot;USD&quot;, &quot;234.467&quot;), locale: &quot;fr&quot; {:ok, &quot;234,47 $US&quot;} iex&gt; Money.to_string Money.new(&quot;USD&quot;, &quot;234.467&quot;), locale: &quot;de&quot; {:ok, &quot;234,47 $&quot;} iex&gt; Money.to_string Money.new(&quot;EUR&quot;, &quot;234.467&quot;), locale: &quot;de&quot; {:ok, &quot;234,47 €&quot;} iex&gt; Money.to_string Money.new(&quot;EUR&quot;, &quot;234.467&quot;), locale: &quot;fr&quot; {:ok, &quot;234,47 €&quot;} Note that the output is influenced by the locale in effect. By default the locale used is that returned by Cldr.get_current_local/0. Its default value is &quot;en-001&quot;. Additional locales can be configured, see Cldr. The formatting options are defined in Cldr.Number.to_string/2. Arithmetic Functions See also the module Money.Arithmetic: iex&gt; m1 = Money.new(:USD, 100) #Money&lt;:USD, 100&gt;} iex&gt; m2 = Money.new(:USD, 200) #Money&lt;:USD, 200&gt;} iex&gt; Money.add(m1, m2) {:ok, #Money&lt;:USD, 300&gt;} iex&gt; Money.add!(m1, m2) #Money&lt;:USD, 300&gt; iex&gt; m3 = Money.new(:AUD, 300) #Money&lt;:AUD, 300&gt; iex&gt; Money.add Money.new(:USD, 200), Money.new(:AUD, 100) {:error, {ArgumentError, &quot;Cannot add monies with different currencies. Received :USD and :AUD.&quot;}} # Split a %Money{} returning the a dividend and a remainder. All # operations respect the number of fractional digits defined for a currency iex&gt; m1 = Money.new(:USD, 100) #Money&lt;:USD, 100&gt; iex&gt; Money.split(m1, 3) {#Money&lt;:USD, 33.33&gt;, #Money&lt;:USD, 0.01&gt;} # Rounding applies the currency definitions of CLDR as implemented in # the hex package [ex_cldr](https://hex.pm/packages/ex_cldr) iex&gt; Money.round Money.new(:USD, &quot;100.678&quot;) #Money&lt;:USD, 100.68&gt; iex&gt; Money.round Money.new(:JPY, &quot;100.678&quot;) #Money&lt;:JPY, 101&gt; Currency Conversion A %Money{} struct can be converted to another currency using Money.to_currency/3 or Money.to_currency!/3. For example: iex&gt; Money.to_currency Money.new(:USD, 100), :AUD {:ok, #Money&lt;:AUD, 136.43&gt;} iex&gt; Money.to_currency Money.new(:USD, 100), :AUD, ExchangeRates.historic_rates(~D[2017-01-01]) {:ok, #Money&lt;:AUD, 128.76&gt;} iex&gt; Money.to_currency Money.new(:USD, 100) , :AUDD, %{USD: Decimal.new(1), AUD: Decimal.new(0.7345)} {:error, {Cldr.UnknownCurrencyError, &quot;Currency :AUDD is not known&quot;}} iex&gt; Money.to_currency! Money.new(:USD, 100), :XXX ** (Money.ExchangeRateError) No exchange rate is available for currency :XXX A user-defined map of exchange rates can also be supplied: iex&gt; Money.to_currency Money.new(:USD, 100), :AUD, %{USD: Decimal.new(1.0), AUD: Decimal.new(1.3)} #Money&lt;:AUD, 130&gt; Historic Conversion Rates As noted in the configuration section, ex_money can preload historic exchange rates when the exchange rates service starts up. It can be anticipated that additional historic rates may be required subsequently. Money.ExchangeRates.Retriever.historic_rates/1 can be called to request retrieval of historic rates at any time. This call will send a message to the retrieval service to request retrieval. It does not return the rates. Money.ExchangeRates.historic_rates/1 is the partner function to Money.ExchangeRates.latest_rates/0. It returns the exchange rates for a given date, and will return an error if no rates are available. Financial Functions A set of basic financial functions are available in the module Money.Financial. These functions are: Present value: Money.Financial.present_value/3 Future value: Money.Financial.future_value/3 Interest rate: Money.Financial.interest_rate/3 Number of periods: Money.Financial.periods/3 Payment amount: Money.Financial.payment/3 Net Present Value of a set of cash flows: Money.Financial.net_present_value/2 Internal rate of return: Money.Financial.internal_rate_of_return/1 For more detail see Money.Financial."},{"ref":"readme.html#subscriptions","title":"Introduction to Money - Subscriptions","type":"extras","doc":"Subscriptions, especially in the context of a SaaS, can involve changing plans - either from a smaller plan to a larger or a larger plan to smaller. In either situation a credit amount needs to be calculated based upon the current plan which is then applied to the new plan. Money.Subscription is a module that provides functions to support this subscription pricing, credit calculations and payment dates. The primary functions supporting subscriptions are: Create a new subscription: Money.Subscription.new/3 Create a subscription plan: Money.Subscription.Plan.new/3 Change a from one plan to another: Money.Subscription.change_plan/3 Calculate the start date for the next interval of a plan: Money.Subscription.next_interval_starts/3 Calculate the number of days in a plan interval: Money.Subscription.plan_days/3 Calculate the number of days left in a plan interval: Money.Subscription.days_remaining/4 Examples # Create the current plan iex&gt; current_plan = Money.Subscription.Plan.new!(Money.new(:USD, 10), :month, 1) %Money.Subscription.Plan{ interval: :month, interval_count: 1, price: #Money&lt;:USD, 10&gt; } # How many days in a billing period? iex&gt; Money.Subscription.plan_days current_plan, ~D[2018-03-01] 31 iex&gt; Money.Subscription.plan_days current_plan, ~D[2018-02-01] 28 # How many days remaining in the current billing period iex&gt; Money.Subscription.days_remaining current_plan, ~D[2018-03-01], ~D[2018-03-10] 22 # When is the next billing date iex&gt; Money.Subscription.next_interval_starts current_plan, ~D[2018-03-01] ~D[2018-04-01] # Create a new plan iex&gt; new_plan = Money.Subscription.Plan.new!(Money.new(:USD, 10), :month, 3) %Money.Subscription.Plan{ interval: :month, interval_count: 3, price: #Money&lt;:USD, 10&gt; } # Change plans at the end of the current billing period iex&gt; Money.Subscription.change_plan current_plan, new_plan, current_interval_started: ~D[2018-03-01] %Money.Subscription.Change{ carry_forward: #Money&lt;:USD, 0&gt;, credit_amount: #Money&lt;:USD, 0&gt;, credit_amount_applied: #Money&lt;:USD, 0&gt;, credit_days_applied: 0, credit_period_ends: nil, first_billing_amount: #Money&lt;:USD, 10&gt;, first_interval_starts: ~D[2018-04-01], next_interval_starts: ~D[2018-07-01] } # Change plans in the middle of the current plan period # and credit the balance of the current plan to the new plan iex&gt; Money.Subscription.change_plan current_plan, new_plan, current_interval_started: ~D[2018-03-01], effective: ~D[2018-03-15] %Money.Subscription.Change{ carry_forward: #Money&lt;:USD, 0&gt;, credit_amount: #Money&lt;:USD, 5.49&gt;, credit_amount_applied: #Money&lt;:USD, 5.49&gt;, credit_days_applied: 0, credit_period_ends: nil, first_billing_amount: #Money&lt;:USD, 4.51&gt;, first_interval_starts: ~D[2018-03-15], next_interval_starts: ~D[2018-06-15] } # Change plans in the middle of the current plan period # but instead of a monetary credit, apply the credit as # extra days on the new plan in the first billing period iex&gt; Money.Subscription.change_plan current_plan, new_plan, current_interval_started: ~D[2018-03-01], effective: ~D[2018-03-15], prorate: :period %Money.Subscription.Change{ carry_forward: #Money&lt;:USD, 0&gt;, credit_amount: #Money&lt;:USD, 5.49&gt;, credit_amount_applied: #Money&lt;:USD, 0&gt;, credit_days_applied: 51, credit_period_ends: ~D[2018-05-04], first_billing_amount: #Money&lt;:USD, 10&gt;, first_interval_starts: ~D[2018-03-15], next_interval_starts: ~D[2018-08-05] } # Create a subscription iex&gt; plan = Money.Subscription.Plan.new!(Money.new(:USD, 200), :month, 3) iex&gt; subscription = Money.Subscription.new! plan, ~D[2018-01-01] %Money.Subscription{ created_at: #DateTime&lt;2018-03-23 07:45:44.418916Z&gt;, id: nil, plans: [ {%Money.Subscription.Change{ carry_forward: #Money&lt;:USD, 0&gt;, credit_amount: #Money&lt;:USD, 0&gt;, credit_amount_applied: #Money&lt;:USD, 0&gt;, credit_days_applied: 0, credit_period_ends: nil, first_billing_amount: #Money&lt;:USD, 200&gt;, first_interval_starts: ~D[2018-01-01], next_interval_starts: ~D[2018-04-01] }, %Money.Subscription.Plan{ interval: :month, interval_count: 3, price: #Money&lt;:USD, 200&gt; }} ] } # Change a subscription&#39;s plan iex&gt; new_plan = Money.Subscription.Plan.new!(Money.new(:USD, 150), :day, 30) iex&gt; Money.Subscription.change_plan! subscription, new_plan %Money.Subscription{ created_at: #DateTime&lt;2018-03-23 07:47:48.593973Z&gt;, id: nil, plans: [ {%Money.Subscription.Change{ carry_forward: #Money&lt;:USD, 0&gt;, credit_amount: #Money&lt;:USD, 0&gt;, credit_amount_applied: #Money&lt;:USD, 0&gt;, credit_days_applied: 0, credit_period_ends: nil, first_billing_amount: #Money&lt;:USD, 150&gt;, first_interval_starts: ~D[2018-04-01], next_interval_starts: ~D[2018-05-01] }, %Money.Subscription.Plan{ interval: :day, interval_count: 30, price: #Money&lt;:USD, 150&gt; }}, {%Money.Subscription.Change{ carry_forward: #Money&lt;:USD, 0&gt;, credit_amount: #Money&lt;:USD, 0&gt;, credit_amount_applied: #Money&lt;:USD, 0&gt;, credit_days_applied: 0, credit_period_ends: nil, first_billing_amount: #Money&lt;:USD, 200&gt;, first_interval_starts: ~D[2018-01-01], next_interval_starts: ~D[2018-04-01] }, %Money.Subscription.Plan{ interval: :month, interval_count: 3, price: #Money&lt;:USD, 200&gt; }} ] }"},{"ref":"readme.html#serializing-to-a-database-with-ecto","title":"Introduction to Money - Serializing to a database with Ecto","type":"extras","doc":"The companion package ex_money_sql provides functions for the serialization of Money data. See the README for further information."},{"ref":"readme.html#installation","title":"Introduction to Money - Installation","type":"extras","doc":"Money can be installed by adding ex_money to your list of dependencies in mix.exs and then executing mix deps.get def deps do [ {:ex_money, &quot;~&gt; 4.0&quot;}, ... ] end"},{"ref":"readme.html#why-yet-another-money-package","title":"Introduction to Money - Why yet another Money package?","type":"extras","doc":"Fully localized formatting and rounding using ex_cldr Provides serialization to Postgres using a composite type and MySQL using a JSON type that keeps both the currency code and the amount together removing a source of potential error Uses the Decimal type in Elixir and the Postgres numeric type to preserve precision. For MySQL the amount is serialised as a string to preserve precision that might otherwise be lost if stored as a JSON numeric type (which is either an integer or a float) Includes a set of financial calculations (arithmetic and cash flow calculations) that follow solid rounding rules"},{"ref":"readme.html#falsehoods-programmers-believe-about-prices","title":"Introduction to Money - Falsehoods programmers believe about prices","type":"extras","doc":"The github gist gives a good summary of the challenges of managing money in an application. The following described how Money handles each of these assertions. 1. You can store a price in a floating point variable. Money operates and serialises in a arbitrary precision Decimal value. 2. All currencies are subdivided in 1/100th units (like US dollar/cents, euro/eurocents etc.). Money leverages CLDR which defines the appropriate number of decimal places of a currency. As of CLDR version 32 there are: 52 currencies with zero decimal digits 241 currencies with two decimal digits 6 currencies with three decimal digits and 1 currency with four decimal digits 3. All currencies are subdivided in decimal units (like dinar/fils) 4. All currencies currently in circulation are subdivided in decimal units. (to exclude shillings, pennies) (counter-example: MGA) 5. All currencies are subdivided. (counter-examples: KRW, COP, JPY... Or subdivisions can be deprecated.) Money correctly manages the appropriate number of decimal places for a currency. It also round correctly when formatting a currency for output (different currencies have different rounding levels for cash or transactions). 6. Prices can&#39;t have more precision than the smaller sub-unit of the currency. (e.g. gas prices) All Money calculations are done with decimal arithmetic to the maxium precision of 28 decimal digits. 7. For any currency you can have a price of 1. (ZWL) Money makes no assumption about the value assigned as long as its a real number. 8. Every country has its own currency. (EUR is the best example, but also Franc CFA, etc.) Money makes no assumptions about the linkage of currencies to territories. 9. No country uses another&#39;s country official currency as its official currency. (many countries use USD: Ecuador, Micronesia...) 10. Countries have only one currency. Money doesn&#39;t link territories (countries) to a currency - it focuses only on the Money domain. The addon package cldr_territories does have knowledge of what curriencies are in effect throughout history for a given territory. See Cldr.Territory.info/1. 11. Countries have only one currency currently in circulation. (Panama officially uses both PAB and USD) Money makes no assumptions about currencies in circulation. 12. I&#39;ll only deal with currencies currently in circulation anyway. Money makes no assumptions about currencies in circulation. 13. All currencies have an ISO 4217 3-letter code. (The Transnistrian ruble has none, for example) Money does validate currency codes against the ISO 4217 list. Custom currencies can be created in accordance with ISO 4217 using Cldr.Currency.new/2. 14. All currencies have a different name. (French franc, &quot;nouveau franc&quot;) Money has localised names for all ISO 4217 currencies in each of the over 500 locales defined by CLDR. 15. You always put the currency symbol after the price. Money formats currency strings according to a format mask that is either defined by CLDR or user supplied. 16. You always put the currency symbol before the price. Money formats currency strings according to a format mask that is either defined by CLDR or user supplied. 17. You always put the currency symbol either after, or before the price, never in the middle. Money formats currency strings according to a format mask that is either defined by CLDR or user supplied. 18. There&#39;s only one currency symbol for any currency. (元, 角, 分 are increasing units of the Chinese renminbi.) Money uses format masks defined by CLDR which, for the Chinese renminbi uses the &quot;￥&quot; symbol. 19. For a given currency, you always, but always, put the symbol in the same place. Money makes no assumpions about symbol placement. The symbol can be places anywhere in a formatted string but is typically, for CLDR format masks, placed either before or after the formatted number. 20. OK. But if you only use the ISO 4217 currency codes, you always put it before the price. (Hint: it depends on the language.) Same as for the answer to 19 above. 21. Before the price means on the left. (ILS) Money formats according to a locale and correctly places symbols for languages written right-to-left. 22. You can always use a dot (or a comma, etc.) as a decimal separator. The decimal separator is defined per locale according to the CLDR definitions. 23. You can always use a space (or a dot, or a comma, etc.) as a thousands separator. The thousands (acutally grouping since not all locales format in thousands) separator is defined per locale according to the CLDR definitions. 24. You separate big prices by grouping numbers in triplets (thousands). (One writes ¥1 0000) Grouping is done according the CLDR definitions. For many languages the grouping is in thousands. Some format other ways. For example in India numbers are formatted with the first group as a triplet and subsequent groups as doublets. 25. Prices at a single company will never range from five digits before the decimal to five digits after. Money&#39;s default precision is 28 decimal digits. All arithmetic is done using arbitrary precision decimal arithemetic. No round is performed unless either explicitly requested or a money value is formatted for output. When formatting rounding is applied according the locale-specific rules. 26. Prices contains only digits and punctuation. (Germans can write 12,- €) Money format masks can contain very flexible formatting masks. A set of formats is defined for each locale and a user-defined masks can also be defined. 27. A price can be at most 10^N for some value of N. See the answer to 25. 28. Given two currencies, there is only one exchange rate between them at any given point in time. Money supports an exchange rate mechansim, currency conversions and retrieval from external exchange rate services. It does not impose any constraint on underlying conversion tables. 29. Given two currencies, there is at least one exchange rate between them at any given point in time. (restriction on export of MAD, ARS, CNY, for example) See the answer to 28. 30. And the final one: a standalone $ character is always pronounced dollar. (It&#39;s also the peso sign.) This is outside the domain of Money."},{"ref":"changelog.html","title":"Changelog for Money v4.1.0","type":"extras","doc":"Changelog for Money v4.1.0 This is the changelog for Money v4.1.0 released on July 13th, 2019. For older changelogs please consult the release tag on GitHub Enhancements Adds Money.abs/1. Thanks to @jeremyjh. Improve @doc consistency using ## Arguments not ## Options. Changelog for Money v4.0.0 This is the changelog for Money v4.0.0 released on July 8th, 2019. For older changelogs please consult the release tag on GitHub Breaking Changes Functions related to the serialization of money types have been extracted to the library ex_money_sql. For applications using the dependency ex_money that do not require serialization no changes are required. For applications using serialization, the dependency should be changed to ex_money_sql (which in turn depends on ex_money). Supports Elixir 1.6 and later only Changelog for Money v3.4.4 This is the changelog for Money v3.4.4 released on June 2nd, 2019. For older changelogs please consult the release tag on GitHub Enhancements Supports passing an Cldr.Number.Formation.Options.t as alternative to a Keyword.t for options to Money.to_string/2. Performance is doubled when using pre-validated options which is useful if formatting is being executed in a tight loop. An example of this usage is: iex&gt; money = Money.new(:USD, 100) # Apply any options required as a keyword list # Money will take care of managing the `:currency` option iex&gt; options = [] iex&gt; {:ok, options} = Cldr.Number.Format.Options.validate_options(0, backend, options) iex&gt; Money.to_string(money, options) The 0 in validate_options is used to determine the sign of the amount because that can influence formatting - for example the accounting format often uses (1234) as its format. If you know your amounts are always positive, just use 0. If the use case may have both positive and negative amounts, generate two option sets (one with the positive number and one with the negative). Then use the appropriate option set. For example: iex&gt; money = Money.new(:USD, 1234) # Add options as required # Money will take care of managing the `:currency` option iex&gt; options = [] iex&gt; {:ok, positive_options} = Cldr.Number.Format.Options.validate_options(0, backend, options) iex&gt; {:ok, negative_options} = Cldr.Number.Format.Options.validate_options(-1, backend, options) iex&gt; if Money.cmp(money, Money.zero(:USD)) == :gt do ...&gt; Money.to_string(money, positive_options) ...&gt; else ...&gt; Money.to_string(money, negative_options) ...&gt; end Changelog for Money v3.4.3 This is the changelog for Money v3.4.3 released on June 2nd, 2019. For older changelogs please consult the release tag on GitHub Bug Fixes Ensure Money.to_string!/2 properly raises Add specs for Money.to_string/2 and Money.to_string!/2 Thanks to @rodrigues for the report and PR. Changelog for Money v3.4.2 This is the changelog for Money v3.4.2 released on April 16th, 2019. For older changelogs please consult the release tag on GitHub Bug Fixes Money.put_fraction/2 now correctly allows setting the fraction to 0. Enhancements Money.round/2 allows setting :currency_digits to an integer number of digits in addition to the options :iso, :cash and :accounting. The default remains :iso. Improves the documentation for Money.to_string/2. Changelog for Money v3.4.1 This is the changelog for Money v3.4.1 released on April 5th, 2019. For older changelogs please consult the release tag on GitHub Bug Fixes Fix README.md markdown formatting error. Thanks to @fireproofsocks for the report and @lostkobrakai for the fix. Closes #99. Changelog for Money v3.4.0 This is the changelog for Money v3.4.0 released on March 28th, 2019. For older changelogs please consult the release tag on GitHub Enhancements Updates to CLDR version 35.0.0 released on March 27th 2019 through ex_cldr version 2.6.0. Changelog for Money v3.3.1 This is the changelog for Money v3.3.1 released on March 8th, 2019. For older changelogs please consult the release tag on GitHub Bug Fixes Fix or silence dialyzer warnings Changelog for Money v3.3.0 This is the changelog for Money v3.3.0 released on February 24th, 2019. For older changelogs please consult the release tag on GitHub Enhancements Adds Money.put_fraction/2. This will set the fractional part of a money to the specified integer amount. Examples: iex&gt; Money.put_fraction Money.new(:USD, &quot;2.49&quot;), 99 #Money&lt;:USD, 2.99&gt; iex&gt; Money.put_fraction Money.new(:USD, &quot;2.49&quot;), 999 {:error, {Money.InvalidAmountError, &quot;Rounding up to 999 is invalid for currency :USD&quot;}} Bug Fixes Parsing money strings now uses a more complete set of character definitions for decimal and grouping separators based upon the characters.json file of the &quot;en&quot; locale. Changelog for Money v3.2.4 This is the changelog for Money v3.2.4 released on February 13th, 2019. For older changelogs please consult the release tag on GitHub Bug Fixes Updates to ex_cldr_currencies version 2.1.2 which correctly removes duplicate currency strings when the same string referred to different currency codes. See the changelog for further detail. Enhancements Adds a :fuzzy option to Money.parse/2 that uses String.jaro_distance/2 to help determine if the provided currency text can be resolved as a currency code. For example: iex&gt; Money.parse(&quot;100 eurosports&quot;, fuzzy: 0.8) #Money&lt;:EUR, 100&gt; iex&gt; Money.parse(&quot;100 eurosports&quot;, fuzzy: 0.9) {:error, {Money.Invalid, &quot;Unable to create money from \\\\&quot;eurosports\\\\&quot; and \\\\&quot;100\\\\&quot;&quot;}} Changelog for Money v3.2.3 This is the changelog for Money v3.2.3 released on February 12th, 2019. For older changelogs please consult the release tag on GitHub Bug Fixes Correctly parse money strings with unicode currency symbols like &quot;€&quot;. Closes #95. Thanks to @crbelaus. Changelog for Money v3.2.2 This is the changelog for Money v3.2.2 released on February 10th, 2019. For older changelogs please consult the release tag on GitHub Enhancements Improves parsing of money strings. Parsing now uses various strings that CLDR knows about. Some examples: iex&gt; Money.parse &quot;$au 12 346&quot;, locale: &quot;fr&quot; #Money&lt;:AUD, 12346&gt; iex&gt; Money.parse &quot;12 346 dollar australien&quot;, locale: &quot;fr&quot; #Money&lt;:AUD, 12346&gt; iex&gt; Money.parse &quot;A$ 12346&quot;, locale: &quot;en&quot; #Money&lt;:AUD, 12346&gt; iex&gt; Money.parse &quot;australian dollar 12346.45&quot;, locale: &quot;en&quot; #Money&lt;:AUD, 12346.45&gt; iex&gt; Money.parse &quot;AU$ 12346,45&quot;, locale: &quot;de&quot; #Money&lt;:AUD, 12346.45&gt; # Can also return the strings available for a given currency # and locale iex&gt; Cldr.Currency.strings_for_currency :AUD, &quot;de&quot; [&quot;aud&quot;, &quot;au$&quot;, &quot;australischer dollar&quot;, &quot;australische dollar&quot;] # Round trip formatting also seems to be ok iex&gt; {:ok, string} = Cldr.Number.to_string 1234, Money.Cldr, currency: :AUD {:ok, &quot;A$1,234.00&quot;} iex&gt; Money.parse string #Money&lt;:AUD, 1234.00&gt; Changelog for Money v3.2.1 This is the changelog for Money v3.2.1 released on February 2nd, 2019. For older changelogs please consult the release tag on GitHub Bug Fixes Added Money.Ecto.Composite.Type.cast/1 and Money.Ecto.Map.Type.cast/1 for a String.t parameter. When a String.t is provided, cast/1 will call Money.parse/2 to create the Money.t. Money.new/3 now uses the current locale on the default backend if no locale or backend is specified. This means that Money.Ecto.Composite.Type.cast/1 and Money.Ecto.Map.Type.cast/1 will be parsed using the locale that has been set for the current process in the default backend. As a result, a simple type=text form field can be used to input a money type (currency code and amount in a single string) that can then be cast to a Money.t. Changelog for Money v3.2.0 This is the changelog for Money v3.2.0 released on February 1st, 2019. For older changelogs please consult the release tag on GitHub Bug Fixes Correctly generate migrations_path/1 function based upon whether Ecto is configured and which version Enhancements Adds Money.parse/2 which will parse a string comprising a currency code and an amount. It will return a Money.t or an error. This function may be helpful in supporting money input in HTML forms. Changelog for Money v3.1.0 This is the changelog for Money v3.1.0 released on December 30th, 2018. For older changelogs please consult the release tag on GitHub Bug Fixes Fix typo in exchange_rates_retriever.ex. Thanks to @lostkobrakai. Closes #91. Remove obsolete cldr compiler Changes the sum aggregate function for money_with_currency to be STRICT which means it handles NULL columns in the same way as the standard SUM function. Thanks to @lostkobrakai. Closes #88. Fixes documentation link errors Fix unhandled terminate typo error in exchange rates server. Thanks to @xavier. Closes #90. Changelog for Money v3.0.0 This is the changelog for Money v3.0.0 released on November 23rd, 2018. For older changelogs please consult the release tag on GitHub The primary purpose of this release is to support ex_cldr version 2.0 Breaking changes Money.from_tuple/1 has been removed Uses ex_cldr version 2. Please see the changelog for configuration changes that are required. Requires a default_cldr_backend to be configured in config.exs. For example: config :ex_money, ... default_cldr_backend: MyApp.Cldr end"},{"ref":"license.html","title":"LICENSE","type":"extras","doc":""},{"ref":"license.html#license","title":"LICENSE - License","type":"extras","doc":"Copyright 2017-2019 Kip Cole Licensed under the Apache License, Version 2.0 (the &quot;License&quot;); you may not use this file except in compliance with the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License."}]